function [Mx, My, Mz, Mt, Flip, Phase] = BlochSimulation( NumPixel, NumChannel, t_vec, G_mat, Idx, Coords, nxyz, B0_vec, B1_mat, RF, InterpolationDepth )

    % This method performs a Bloch simulation

    %error handling
    if size( Coords, 2 ) ~= 3
        error('Coords matrix must have three columns!') 
    end
    
    if size( Idx, 2 ) ~= 3
        error('Idx matrix must have three columns!') 
    end
    
    if size( Coords, 1 ) ~= size( B1_mat, 1 ) || size( Coords, 1 ) ~= size( B0_vec, 1 ) || size( Coords, 1 ) ~= size( Idx, 1 )
    	error('Coords, Idx, B1 and B0 matrices must have same number of rows (Coords = %i, Idx = %i, B1 = %i, B0 = %i)!',size( Coords, 1 ),size( Idx, 1 ),size( B1_mat, 1 ),size( B0_vec, 1 )) 
    end
    
    if size( G_mat, 1 ) ~= size( t_vec, 1 )
    	error('G(t) and t vectors must have same number of rows (G(t) = %i, t = %i)!',size( G_mat, 1 ),size( t_vec, 1 )) 
    end
    
    if size(RF, 1) / NumChannel ~= numel(t_vec)
            error('RF matrix, NumChannel and t vector are inconsistent (numel(RF)/NumChannel == numel(t))')
    end
    
    % initial magnetization state
    Mt_0    = 0.0; 
    Mz_0  	= 1.0;  

    Gamma 	= 42.5774*1e+6;

    %initialize magnetization matrices
    Mx_f  = zeros( nxyz );
    My_f  = zeros( nxyz );
    Mz_f  = zeros( nxyz );

    %store N q (2x2) matrices in Nx4 matrix
    q       = complex( zeros( NumPixel, 4), zeros( NumPixel, 4) ); 
    q(:,1)  = complex( ones( NumPixel, 1), zeros( NumPixel, 1) );
    q(:,4)  = complex( ones( NumPixel, 1), zeros( NumPixel, 1) );

    %set some additional variables
    NumSamples  = size(RF, 1) / NumChannel;

    %prepare sampled data according to Bloch simulation accuracy
    %use higher sampling in Bloch simulation
    delta_t_orig    = t_vec(2) - t_vec(1);
    delta_t         = delta_t_orig / InterpolationDepth;
    t_sam           = 0 : delta_t : t_vec(end);
    NumBlochSamples = numel(t_sam);

    %calculate sampled gradients, interpolation is done in a nearest manner
    G_sam 	= [ interp1( t_vec + delta_t_orig/2, G_mat(:,1), t_sam, 'nearest' )', ...
                interp1( t_vec + delta_t_orig/2, G_mat(:,2), t_sam, 'nearest' )', ...
                interp1( t_vec + delta_t_orig/2, G_mat(:,3), t_sam, 'nearest' )' ];

    %RF is assigned between two subsequent k-space locations, therefore
    %interpolate with nearest option and shifted time points
    RF_mat	= reshape( RF, NumSamples, NumChannel );
    RF_sam 	= ones( numel(t_sam), NumChannel ) + 1j * ones( numel(t_sam), NumChannel );
    for p = 1 : NumChannel
        RF_sam(:,p) = interp1( t_vec + delta_t_orig/2, RF_mat(:,p), t_sam, 'nearest' ).';
    end
    RF_sam( isnan( RF_sam ) ) = 0.0;

    %perform actual simulation
    for time = 1 : NumBlochSamples - 1

        % Cayley-Klein parameters
        b1s 	= B1_mat * RF_sam(time,:).';  
        grad    = Coords * G_sam(time,:)' + B0_vec/Gamma;
        norm    = sqrt( abs(b1s).^2 + grad.^2 );
        phi     = -2 * pi * Gamma * delta_t * norm;

        if sum(abs(phi)) <= 0.0
            continue;
        end

        n       = [real(b1s), imag(b1s), grad];
        n       = n./repmat(norm,1,3);
        a       = cos(phi/2.0) - 1j * (n(:,3)) .* sin(phi/2.0);
        b       = (-1j * n(:,1) + n(:,2)) .* sin(phi/2.0);    
        ind2    = find( norm == 0 | isnan(norm));
        a(ind2) = 1.0;
        b(ind2) = 0.0;

        % update rotation matrix
        q1 = q(:,1); 
        q2 = q(:,2); 
        q3 = q(:,3); 
        q4 = q(:,4);

        %perform multiplication
        q(:,1) = a .* q1 - conj(b) .* q3;
        q(:,2) = a .* q2 - conj(b) .* q4;
        q(:,3) = b .* q1 + conj(a) .* q3;
        q(:,4) = b .* q2 + conj(a) .* q4;

    end

    % obtain final magnetization
    a   = q(:,1); 
    b   = q(:,3);
    Mt  = (conj(a) .* conj(a)) * Mt_0 - ( b .* b ) .* conj(Mt_0) + ( 2.0 * conj(a) .* b) .* Mz_0;
    Mz  = (-conj(a) .* conj(b)) * Mt_0 - ( a .* b ) .* conj(Mt_0) + ( a .* conj(a) - b .* conj(b))  * Mz_0;

    Ind         = sub2ind( nxyz, Idx(:,1), Idx(:,2), Idx(:,3) );
    Mx_f(Ind)   = real(Mt);
    My_f(Ind)   = imag(Mt);
    Mz_f(Ind)   = real(Mz);

    %display central slices
    Mx      = reshape( Mx_f, nxyz );
    My      = reshape( My_f, nxyz );
    Mz      = reshape( Mz_f, nxyz );

    Mt      = Mx + 1j * My;
    Flip  	= real( asin(abs(Mt))/pi*180.0 );
    Phase 	= real( angle(Mt) );

end
