function [RF_final, Err_final] = DesignPulse_LeastSquares( options, A, targetmagn_mat, RF_init )

    % This method solves the RF design problem based on a constraint least
    % squares formalism. This method is slower than the conjugate gradient
    % solver in DesignPulse_ConjugateGradient.m but designs more powerful
    % RF pulses and always respects the RF peak power constraint

    ncunk   = size( A, 2 ); %--> NumChannels * NumberKtPoints
    if nargin < 4
        RF_init  	= zeros( ncunk, 1);
    else
        if isempty(RF_init)
            RF_init  	= zeros( ncunk, 1);
        end
    end

    %error handling
    [Arows, Acols]      = size( A );
    [RFrows, RFcols]    = size( RF_init );
    
    if Acols ~= RFrows || RFcols ~= 1 || Arows ~= numel( targetmagn_mat )
       error('Input dimension mismatch, A = %i x %i, RF = %i x %i, Ind = %i x %i\n',size(A,1), size(A,2), size(RF_init,1), size(RF_init,2), size(Ind,1), size(Ind,2)) 
    end

    %divide RF into real and imaginary parts
    RF_init    = [real( RF_init ); imag( RF_init )];

    %precompute A' * A, store it as global variable, this might be done
    %outside as well
    global AtransA
    AtransA = A' * A;

    fmincon_options = optimset(	'Largescale','off', ...
                        'MaxFunEvals',10^5,...
                        'TolFun',1e-6,...
                        'TolCon',1e-6,...
                        'TolX',1e-6,...
                        'MaxIter',20,...
                        'Algorithm','interior-point',...
                        'Hessian','user-supplied',...
                        'HessFcn',@(RF, lambda) dualhess(RF, lambda),...
                        'GradObj','on',...
                        'GradConstr','on',...
                        'SubproblemAlgorithm','cg',...
                        ...'SubproblemAlgorithm','ldl-factorization',...
                        ...'Display','iter-detailed'...
                        'Display','none'...
                        );
    
    [RF, Err_final] 	= fmincon(@(RF) f0(RF, A, targetmagn_mat), RF_init, [], [], [], [], [], [], @(RF) fn(RF, options.Vmax), fmincon_options);
    RF_final            = RF(1 : ncunk) + 1j * RF(ncunk + 1 : 2 * ncunk);
    

end

function H = dualhess( RF, lambda )

    global AtransA
    ncunk  	= size(RF,1)/2;

    % Hessian of the main function
    ch  = 2.0 * AtransA;
    H   = [ real(ch), -imag(ch); ...
            imag(ch),  real(ch)];

    lam = lambda.ineqnonlin(1:ncunk);    
    
    % Bastien -- faster when lots of variables
    % H       = H + 2.0 * diag([ lam; lam ]);
    N               = 2*ncunk;
    H(1:N+1:end)	= H(1:N+1:end) + 2.0 * repmat(lam.',[1 2]);
   
end

function [f, df] = f0( RF, A, Mt_target)

    % Minimization functional to be handed to fmincon. Accepts RF vector 
    %(first half represents magnitude, second half phase) and returns error 
    %of currently achieved magnetization to desired magnetization

    %get complex RF vector
    ncunk       = size(RF,1)/2;
	RF_comp  	= RF(1:ncunk) + 1j*RF(ncunk+1:2*ncunk);

    % function: multiply with system matrix, calculate L2 norm
    y   = A * RF_comp - Mt_target;
    f   = real(y'*y);

    if nargout>1  % Bastien: only if needed!!!
        % gradient: analytic gradient of error in L2 norm
        cdf                     = 2.0 * A' * y;
        df                      = zeros(2*ncunk,1);
        df(1:ncunk,1)           = real(cdf);
        df(ncunk+1:2*ncunk,1)   = imag(cdf);
    end

end

function [c, ceq, dc, dceq] = fn( RF, Vmax )

    ncunk   = size(RF,1)/2;
    RF_comp	= RF(1:ncunk).' + 1j*RF(ncunk+1:2*ncunk).';

    % no equality constraints
    ceq     = []; 
    dceq    = [];

    % inequality constraints (max. RF voltage)
    c   = abs(RF_comp.').^2 - Vmax^2;
    if nargout>2
        dc  = [ spdiags(2 * RF(1:ncunk),0,ncunk,ncunk); spdiags(2 * RF(ncunk+1:2*ncunk),0,ncunk,ncunk) ];
    end

end






