function [ Gx_delayed, Gy_delayed, Gz_delayed, RF_delayed, t_delayed ] = GetRFGradientDelay( RF_mat, T, options )

    % This method introduces a delay between RF and gradient waveforms to
    % compensate for hardware imperfections.

    if ~isfield( options, 'RF_delay' )
        warning('Missing field ''RF_delay'' in options, no delay is used! (RF_delay = 0)')
        RF_delay = 0.0;
    else
        RF_delay	= options.RF_delay;
    end
    
    dt          = T.Pulse.t(2) - T.Pulse.t(1);  %extract sampling time
    NumChannel  = size( RF_mat, 1 );

    if RF_delay > 0.0
        %RF is delayed to gradient waveform
        NumDelaySamples = ceil( abs(RF_delay)/dt );
        RF_delayed      = [ zeros( NumChannel, NumDelaySamples ), RF_mat ];

        %instead the gradient waveforms are interpolated such that the
        %delay is established accurately
        preGradDelay    = NumDelaySamples * dt - RF_delay;   %add residual at the beginning
        postGradDelay   = RF_delay;                          %make sure that gradients have same length as RF_delayed

        %some command output
        fprintf('\nRF Delay (Analytic Gradients):\n')
        fprintf('------------------------------\n')
        fprintf('           Nominal Sampling: %4.1f us\n',dt * 1e+6)
        fprintf('                   RF Delay: %4.1f us\n',options.RF_delay * 1e+6)
        fprintf('   -->       RF Delay START: %4.1f us (%i full samples)\n',NumDelaySamples * dt * 1e+6, NumDelaySamples)
        fprintf('   -->         RF Delay END: %4.1f us (%i full samples)\n',0,0)
        fprintf('   --> Gradient Delay START: %4.1f us\n',preGradDelay * 1e+6)
        fprintf('   -->   Gradient Delay END: %4.1f us\n',postGradDelay * 1e+6)
    else
        %Gradient waveform is delayed to RF
        NumDelaySamples = ceil( abs(RF_delay)/dt );
        RF_delayed      = [ RF_mat, zeros( NumChannel, NumDelaySamples ) ];

        %instead the gradient waveforms are interpolated such that the
        %delay is established accurately
        preGradDelay    = abs(RF_delay);                        	%add full anticipated delay at the beginning
        postGradDelay   = NumDelaySamples * dt - abs(RF_delay);  %add residual at the end

        %some command output
        fprintf('\nRF Delay (Analytic Gradients):\n')
        fprintf('-------------------------\n')
        fprintf('           Nominal Sampling: %4.1f us\n',dt * 1e+6)
        fprintf('                   RF Delay: %4.1f us\n',options.RF_delay * 1e+6)
        fprintf('   -->       RF Delay START: %4.1f us (%i full samples)\n',0,0)
        fprintf('   -->         RF Delay END: %4.1f us (%i full samples)\n',NumDelaySamples * dt * 1e+6, NumDelaySamples)
        fprintf('   --> Gradient Delay START: %4.1f us\n',preGradDelay * 1e+6)
        fprintf('   -->   Gradient Delay END: %4.1f us\n',postGradDelay * 1e+6)
    end 
    
    T.Pulse         = [];   %remove old pulse sampling data
    T.Ana.tx        = [ 0.0, T.Ana.tx + preGradDelay * 1e+3, T.Ana.tx(end) + (preGradDelay + postGradDelay) * 1e+3 ];   %ms
    T.Ana.ty        = [ 0.0, T.Ana.ty + preGradDelay * 1e+3, T.Ana.ty(end) + (preGradDelay + postGradDelay) * 1e+3 ];   %ms
    T.Ana.tz        = [ 0.0, T.Ana.tz + preGradDelay * 1e+3, T.Ana.tz(end) + (preGradDelay + postGradDelay) * 1e+3 ];   %ms
    T.t             = [ 0.0, T.t + preGradDelay * 1e+3, T.t(end) + (preGradDelay + postGradDelay) * 1e+3  ];  %ms
    T.Ana.Gx        = [ 0.0, T.Ana.Gx, 0.0 ];
    T.Ana.Gy        = [ 0.0, T.Ana.Gy, 0.0 ];
    T.Ana.Gz        = [ 0.0, T.Ana.Gz, 0.0 ];
    T               = CalculateRFTrajectory( T, options );

    %extract delayed gradients
    Gx_delayed      = T.Pulse.Gx;
    Gy_delayed      = T.Pulse.Gy;
    Gz_delayed      = T.Pulse.Gz;
    t_delayed       = T.Pulse.t;
    
    %make sure everything's correct
    if ~all( [numel(Gx_delayed), numel(Gy_delayed), numel(Gz_delayed), size(RF_delayed,2)] == numel(t_delayed))
        error('Dimension mismatch in delayed gradients and RF!')
    end
    
    return %remove this line to plot the delayed RF and gradients
    
    figure
    ha = tight_subplot(4,1);
    axes(ha(1))
    
    hold on, grid on, box on
    xlim([0,T.Pulse.t(end)])
    ylabel('Gradient: Original','Fontweight','bold')
    stairs( T.Pulse.t, T.Pulse.Gx, 'Color', [0.6,0.6,0.6] )
    stairs( T.Pulse.t, T.Pulse.Gy, 'Color', [0.6,0.6,0.6] )
    stairs( T.Pulse.t, T.Pulse.Gz, 'Color', [0.6,0.6,0.6] )

    plot( T.Ana.tx * 1e-3, T.Ana.Gx * 1e-3, 'Color','red' )
    plot( T.Ana.ty * 1e-3, T.Ana.Gy * 1e-3, 'Color','blue' )
    plot( T.Ana.tz * 1e-3, T.Ana.Gz * 1e-3, 'Color',[0.13, 0.54, 0.13] )
    
    axes(ha(3))
    hold on, grid off, box on
    xlim([0,T.Pulse.t(end)])
    ylabel('Chn1 RF: Original','Fontweight','bold')
    stairs( T.Pulse.t, abs(RF_mat(1,:)).', 'red' )
    stairs( T.Pulse.t, angle(RF_mat(1,:)).', 'blue' )
    
    axes(ha(2))
    hold on, grid on, box on
    xlim([0,t_delayed(end)])
    ylabel('Gradient: Delayed','Fontweight','bold')
    stairs( t_delayed, Gx_delayed, 'Color', [0.6,0.6,0.6] )
    stairs( t_delayed, Gy_delayed, 'Color', [0.6,0.6,0.6] )
    stairs( t_delayed, Gz_delayed, 'Color', [0.6,0.6,0.6] )

    plot( T.Ana.tx * 1e-3, T.Ana.Gx * 1e-3, 'Color','red' )
    plot( T.Ana.ty * 1e-3, T.Ana.Gy * 1e-3, 'Color','blue' )
    plot( T.Ana.tz * 1e-3, T.Ana.Gz * 1e-3, 'Color',[0.13, 0.54, 0.13] )
    
    axes(ha(4))
    hold on, grid off, box on
    xlim([0,t_delayed(end)])
    ylabel('Chn1 RF: Delayed','Fontweight','bold')
    stairs( t_delayed, abs(RF_delayed(1,:)).', 'red' )
    stairs( t_delayed, angle(RF_delayed(1,:)).', 'blue' )
    
    linkaxes([ha(1),ha(2),ha(3),ha(4)],'x')
    linkaxes([ha(1),ha(2)],'xy')
    set(gcf,'units','normalized','outerposition',[0 0 1 1])
    
end