function [f, J, H] = EvalObjective2D( x, AuxData )    

    %#codegen

    %{ 
    Method:
    -------
    Evaluates the objective function (equivalent to the pulse duration)
    based on the vector x to be optimized (x constains the variable 
    gradient magnitudes to be optimized) and the AuxData struct. AuxData
    contains information about the constraint on the control points,
    several options such as the smoothing behaviour, etc. In the
    method the gradients are solved within two subsequent control points
    based on the current gradient values at the control points. In addition
    the Jacobian and the Hessian are stored and formated accordingly to the
    constraints
    
    Input:
    ------
    
    x       - vector of gradient magnitudes
    AuxData - struct (cf. SetUpAuxData2D/3D)

    Output:
    -------
    
    f   - objective function value at x
    J   - Jacobian at x
    H   - Hessian at x
    
    %}
    
    NCP = size(AuxData.R,1);
    f   = 0.0;
    
    %back out full gradient vector
	G_full  = AuxData.D' * x;

    if nargout == 1
        %******************************************************************
        %           C O M P U T E   O B J E C T I V E   O N L Y 
        %******************************************************************
        %fprintf('Evaluating (%i): Objective\n',nargout)
        
        for k = 1 : NCP-1
        
            %evaluate objective function using the previously defined gradient
            %vectors, depending on the number of variables to be returned, the 
            % associated method is used
            fk = FastTimeOptGrad_2D_deriv0(  G_full( 2*k-1 ), G_full( 2*k+1 ), G_full( 2*k ) , G_full( 2*k+2 ), ...
                                           	AuxData.dkx(k), AuxData.dky(k), ...
                                          	AuxData.Gmax, AuxData.Smax, AuxData.OptsVector );
                                        
            f	= f + fk;
        end
 
    elseif nargout == 2
     	%******************************************************************
        %   C O M P U T E   O B J E C T I V E   A N D   J A C O B I A N
        %******************************************************************  
        %fprintf('Evaluating (%i): Objective, Jacobian\n',nargout)
        
        Jt   = zeros( 2*NCP, 1 );

        for k = 1 : NCP-1

            %evaluate objective function using the previously defined gradient
            %vectors, depending on the number of variables to be returned, the 
            % associated method is used
            [fk, gradIn, gradOut] = FastTimeOptGrad_2D_deriv1(  G_full( 2*k-1 ), G_full( 2*k+1 ), G_full( 2*k ) , G_full( 2*k+2 ), ...
                                            AuxData.dkx(k), AuxData.dky(k), ...
                                            AuxData.Gmax, AuxData.Smax, AuxData.OptsVector );
                                                          
            %set Jacobian
            Jt(2*(k-1)+1)  = Jt(2*(k-1)+1) + gradIn(1);   Jt(2*k+1)  = Jt(2*k+1) + gradOut(1);     
            Jt(2*(k-1)+2)  = Jt(2*(k-1)+2) + gradIn(2);   Jt(2*k+2)  = Jt(2*k+2) + gradOut(2);   
            f	= f + fk;
        end

        J 	= AuxData.D * Jt;
        
    else
        %******************************************************************
        %     O B J E C T I V E,    J A C O B I A N,    H E S S I A N
        %******************************************************************
        %fprintf('Evaluating (%i): Objective, Jacobian\n',nargout)
        
        Jt   = zeros( 2*NCP, 1 );
        Ht   = zeros( 2*NCP, 2*NCP );

        for k = 1 : NCP-1

            %evaluate objective function using the previously defined gradient
            %vectors, depending on the number of variables to be returned, the 
            % associated method is used
            [fk, gradIn, gradOut, gradIn_In, gradIn_Out, gradOut_In, gradOut_Out ] = FastTimeOptGrad_2D_deriv2(  G_full( 2*k-1 ), G_full( 2*k+1 ), G_full( 2*k ) , G_full( 2*k+2 ), ...
                                            AuxData.dkx(k), AuxData.dky(k), ...
                                            AuxData.Gmax, AuxData.Smax, AuxData.OptsVector );

            %set objective function value
            f	= f + fk;                                               

            %set Jacobian
            Jt(2*(k-1)+1)  = Jt(2*(k-1)+1) + gradIn(1);   Jt(2*k+1)  = Jt(2*k+1) + gradOut(1);     
            Jt(2*(k-1)+2)  = Jt(2*(k-1)+2) + gradIn(2);   Jt(2*k+2)  = Jt(2*k+2) + gradOut(2);  
            
            %set Hessian
            Ht( 2*k-1, 2*k+1 ) = Ht( 2*k-1, 2*k+1 ) + gradIn_Out(1);
            Ht( 2*k  , 2*k+2 ) = Ht( 2*k  , 2*k+2 ) + gradIn_Out(2);

            Ht( 2*k-1, 2*k-1 ) = Ht( 2*k-1, 2*k-1 ) + gradIn_In(1);
            Ht( 2*k  , 2*k   ) = Ht( 2*k  , 2*k   ) + gradIn_In(2);

            Ht( 2*k+1, 2*k+1 ) = Ht( 2*k+1, 2*k+1 ) + gradOut_Out(1);
            Ht( 2*k+2, 2*k+2 ) = Ht( 2*k+2, 2*k+2 ) + gradOut_Out(2);

            Ht( 2*k+1, 2*k-1 ) = Ht( 2*k+1, 2*k-1 ) + gradOut_In(1);
            Ht( 2*k+2, 2*k   ) = Ht( 2*k+2, 2*k   ) + gradOut_In(2);
            
            %Ht(2*(k-1)+1 : 2*(k-1)+2,1) = [gradOut_In(1); gradOut_In(2)];
        	%Ht(2*(k-1)+1 : 2*(k-1)+4,2) = Ht(2*(k-1)+1:2*(k-1)+4,2) + [gradIn_In(1); gradIn_In(2); gradOut_Out(1); gradOut_Out(2)];
         	%Ht(2*(k-1)+1 : 2*(k-1)+2,3) = [gradIn_Out(1); gradIn_Out(2)];
     
        end

        H  = AuxData.Hs * Ht * AuxData.D';
        J  = AuxData.D * Jt;
        
    end
    
    
    

