function T = PerformOptimization3D( kx_vec, ky_vec, kz_vec, R_mat, Gmax, Smax, G0, Options, doPlot )

    %check if Options is correctly defined
    ValidSolverID = {'InterPoint_Jacobian','InterPoint_Hessian','TrustRegion',...
                     'MinConF_SPG','MinConF_BBST','MinConF_OPG','MinConF_PQN','MinConF_QNST',...
                     'ModifiedShor'};
    
    if nargin < 8
        Options     = []; %use default options
        verbosity   = 0;
        SolverID    = 'InterPoint_Jacobian';
        
        PrintDefaultParameter( 'SolverID', SolverID )
    else
        %check if Options are correctly defined
        if ~isfield( Options, 'SolverID' )
            SolverID = 'InterPoint_Jacobian';
            PrintDefaultParameter( 'SolverID', SolverID )
        else
            SolverID = Options.SolverID;
            if all( ~cellfun( @(x) strcmpi( x, SolverID ), ValidSolverID ) )
                error(['Undefined SolverID, only the following strings are allowed: ',sprintf('%s, ',ValidSolverID{1:end-1}),sprintf('%s',ValidSolverID{end})])
            end
        end
        
        if ~isfield( Options, 'verbosity' )
            verbosity = 0;
        else
            verbosity = Options.verbosity;
        end
        
        %other option entries will be handled in SetUpAuxData
    end
    
    if nargin < 9
       doPlot = 0; 
    end
                 
    %tuning of optimization
    TolFun 	= 1e-8; %Default 1e-6
    TolX  	= 1e-8; %Default 1e-6
    TolCon  = 1e-8; %Default 1e-6

    %some general properties
    MaxIter     = 1000;
    MaxIterShor = 250;

    %% some format checking and correcting
    %check correct format
    if size(kx_vec,1) == 1
        kx_vec          = kx_vec';
    end
    if size(ky_vec,1) == 1
        ky_vec          = ky_vec';
    end
    if size(kz_vec,1) == 1
        kz_vec          = kz_vec';
    end

    %manually define the Gx_opt and Gy_opt value vectors to be able to start
    %the optimization at any point (to ensure correct start values)   

    %% preparation

    NV          = sum( R_mat(:,end) ); % number of variables
    NCP         = numel(kx_vec);       % number of control points 
    dkx         = kx_vec(2:end) - kx_vec(1:end-1);
    dky         = ky_vec(2:end) - ky_vec(1:end-1);
    dkz         = kz_vec(2:end) - kz_vec(1:end-1);
    
    if nargin < 7
        G0 = zeros(NV,1);
    else
        if any( size( G0 ) ~= [NV,1] )
            if ~isempty( G0 )
                error('G0 has wrong dimension: size(G0) = [%i,%i], expected [%i,%i]',size( G0,1 ),size( G0,2 ),NV,1) 
            else
                G0 = zeros(NV,1);
            end
        end
    end

    %Get Void Trajectory
    T           = GetVoidTrajectory( NCP, Gmax, Smax );
    T.kx        = kx_vec;
    T.ky        = ky_vec;
    T.kz        = kz_vec;
    T.R         = R_mat;

    %% optimization
    %set up AuxData
    AuxData = SetUpAuxData3D(   R_mat, ...  	%slope constraint matrix
                                dkx, ...    	%k-space delta kx
                                dky, ...    	%k-space delta ky
                                dkz, ...    	%k-space delta kz
                                Gmax,Smax, ... 	%hardware constraints
                                Options, ...  	%functional options
                                verbosity ) ; 

    %define bounds
    LowerBounds     = -(Gmax - TolCon) * ones(NV,1);
    UpperBounds     = +(Gmax - TolCon) * ones(NV,1);

    %perform optimization
    switch SolverID
        case 'InterPoint_Jacobian'
            % ------------------------------------------------------------
            % ------------------------------------------------------------
            % ------------------------------------------------------------
            SolverOptions = optimset(  ...
                'TolFun', TolFun, ...
                'TolCon',TolCon, ...
                'TolX',TolX,...
                'DerivativeCheck','off',...
                'SubproblemAlgorithm','cg',...
                'GradObj','on',...
                'GradConstr','on',...
                'Hessian','lbfgs',...
                'FunValCheck','on',...
                'TolProjCG',1e-4,...
                'Algorithm','interior-point', ...
                'MaxFunEvals',inf, ...
                'MaxIter', MaxIter );
            
            if verbosity >= 2
              	SolverOptions.Display = 'iter-detailed';
            else
                SolverOptions.Display = 'none';
            end
            
            tic
            if AuxData.Options.UseMex
                %use mex compiled objective functions
                [G_opt,tmin,exitflag,output,lambda,grad,hessian] ...
                            = fmincon( @(x) EvalObjective3D_mex( x, AuxData ), G0 ,[],[],[],[],LowerBounds,UpperBounds,[],SolverOptions); 
            else
                %use regular (m-file) objective function
                [G_opt,tmin,exitflag,output,lambda,grad,hessian] ...
                            = fmincon( @(x) EvalObjective3D( x, AuxData ), G0 ,[],[],[],[],LowerBounds,UpperBounds,[],SolverOptions); 
            end

            %consistent entries thoughout all solvers
            T.OptimOut.runtime        = toc;
            T.OptimOut.SolverID       = SolverID;
            T.OptimOut.tmin           = tmin;
            T.OptimOut.exitflag       = exitflag;
            T.OptimOut.exitreason     = ParseFlag( exitflag, SolverID );
            T.OptimOut.iterations     = output.iterations;
            
            T.OptimOut.output         = output;
            T.OptimOut.lambda         = lambda;
            T.OptimOut.grad           = grad;
            T.OptimOut.hessian        = hessian;
            T.OptimOut.SolverOptions  = SolverOptions;
                    
        case 'InterPoint_Hessian'
            % ------------------------------------------------------------
            % ------------------------------------------------------------
            % ------------------------------------------------------------
            SolverOptions = optimset(  ...
                'TolFun', TolFun, ...
                'TolCon',TolCon, ...
                'TolX',TolX,...
                'DerivativeCheck','off',...
                'SubproblemAlgorithm','cg',...
                'GradObj','on',...
                'Hessian','user-supplied',...
                'FunValCheck','on',...
                'TolProjCG',1e-4,...
                'Algorithm','interior-point', ...
                'MaxFunEvals',inf, ...
                'MaxIter',MaxIter );
            
            if verbosity >= 2
              	SolverOptions.Display = 'iter-detailed';
            else
                SolverOptions.Display = 'none';
            end
            
            tic
            if AuxData.Options.UseMex
                %use mex compiled objective functions
                SolverOptions.HessFcn = @(x, lambda) EvalObjective3D_Hess_mex( x, AuxData );
                [G_opt,tmin,exitflag,output,lambda,grad,hessian] ...
                        = fmincon( @(x) EvalObjective3D_mex( x, AuxData ), G0 ,[],[],[],[],LowerBounds,UpperBounds,[],SolverOptions);
            else
                %use regular (m-file) objective function
                SolverOptions.HessFcn = @(x, lambda) EvalObjective3D_Hess( x, AuxData );
                [G_opt,tmin,exitflag,output,lambda,grad,hessian] ...
                     	= fmincon( @(x) EvalObjective3D( x, AuxData ), G0 ,[],[],[],[],LowerBounds,UpperBounds,[],SolverOptions); 
            end

            %consistent entries thoughout all solvers
            T.OptimOut.runtime        = toc;
            T.OptimOut.SolverID       = SolverID;
            T.OptimOut.tmin           = tmin;
            T.OptimOut.exitflag       = exitflag;
            T.OptimOut.exitreason     = ParseFlag( exitflag, SolverID );
            T.OptimOut.iterations     = output.iterations;
                    
            T.OptimOut.output         = output;
            T.OptimOut.lambda         = lambda;
            T.OptimOut.grad           = grad;
            T.OptimOut.hessian        = hessian;
            T.OptimOut.SolverOptions = SolverOptions;
                    
        case 'TrustRegion'
            % ------------------------------------------------------------
            % ------------------------------------------------------------
            % ------------------------------------------------------------
            SolverOptions = optimset(  ...
                'TolFun', TolFun, ...
                'TolCon',TolCon, ...
                'TolX',TolX,...
                'DerivativeCheck','off',...
                'GradObj','on',...
                'Hessian','user-supplied',...
                'FunValCheck','on',...
                'TolProjCG',1e-4,...
                'Algorithm','trust-region-reflective', ...
                'MaxFunEvals',inf, ...
                'MaxIter',MaxIter );
            
            if verbosity >= 2
              	SolverOptions.Display = 'iter-detailed';
            else
                SolverOptions.Display = 'none';
            end

            tic
            if AuxData.Options.UseMex
                %use mex compiled objective functions
                SolverOptions.HessFcn = @(x, lambda) EvalObjective3D_Hess_mex( x, AuxData );
                [G_opt,tmin,exitflag,output,lambda,grad,hessian] ...
                        = fmincon( @(x) EvalObjective3D_mex( x, AuxData ), G0 ,[],[],[],[],LowerBounds,UpperBounds,[],SolverOptions);
            else
                %use regular (m-file) objective function
                SolverOptions.HessFcn = @(x, lambda) EvalObjective3D_Hess( x, AuxData );
                [G_opt,tmin,exitflag,output,lambda,grad,hessian] ...
                     	= fmincon( @(x) EvalObjective3D( x, AuxData ), G0 ,[],[],[],[],LowerBounds,UpperBounds,[],SolverOptions); 
            end

            %consistent entries thoughout all solvers
            T.OptimOut.runtime        = toc;
            T.OptimOut.SolverID       = SolverID;
            T.OptimOut.tmin           = tmin;
            T.OptimOut.exitflag       = exitflag;
            T.OptimOut.exitreason     = ParseFlag( exitflag, SolverID );
            T.OptimOut.iterations     = output.iterations;
                    
            T.OptimOut.output         = output;
            T.OptimOut.lambda         = lambda;
            T.OptimOut.grad           = grad;
            T.OptimOut.hessian        = hessian;
            T.OptimOut.SolverOptions  = SolverOptions;
        
        case {'MinConF_SPG','MinConF_BBST','MinConF_OPG','MinConF_PQN','MinConF_QNST'}
            % ------------------------------------------------------------
            % ------------------------------------------------------------
            % ------------------------------------------------------------
            addpath(genpath('solvers/'))
            
            SolverOptions.optTol          = 1e-5;   % (default: 1e-5)
            SolverOptions.progTol         = 1e-5;   % (default: 1e-9)
            SolverOptions.maxIter         = MaxIter;% (default: 500)
            SolverOptions.numDiff         = 0 ;     % (default: 0 user-defined)
            SolverOptions.suffDec         = 1e-4;   % (default: 1e-4)
            SolverOptions.interp          = 2;      % (0: step-size halving, 1: quadratic, 2: cubic)
            SolverOptions.memory          = 100;    %
            SolverOptions.useSpectral     = 1;      % (default: 1)
            SolverOptions.curvilinear     = 0;      % (default: 0)
            SolverOptions.testOpt         = 1;      % (default: 1)
            SolverOptions.testInit        = 1;      % (default: 1)
            SolverOptions.optNorm         = inf;    % (default: 1)
            SolverOptions.feasibleInit    = 0;      % (default: 1)
            SolverOptions.bbType          = 1;      % (default: 1)
            
            if verbosity >= 2
              	SolverOptions.verbose = 2;
            else
                SolverOptions.verbose = 0;
            end
            
            tic
            warning('off', 'MATLAB:nearlySingularMatrix')
            if AuxData.Options.UseMex
                %use mex compiled objective functions
                switch SolverID
                    case 'MinConF_SPG',  [G_opt,tmin,funEvals,exitflag]  = minConf_SPG( @(x) EvalObjective3D_fvalJac_mex( x, AuxData ), G0, @(x) ProjectOnConvexSet( x, AuxData ), SolverOptions );
                    case 'MinConF_BBST', [G_opt,tmin,funEvals,exitflag]  = minConf_BBST( @(x) EvalObjective3D_fvalJac_mex( x, AuxData ), @(x) 0.0, G0, @(x,t) ProjectOnConvexSet( x, AuxData ), SolverOptions );
                    case 'MinConF_OPG',  [G_opt,tmin,funEvals,exitflag]  = minConf_OPG( @(x) EvalObjective3D_fvalJac_mex( x, AuxData ), G0, @(x) ProjectOnConvexSet( x, AuxData ), SolverOptions );
                    case 'MinConF_PQN',  [G_opt,tmin,funEvals,exitflag]  = minConf_PQN( @(x) EvalObjective3D_fvalJac_mex( x, AuxData ), G0, @(x) ProjectOnConvexSet( x, AuxData ), SolverOptions );
                    case 'MinConF_QNST', [G_opt,tmin,funEvals,exitflag]  = minConf_QNST( @(x) EvalObjective3D_fvalJac_mex( x, AuxData ), @(x) 0.0, G0, @(x,t) ProjectOnConvexSet( x, AuxData ), SolverOptions );
                end
            else
                %use regular (m-file) objective function
                switch SolverID
                    case 'MinConF_SPG',  [G_opt,tmin,funEvals,exitflag]  = minConf_SPG( @(x) EvalObjective3D( x, AuxData ), G0, @(x) ProjectOnConvexSet( x, AuxData ), SolverOptions );
                    case 'MinConF_BBST', [G_opt,tmin,funEvals,exitflag]  = minConf_BBST( @(x) EvalObjective3D( x, AuxData ), @(x) 0.0, G0, @(x,t) ProjectOnConvexSet( x, AuxData ), SolverOptions );
                    case 'MinConF_OPG',  [G_opt,tmin,funEvals,exitflag]  = minConf_OPG( @(x) EvalObjective3D( x, AuxData ), G0, @(x) ProjectOnConvexSet( x, AuxData ), SolverOptions );
                    case 'MinConF_PQN',  [G_opt,tmin,funEvals,exitflag]  = minConf_PQN( @(x) EvalObjective3D( x, AuxData ), G0, @(x) ProjectOnConvexSet( x, AuxData ), SolverOptions );
                    case 'MinConF_QNST', [G_opt,tmin,funEvals,exitflag]  = minConf_QNST( @(x) EvalObjective3D( x, AuxData ), @(x) 0.0, G0, @(x,t) ProjectOnConvexSet( x, AuxData ), SolverOptions );
                end
            end
            warning('on', 'MATLAB:nearlySingularMatrix')
            
            %consistent entries thoughout all solvers
            T.OptimOut.runtime        = toc;
            T.OptimOut.SolverID       = SolverID;
            T.OptimOut.tmin           = tmin;
            T.OptimOut.exitflag       = exitflag;
            T.OptimOut.exitreason     = ParseFlag( exitflag, SolverID );
            T.OptimOut.iterations     = funEvals;
            
            T.OptimOut.SolverOptions  = SolverOptions;
            
        case 'ModifiedShor'
            % ------------------------------------------------------------
            % ------------------------------------------------------------
            % ------------------------------------------------------------
            addpath(genpath('solvers/'))
            
            SolverOptions = [   -1, ...         % minimize or maximize
                                1.e-4, ...      % relative error of argument
                                1.e-5, ...      % relative error of function value
                                MaxIterShor, ...% max iter
                                NaN,...         % verbosity
                                1e-8, ...       % admissible maximal constraint residual
                                0.5, ...        % space dilation
                                1e-6 ];        % lower bound of stepsize

            if verbosity >= 2
              	SolverOptions(5) = 2;
            else
                SolverOptions(5) = -1;
            end
            
            tic 
            if AuxData.Options.UseMex
                %use mex compiled objective functions
                [G_opt,tmin,Options,exitflag] = solvopt( G0, @(x) EvalObjective3D_fval_mex( x, AuxData ), @(x) EvalObjective3D_Jac_mex( x, AuxData ), SolverOptions, @(x) max(abs(x) - Gmax * ones(size(x)) ) );
            else
                %use regular (m-file) objective function
                [G_opt,tmin,Options,exitflag] = solvopt( G0, @(x) EvalObjective3D_fval( x, AuxData ), @(x) EvalObjective3D_Jac( x, AuxData ), SolverOptions, @(x) max(abs(x) - Gmax * ones(size(x)) ) );
            end

            %consistent entries thoughout all solvers
            T.OptimOut.runtime        = toc;
            T.OptimOut.SolverID       = SolverID;
            T.OptimOut.tmin           = tmin;
            T.OptimOut.exitflag       = exitflag;
            T.OptimOut.exitreason     = ParseFlag( exitflag, SolverID );
            T.OptimOut.iterations     = Options(9);
                        
            T.OptimOut.objEvals       = Options(10);
            T.OptimOut.gradEvals      = Options(11);
            T.OptimOut.confunEvals    = Options(12);
            T.OptimOut.congradEvals   = Options(13);
            T.OptimOut.SolverOptions  = SolverOptions;

    end

    %command output
    if verbosity > 0
        fprintf('Final Results:\n')
        fprintf('---------------\n')
        fprintf('                 Optimized Duration: %.3f ms\n', T.OptimOut.tmin )
        fprintf('                  Algorithm Runtime: %.2f sec\n', T.OptimOut.runtime )
        fprintf('                         Iterations: %i\n', T.OptimOut.iterations )
        fprintf('                        Solver Type: %s\n', T.OptimOut.SolverID )
        fprintf('                        Exit Reason: %s\n', T.OptimOut.exitreason )
        fprintf('\n')
    end
    
    %back out the full optimal gradient vectors in the control points
    [Gx_opt, Gy_opt, Gz_opt] = GetGradientVectors( G_opt, AuxData );

    %store optimal node gradients
    T.Ana.G_opt     = G_opt;
    T.Ana.Gx_opt    = Gx_opt;
    T.Ana.Gy_opt    = Gy_opt;
    T.Ana.Gz_opt    = Gz_opt;

    %from the optimal gradients in [Gx_opt, Gy_opt, Gz_opt], compute the final
    %gradient waveform
    T = ComputeAnalyticGradients( T );
    T = CalculateSampledTrajectory( T );

    % plot trajectory and gradient waveforms
    if doPlot
        PlotTrajectory( T );
        PlotGradients( T, 'gs' );
    end
    
end

function exitreason = ParseFlag( exitflag, SolverID )

    switch SolverID
        case {'InterPoint_Jacobian','InterPoint_Hessian','TrustRegion'}
            
            switch exitflag
                case +1, exitreason = 'First-Order Optimality';
                case 0,  exitreason = 'MaxIter or MaxFunEvals';
                case -1, exitreason = 'OutputFun/PlotFun';
                case -2, exitreason = 'No Feasible Point Found';
                case +2, exitreason = 'TolX and TolCon';
                case +3, exitreason = 'TolFun and TolCon';
                case +4, exitreason = 'Small Search Direction';
                case +5, exitreason = 'Small Derivative';
                case -3, exitreason = 'ObjectiveLimit/TolCon';
                otherwise
                    exitreason = 'exitflag unknown';
            end  
            
        case {'MinConF_SPG','MinConF_BBST','MinConF_OPG','MinConF_PQN','MinConF_QNST'}

            switch exitflag
                case +1, exitreason = 'Optimality at Initial';
                case +2, exitreason = 'Small Dir. Derivative';
                case +3, exitreason = 'First-Order Optimality';
                case +4, exitreason = 'Step Size';
                case +5, exitreason = 'Function Value Change';
                case +6, exitreason = 'FunEval exceeds MaxIter';
                case +7, exitreason = 'NumProj exceeds MaxProject';
                case +8, exitreason = 'Iterations exceeds MaxProject';
                case +9, exitreason = 'Line search failed';
                otherwise
                    exitreason = 'exitflag unknown';
            end  
            
        case 'ModifiedShor'
            
            switch exitflag
                case -1,  exitreason = 'Missing Information';
                case -2,  exitreason = 'X has wrong dimension';
                case -3,  exitreason = 'No function specified';
                case -4,  exitreason = 'Invalid gradient';
                case -5,  exitreason = 'Invalid inital Function';
                case -6,  exitreason = 'Invalid inital Gradient';
                case -7,  exitreason = 'Function is Unbound';
                case -8,  exitreason = 'Gradient vanishes';
                case -9,  exitreason = 'MaxIter';
                case -11, exitreason = 'Premature stop possible';
                case -12, exitreason = 'Multiple extrema possible';
                case -13, exitreason = 'Flat at optimum';
                case -14, exitreason = 'Steep at optimum';
                otherwise
                    exitreason = 'exitflag unknown';
            end
    end
    
end