function AuxData = SetUpAuxData2D( R_mat, dkx, dky, Gmax, Smax, Options, verbosity )    

    %{ 
    Method:
    -------
    Set up an AuxData struct to be passed to EvalObjective2D(). The method
    checks the validity of the constraint matrix and sets up the correct
    derivative matrix D. Idx vectors are set to efficiently compute the
    objective function value. 
    
    Input:
    ------
    
    R       - constraint matrix (NCP x 3)
    dkx     - vector of distances in between consecutive control points in
              x dimension (NCP-1 x 1)
    dky     - vector of distances in between consecutive control points in
              y dimension (NCP-1 x 1)
    Gmax    - gradient magnitude constraint (single, [mT/m])
    Smax    - gradient slew rate constraint (single, [T/m/s])
    Options - options struct
    verbose - print verbosity

    Output:
    -------
    
    AuxData   - struct
    %}

    %set default value
    if nargin < 6
        warning('Using default options!')
        Options.OnlyContinuousRanges    = 0;
        Options.GradSmooth_On           = 1;
        Options.GradSmooth_Lambda       = 1e-3; %[ms]
        Options.GradSmooth_MaxDistFac   = 2;
        Options.GradSmooth_MaxDistFac   = 2;
        Options.UseMex                  = 0;
        Options.verbosity              	= 0;
        Options.SolverID                = 'ModifiedShor';
        
        % PrintDefaultParameter( 'OnlyContinuousRanges', Options.OnlyContinuousRanges )
        % PrintDefaultParameter( 'GradSmooth_On', Options.GradSmooth_On )
        % PrintDefaultParameter( 'GradSmooth_Lambda', Options.GradSmooth_Lambda )
        % PrintDefaultParameter( 'GradSmooth_MaxDistFac', Options.GradSmooth_MaxDistFac )
        % PrintDefaultParameter( 'UseMex', Options.UseMex )
    end
    
    if nargin < 7
        verbosity = 1;
    end

    N = numel(dkx) + 1;
    
    %correct dimension
    if size(R_mat,1) ~= N || size(R_mat,2) ~= 3
            error('Constraint cell R has wrong dimension.')
    end

    if  numel(dkx) ~= N-1
            error('dkx input values has to have length of N+1!')
    end
    if  numel(dky) ~= N-1
            error('dky input values has to have length of N+1!')
    end

    %default options for the functional
    if ~isfield( Options, 'OnlyContinuousRanges' )
        Options.OnlyContinuousRanges = 0;
        PrintDefaultParameter( 'OnlyContinuousRanges', Options.OnlyContinuousRanges )
    end
    if ~isfield( Options, 'GradSmooth_On' )
        Options.GradSmooth_On = 1;
        PrintDefaultParameter( 'GradSmooth_On', Options.GradSmooth_On )
    end
    if ~isfield( Options, 'GradSmooth_Lambda' )
        Options.GradSmooth_Lambda = 1e-3; %[ms]
        PrintDefaultParameter( 'GradSmooth_Lambda', Options.GradSmooth_Lambda )
    end
    if ~isfield( Options, 'GradSmooth_MaxDistFac' )
        Options.GradSmooth_MaxDistFac = 2;
        PrintDefaultParameter( 'GradSmooth_MaxDistFac', Options.GradSmooth_MaxDistFac )
    end
    if ~isfield( Options, 'UseMex' )
        Options.UseMex 	= 0;
        PrintDefaultParameter( 'UseMex', Options.UseMex )
    end
    if ~isfield( Options, 'verbosity' )
        Options.verbosity = 0;
        PrintDefaultParameter( 'verbosity', Options.verbosity )
    end
    if ~isfield( Options, 'SolverID' )
        Options.SolverID = 'ModifiedShor';
        PrintDefaultParameter( 'SolverID', Options.SolverID )
    end

    %check, if constraint matrix appears to be well defined
    if any( all( isnan(R_mat(:,1:2)), 2 ) & R_mat(:,3) ~= 2 )
        %any row with all NaN (except the identifier in the last column)
        %should have an identifier '2'
        error('Constraint matrix seems to be invalid: if no constraint for a control point is required, an identifier of "2" is expected')
    elseif any( all( ~isnan(R_mat(:,1:2)), 2 ) & ( R_mat(:,3) > 1 ) )
        %any row with no NaN (except the identifier in the last column)
        %should have an identifier of either '1' (vector constraint), 
        %'0' (fixed gradients), or be < 0 (linked control point)
        error('Constraint matrix seems to be invalid: if constraints for a control point are required, an identifier of either "1" (vector constraint), "0" (fixed gradients) or "< 0" (linked control points) is expected')
 	elseif any( any( isnan(R_mat(:,1:2)), 2 ) & any( ~isnan(R_mat(:,1:2)), 2 ) )
        %any row with no NaN (except the identifier in the last column)
        %should have an identifier of either '1' (vector constraint) or 
        %'0' (fixed gradients)
        error('Constraint matrix seems to be invalid: at least one of the control point constraints contains both a NaN and non-NaN entry!')
 	elseif any( all( R_mat(:,1:2) == 0, 2 ) & R_mat(:,3) ~= 0 )
        %zero constraint only valid with fixed gradient
        error('Constraint matrix seems to be invalid: directional vectors of zero-length are not supported. Set fixed gradients (ID = 0) instead!')
    elseif any( any( isnan(R_mat(:,1:2)), 2 ) & R_mat(:,3) < 0 )
        %negative entries of the constraint identifier indicate, that the
        %control point's gradient vector is linked by another control
        %point. In this case only non-nan entries are allowed.
        error('Constraint matrix seems to be invalid: NaN-entries in linked control points are found!')
    elseif any( R_mat( abs(R_mat( R_mat(:,3) < 0, 3 ) ), 3 ) < 0 )
        %linked control points must not be linked to control points that in
        %turn are linked to other points.
        error('Constraint matrix seems to be invalid: control points seem to be linked to linked control points!')
    end
    
    OptsVector  = [ Options.OnlyContinuousRanges, ...
                    Options.GradSmooth_On, ...
                    Options.GradSmooth_Lambda, ...
                    Options.GradSmooth_MaxDistFac ];
 
    NumOfVars   = sum( R_mat(:,3) );
    %print info on control points
    if verbosity > 0
        fprintf('Control Points:\n')
        fprintf('---------------\n')
        fprintf('     Number of unconstrained points: %i (%i variables)\n',sum( double( R_mat(:,3) == 2 ) ),2 * sum( double( R_mat(:,3) == 2 ) ) )
        fprintf('Number of vector constrained points: %i (%i variables)\n',sum( double( R_mat(:,3) == 1 ) ),1 * sum( double( R_mat(:,3) == 1 ) ) )
        fprintf('             Number of fixed points: %i (%i variables)\n',sum( double( R_mat(:,3) == 0 ) ),0 * sum( double( R_mat(:,3) == 0 ) ) )
        fprintf('            Number of linked points: %i (%i variables)\n',sum( double( R_mat(:,3)  < 0 ) ),0 * sum( double( R_mat(:,3)  < 0 ) ) )
        fprintf('                Number of variables: %i\n',NumOfVars)
        fprintf('\n')
        fprintf('Options:\n')
        fprintf('---------------\n')
        fprintf('       Options.OnlyContinuousRanges: %i \n',Options.OnlyContinuousRanges)
        fprintf('              Options.GradSmooth_On: %i \n',Options.GradSmooth_On)
        fprintf('          Options.GradSmooth_Lambda: %.1e \n',Options.GradSmooth_Lambda)
        fprintf('      Options.GradSmooth_MaxDistFac: %i \n',Options.GradSmooth_MaxDistFac)
        fprintf('\n')
    end

    %set up an AuxData struct
    AuxData.R               = R_mat;
    AuxData.dkx             = dkx;
    AuxData.dky             = dky;
    AuxData.dkz             = zeros( size( dkx ) );
    AuxData.Gmax            = Gmax;
    AuxData.Smax            = Smax;
    AuxData.Options         = Options;
    
    %store variables needed for objective function evaluation
    AuxData.OptsVector          = OptsVector;
    AuxData.D                   = GetDerivativeMatrix2D( R_mat );
    AuxData.Hs                  = double( AuxData.D ~= 0 );
    
    AuxData.Options = orderfields(AuxData.Options, {'OnlyContinuousRanges', ...
                                                    'GradSmooth_On', ...
                                                    'GradSmooth_Lambda', ...
                                                    'GradSmooth_MaxDistFac', ...
                                                    'verbosity', ...
                                                    'UseMex', ...
                                                    'SolverID' } );
    