function PlotIterate( objfundata, T, RF, forwardmagn, options, IsWithinOptim, savename )

    % This method is used to create and save plots during optimization to
    % check the magnetization etc. during runtime

    if nargin < 6
       IsWithinOptim = 1; 
    end
    
    if nargin < 7
       savename = []; 
    end
    
    global DateString
    global CounterIter
    global CounterFunEval
    
    % ---------------------------------------
    % -----  T R Y   P L O T T I N G    -----
    % ---------------------------------------

    %close if another instance is open
    close(findobj('type','figure','name','PlotIterateFigure'))

    %open figure
    fig = PlotIterateFigure();

    %set strings
    %-----------
    set( findobj('Tag','string_functionvalue'), 'String', sprintf('Objective Function Value = %.2f', objfundata.funValue ) )
    set( findobj('Tag','string_iter'), 'String', sprintf('Iterations = %i         Function Evaluations = %i', CounterIter - 1, CounterFunEval - sum(options.phi_isvariable) ) )

    set( findobj('Tag','string_time'), 'String', sprintf('T = %.2f ms', objfundata.time ) )
    set( findobj('Tag','string_RMSE'), 'String', sprintf('NRMSE = %.2f%%', objfundata.NRMSE	 ) )
    set( findobj('Tag','string_RF'), 'String', sprintf('RF Waveforms (%i-channel)', options.nchannel	 ) )

    if all( T.Sam.kz == 0 )
        %remove some labels
        set( findobj('Tag','famap_xlabel'), 'Visible', 'off' )
        set( findobj('Tag','famap_ylabel'), 'Visible', 'off' )
        set( findobj('Tag','famap_zlabel'), 'Visible', 'off' )

        set( findobj('Tag','trajectory_xlabel'), 'Visible', 'off' )
        set( findobj('Tag','trajectory_ylabel'), 'Visible', 'off' )
        set( findobj('Tag','trajectory_zlabel'), 'Visible', 'off' )
    end

    %plot trajectory
    %---------------
    axes( findobj('Type','axes','Tag','trajectory') )
    cla
    hold on

    %estimate a good kmax value for plotting

    Step        = 55.0;
    kmax_T      = max( abs( [T.Sam.kx(:); T.Sam.ky(:); T.Sam.kz(:)] ) );
    kmax_steps  = 0.0 : Step : Step*ceil(max( 1.5 * options.phi_bounds(:))/Step);
    kmax        = kmax_steps( find( kmax_T < kmax_steps,1,'first' ) );
    if isempty( kmax )
       kmax = 1.1 * round(kmax_T);
    end

    gui_PlotTrajectoryVelocity3D( T );
    plot3( T.kx, T.ky, T.kz,'Marker','.','Markersize',15,'Color','black','Linestyle','none')
    cbh = colorbar;
    set(get(cbh,'xlabel'),'String', 'Velocity ||\gamma G ||_2 [m^{-1}/ms]','Fontsize',9,'Fontweight','bold');
    set(cbh,'Location','East','Fontsize',9,'Fontweight','bold')

    %also plot previous iteration
    T_0 = [];
    if isfield( T_0, 'Sam' ) && 0
        if isfield( T_0.Sam, 'kx' ) && isfield( T_0.Sam, 'ky' ) && isfield( T_0.Sam, 'kz' )
            plot3( T_0.Sam.kx, T_0.Sam.ky, T_0.Sam.kz,'Color',[0.85,0.85,0.85],'Linestyle','-')
            plot3( T_0.kx, T_0.ky, T_0.kz,'Marker','.','Markersize',10,'Color',[0.7,0.7,0.7],'Linestyle','none')
        end
    end

    xlim([-kmax, +kmax])
    ylim([-kmax, +kmax])
    zlim([-kmax, +kmax])

    %plot gradient waveforms
    %----------------------- 
    axes( findobj('Type','axes','Tag','gradient') )
    gui_PlotGradients( T, 'g' );

    %plot Flip Angle Map
    %-------------------
    axes( findobj('Type','axes','Tag','famap') )
    cla

    if iscell( forwardmagn )  
        display_nimages( magn2flip(forwardmagn.mt), 0); axis image off;
        colormap(ColormapBlueHot()); colorbar; title('STA -- FA [deg.]')
        set(gca,'Clim',[0.0, 1.2 * max(options.targetflip_mat(:))])
    else
        display_nimages( magn2flip(forwardmagn.mt), 0 ); axis image off;
        colormap(ColormapBlueHot()); colorbar; title('STA -- FA [deg.]')
        set(gca,'Clim',[0.0, 1.2 * max(options.targetflip_mat(:))])
    end

    %plot Flip Angle profiles
    %------------------------

    if iscell( forwardmagn )  
        gui_PlotProfiles( forwardmagn.flip, [], options.targetflip_mat, options, ...
            findobj('Type','axes','Tag','faprofile1'), ...
            findobj('Type','axes','Tag','faprofile2'),...
            findobj('Type','axes','Tag','faprofile3') );
    else
        gui_PlotProfiles( forwardmagn.flip, [], options.targetflip_mat, options, ...
            findobj('Type','axes','Tag','faprofile1'), ...
            findobj('Type','axes','Tag','faprofile2'),...
            findobj('Type','axes','Tag','faprofile3') );
    end

    %plot RF waveforms
    %----------------- 
    axes( findobj('Type','axes','Tag','RF') )

    gui_PlotRF( RF, options );

    if IsWithinOptim || ~isempty( savename )
        set(gcf, 'InvertHardCopy', 'off');
        set(fig,'Visible','off')
        set(gcf, 'PaperPositionMode','auto')

        if isempty( savename )
            %get current index
            Iter = 0;
            while( exist( sprintf('SavedIterations/%s/iter_%03d_figure.png',DateString,Iter), 'file' ) )
                Iter = Iter + 1;
            end

            print('-dpng','-r0',sprintf('SavedIterations/%s/iter_%03d_figure.png',DateString,Iter) )
            close( fig )
            pause(0.01)
        else
            print('-dpng','-r0',sprintf('SavedIterations/%s/%s.png',DateString,savename) )
            close( fig )
            pause(0.01)
        end
        
    end
             
end

function [ Hx, Hy, Hz ] = gui_PlotGradients( T, ID )

    if nargin < 2
       ID = 'g';
    else
       if ~any( cellfun( @(s) strcmpi( s, ID ), {'g','s','gs','gc','h'} ) )
          warning('Identifier ''%s'' undefined, only ''g'',''s'',''gs'', ''gc'', or ''p'' are allowed. Using default ''g''!',ID) 
       end
    end
 
    %figure
    switch ID
        case 'g'
            %Gradient Magnitude Only
            [Hx,Hy,Hz,C,~] = PlotWaveforms( T.Ana.tx, T.Ana.ty, T.Ana.tz, T.Ana.Gx, T.Ana.Gy, T.Ana.Gz, T.t, T.Gmax, 'plot' );
            xlabel('Time t [ms]','Fontsize',9,'Fontweight','bold')
            ylabel('Gradient Magnitude G(t) [mT/m]','Fontsize',9,'Fontweight','bold')
            LegHandle = legend([Hx,Hy,Hz,C],'G_x(t)','G_y(t)','G_z(t)','G_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.t), max(T.t)])
            ylim([-1.1 * T.Gmax, +1.1 * T.Gmax ])
            
        case 's'
            %Gradient SlewRate Only
            [Hx,Hy,Hz,C,~] = PlotWaveforms( T.Ana.tx, T.Ana.ty, T.Ana.tz, T.Ana.Sx, T.Ana.Sy, T.Ana.Sz, T.t, T.Smax, 'stairs'  );
            xlabel('Time t [ms]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Slew Rate G(t) [T/m/s]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'S_x(t)','S_y(t)','S_z(t)','S_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.t), max(T.t)])
            ylim([-1.1 * T.Smax, +1.1 * T.Smax ])
            
    	case 'gs'
            %Gradient Magnitude and SlewRate
            subplot(4,10,1:20)
            [Hx,Hy,Hz,C,ah1] = PlotWaveforms( T.Ana.tx, T.Ana.ty, T.Ana.tz, T.Ana.Gx, T.Ana.Gy, T.Ana.Gz, T.t, T.Gmax, 'plot' );
            xlabel('Time t [ms]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Magnitude G(t) [mT/m]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'G_x(t)','G_y(t)','G_z(t)','G_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.t), max(T.t)])
            ylim([-1.1 * T.Gmax, +1.1 * T.Gmax ])
            
            subplot(4,10,21:40)
            [Hx,Hy,Hz,C,ah2] = PlotWaveforms( T.Ana.tx, T.Ana.ty, T.Ana.tz, T.Ana.Sx, T.Ana.Sy, T.Ana.Sz, T.t, T.Smax, 'stairs'  );
            xlabel('Time t [ms]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Slew Rate G(t) [T/m/s]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'S_x(t)','S_y(t)','S_z(t)','S_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.t), max(T.t)])
            ylim([-1.1 * T.Smax, +1.1 * T.Smax ])
            
            linkaxes([ah1, ah2],'x')
            
        case 'gc'
            %Gradient Magnitude and Constraint Behaviour
            subplot(4,10,1:20)
            [Hx,Hy,Hz,C,ah1] = PlotWaveforms( T.Ana.tx, T.Ana.ty, T.Ana.tz, T.Ana.Gx, T.Ana.Gy, T.Ana.Gz, T.t, T.Gmax, 'plot' );
            xlabel('Time t [ms]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Magnitude G(t) [mT/m]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'G_x(t)','G_y(t)','G_z(t)','G_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.t), max(T.t)])
            ylim([-1.1 * T.Gmax, +1.1 * T.Gmax ])
            
            subplot(4,10,21:40)
            [Magn, Slew, ah2] = PlotConstraints( T );
            xlabel('Time t [ms]','Fontsize',12,'Fontweight','normal')
            ylabel('Active Constraints','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Magn,Slew],'Magnitude Constraint','Slew Rate Constraint');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.t), max(T.t)])
            ylim([-1,4])
            
            linkaxes([ah1, ah2],'x')
            
        case 'p'
            
            %first check, if hardware gradients exist
            if  isempty( T.Pulse.Gx ) || isempty( T.Pulse.Gy ) || isempty( T.Pulse.Gz ) || ...
                isempty( T.Pulse.Sx ) || isempty( T.Pulse.Sy ) || isempty( T.Pulse.Sz ) || ...
                isempty( T.Pulse.kx ) || isempty( T.Pulse.ky ) || isempty( T.Pulse.kz ) || ...
                isempty( T.Pulse.t )
                    error('No hardware trajectory has beend efined T-struct!')
            end
            
            %Gradient Magnitude and SlewRate
            subplot(4,10,1:20)
            [Hx,Hy,Hz,C,ah1] = PlotWaveforms( T.Pulse.t, T.Pulse.t, T.Pulse.t, T.Pulse.Gx, T.Pulse.Gy, T.Pulse.Gz, [], T.Gmax, 'stairs' );
            xlabel('Time t [s]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Magnitude G(t) [T/m]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'G_x(t)','G_y(t)','G_z(t)','G_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.Pulse.t), max(T.Pulse.t)])
            ylim([-1.1 * T.Gmax / 1e+3, +1.1 * T.Gmax / 1e+3 ])
            
            subplot(4,10,21:40)
            [Hx,Hy,Hz,C,ah2] = PlotWaveforms( T.Pulse.t, T.Pulse.t, T.Pulse.t, T.Pulse.Sx, T.Pulse.Sy, T.Pulse.Sz, [], T.Smax, 'stairs' );
            xlabel('Time t [s]','Fontsize',12,'Fontweight','normal')
            ylabel('Gradient Slew Rate S(t) [T/m/s]','Fontsize',12,'Fontweight','normal')
            LegHandle = legend([Hx,Hy,Hz,C],'S_x(t)','S_y(t)','S_z(t)','S_{max}');
            set(LegHandle,'orientation','horizontal')
            
            xlim([min(T.Pulse.t), max(T.Pulse.t)])
            ylim([-1.1 * T.Smax, +1.1 * T.Smax ])
            
            linkaxes([ah1, ah2],'x')
            
    end
        
end

function [Hx,Hy,Hz,C,ah] = PlotWaveforms( tx, ty, tz, fx, fy, fz, Interp, Constraint, FunType )

    Linewidth = 1.0;

    %prepare axis
    ah = gca;
    cla
    box on
    grid on
    hold on
    
    %plot constraints
        line(   [min([tx(1),ty(1)]), max([tx(end),ty(end)])], ...
            [+Constraint, +Constraint],'Linestyle','--','Linewidth',2,'Color',[0 0.7 0]);
    C = line(   [min([tx(1),ty(1)]), max([tx(end),ty(end)])], ...
            [-Constraint, -Constraint],'Linestyle','--','Linewidth',2,'Color',[0 0.7 0]);    
    
	%this method plots the analytic gradients
    switch FunType
        case 'plot'
            Hx = plot( tx, fx, 'Linewidth',Linewidth,'Color','red');
            Hy = plot( ty, fy, 'Linewidth',Linewidth,'Color','blue');
            Hz = plot( tz, fz, 'Linewidth',Linewidth,'Color',[0 0.7 0]);
            
            %plot patch
            AlphaVal = 0.35;
            phx = patch([tx(1),tx,tx(end)],[0.0,fx,0.0],'red','Edgecolor','none');
            alpha(phx,AlphaVal);
            phy = patch([ty(1),ty,ty(end)],[0.0,fy,0.0],'blue','Edgecolor','none');
            alpha(phy,AlphaVal);
            phz = patch([tz(1),tz,tz(end)],[0.0,fz,0.0],[0 0.7 0],'Edgecolor','none');
            alpha(phz,AlphaVal);
            
        case 'stairs'
            Hx = stairs( tx, fx, 'Linewidth',Linewidth,'Color','red');
            Hy = stairs( ty, fy, 'Linewidth',Linewidth,'Color','blue');
            Hz = stairs( tz, fz, 'Linewidth',Linewidth,'Color',[0 0.7 0]);
            
            %create some auxilary vectors to allow for nice plotting of the
            %Smax waveforms
            
            tx_temp = reshape([tx; tx],1,[]);
            tx_temp = tx_temp( 2 : end-1 );
            fx_temp = reshape([fx; fx],1,[]);
            fx_temp = fx_temp( 1 : end-2 );
            
            ty_temp = reshape([ty; ty],1,[]);
            ty_temp = ty_temp( 2 : end-1 );
            fy_temp = reshape([fy; fy],1,[]);
            fy_temp = fy_temp( 1 : end-2 );
            
            tz_temp = reshape([tz; tz],1,[]);
            tz_temp = tz_temp( 2 : end-1 );
            fz_temp = reshape([fz; fz],1,[]);
            fz_temp = fz_temp( 1 : end-2 );
            
            
            %plot patch
            AlphaVal = 0.35;
            phx = patch([tx_temp(1),tx_temp,tx_temp(end)],[0.0,fx_temp,0.0],'red','Edgecolor','none');
            alpha(phx,AlphaVal);
            phy = patch([ty_temp(1),ty_temp,ty_temp(end)],[0.0,fy_temp,0.0],'blue','Edgecolor','none');
            alpha(phy,AlphaVal);
            phz = patch([tz_temp(1),tz_temp,tz_temp(end)],[0.0,fz_temp,0.0],[0 0.7 0],'Edgecolor','none');
            alpha(phz,AlphaVal);
            
    end

    %this method plots the times of the interpolation as vertical lines
    for k = 1 : numel(Interp)
       line([Interp(k), Interp(k)], [-Constraint, +Constraint], ...
          'Linestyle','-','Linewidth',1,'Color',[1 0.55 0]); 
    end

end

function [Magn, Slew, ah1] = PlotConstraints( T )

    %prepare axis
    ah1 = gca;
    cla
    box on
    grid on
    hold on
    
    % first of all, determine intervals at which the Magnitude OR SlewRate
    % is the current active constraint
    T = CalculateSampledTrajectory( T );
    ActiveConstraint_SlewRate   = max( abs([T.Sam.Sx(:), T.Sam.Sy(:), T.Sam.Sz(:)]), [], 2 ) >= 0.99 * T.Smax;
    ActiveConstraint_Magnitude  = max( abs([T.Sam.Gx(:), T.Sam.Gy(:), T.Sam.Gz(:)]), [], 2 ) >= 0.99 * T.Gmax;
    
    %perform morphological closing on both vectors
    ActiveConstraint_SlewRate   = imclose( ActiveConstraint_SlewRate, [1,1]' );
    ActiveConstraint_Magnitude  = imclose( ActiveConstraint_Magnitude, [1,1]' );
    Components_SlewRate         = bwconncomp( ActiveConstraint_SlewRate, 4 );
    Components_Magnitude        = bwconncomp( ActiveConstraint_Magnitude, 4 );
    
    MagnBounds  = [1.5, 3.0];
    SlewBounds  = [0.0, 1.5];
    
    %plot patches
    for idx_slew = 1 : Components_SlewRate.NumObjects
        Slew = rectangle('Position',[T.Sam.t(Components_SlewRate.PixelIdxList{idx_slew}(1)),SlewBounds(1), ...
                                     T.Sam.t(Components_SlewRate.PixelIdxList{idx_slew}(end)) - T.Sam.t(Components_SlewRate.PixelIdxList{idx_slew}(1)),diff(SlewBounds)], ...
                                     'FaceColor','black','EdgeColor','none');
    end
    for idx_magn = 1 : Components_Magnitude.NumObjects
        Magn = rectangle('Position',[T.Sam.t(Components_Magnitude.PixelIdxList{idx_magn}(1)),MagnBounds(1), ...
                                     T.Sam.t(Components_Magnitude.PixelIdxList{idx_magn}(end)) - T.Sam.t(Components_Magnitude.PixelIdxList{idx_magn}(1)),diff(MagnBounds)], ...
                                     'FaceColor','red','EdgeColor','none');
    end
    
    Slew = plot(nan,nan,'s','markeredgecolor',get(Slew,'edgecolor'),'markerfacecolor',get(Slew,'facecolor'));
    Magn = plot(nan,nan,'s','markeredgecolor',get(Magn,'edgecolor'),'markerfacecolor',get(Magn,'facecolor'));
    
    
    xlim([0.0, T.Sam.t(end)])
    ylim([-1.0, 4.0])
    set(gca,'YTick',[])
    
end

function gui_PlotProfiles( Mat1, Mat2, Target, options, hand1, hand2, hand3 )

    Mat1        = abs( Mat1 );
    if ~isempty( Mat2 )
        Mat2        = abs( Mat2 );
    end
    Target      = abs( Target );
    
    %set nan according to mask
    Mat1( options.roi == 0 )     	= nan;
    if ~isempty( Mat2 )
        Mat2( options.roi == 0 )    	= nan;
    end
    Target( options.roi == 0 )    = nan;
    
    %prepare
    PixPos_X  = options.xpos_vec;
    PixPos_Y  = options.ypos_vec;
    PixPos_Z  = options.zpos_vec;
   
    %extract lines
    [siz_x, siz_y, siz_z]   = size( Mat1 );
    
    %get center of target
    [largeval_x, largeval_y, largeval_z] = ind2sub( size(Target), find( Target > 0.5 * max(Target(:)) ) );
    cen_x   = round( mean( largeval_x ) );
    cen_y   = round( mean( largeval_y ) );
    cen_z   = round( mean( largeval_z ) );
    
    % --- x ---
    Mat1_X      = squeeze( Mat1( :,cen_y, cen_z ) );
    Target_X    = squeeze( Target( :,cen_y,cen_z ) );
    
    % --- y ---
    Mat1_Y      = squeeze( Mat1( cen_x, :, cen_z ) );
    Target_Y    = squeeze( Target( cen_x, :, cen_z ) );
    
    % --- z ---
    Mat1_Z      = squeeze( Mat1( cen_x, cen_y, : ) );
    Target_Z    = squeeze( Target( cen_x, cen_y, : ) );
    
    if ~isempty( Mat2 )
        Mat2_X      = squeeze( Mat2( :,round(siz_y/2),round(siz_z/2) ) );
        Mat2_Y      = squeeze( Mat2( round(siz_x/2), :, round(siz_z/2) ) );
        Mat2_Z      = squeeze( Mat2( round(siz_x/2), round(siz_y/2), : ) );
        
        Mat2_X( isnan( Target_X ) )     = [];
        Mat2_Y( isnan( Target_Y ) )     = [];
        Mat2_Z( isnan( Target_Z ) )     = [];
    end
    
    %remove nan's
    PixPos_X( isnan( Target_X ) )   = [];
    Mat1_X( isnan( Target_X ) )     = [];
    Target_X( isnan( Target_X ) )   = [];
    
    PixPos_Y( isnan( Target_Y ) )   = [];
    Mat1_Y( isnan( Target_Y ) )     = [];
    Target_Y( isnan( Target_Y ) )   = [];
    
    PixPos_Z( isnan( Target_Z ) )   = [];
    Mat1_Z( isnan( Target_Z ) )     = [];
    Target_Z( isnan( Target_Z ) )   = [];
    
    %prepare figure
    Target_Style    = '-';
    Target_Color    = [0.4, 0.4, 0.4];
    Target_Width    = 1.5;
    
    Mat1_Style      = '-';
    Mat1_Color      = 'red';
    Mat1_Width      = 1.5;
    
    Mat2_Style      = '--';
    Mat2_Color      = 'black';
    Mat2_Width      = 1.5;
    
    
    %plot x profiles
    %---------------
    axes( hand1 )
    cla
    hold on
    box on
    grid on
    
    plot( PixPos_X, Target_X, 'Linewidth', Target_Width, 'Linestyle', Target_Style, 'Color', Target_Color,'DisplayName','Target');
    plot( PixPos_X, Mat1_X, 'Linewidth', Mat1_Width, 'Linestyle', Mat1_Style, 'Color', Mat1_Color,'DisplayName','Achieved');
    if ~isempty( Mat2 )
        plot( PixPos_X, Mat2_X, 'Linewidth', Mat2_Width, 'Linestyle', Mat2_Style, 'Color', Mat2_Color,'DisplayName','Optimized');
    end
    
    xlim([min(PixPos_X), max(PixPos_X)+0.001])
    ylim([-0.1*max(options.targetflip_mat(:)), 1.1*max(options.targetflip_mat(:))])
    %xlabel('x [m]','Fontsize',8,'Fontweight','bold')
    ylabel('Flipangle','Fontsize',8,'Fontweight','bold')
    
    %plot y profiles
    %---------------
    axes( hand2 )
    cla
    hold on
    box on
    grid on
    
    plot( PixPos_Y, Target_Y, 'Linewidth', Target_Width, 'Linestyle', Target_Style, 'Color', Target_Color )
    plot( PixPos_Y, Mat1_Y, 'Linewidth', Mat1_Width, 'Linestyle', Mat1_Style, 'Color', Mat1_Color )
    if ~isempty( Mat2 )
        plot( PixPos_Y, Mat2_Y, 'Linewidth', Mat2_Width, 'Linestyle', Mat2_Style, 'Color', Mat2_Color )
    end
    
    xlim([min(PixPos_Y), max(PixPos_Y)+0.001])
    ylim([-0.1*max(options.targetflip_mat(:)), 1.1*max(options.targetflip_mat(:))])
    %xlabel('y [m]','Fontsize',8,'Fontweight','bold')
    ylabel('Flipangle','Fontsize',8,'Fontweight','bold')
    
    %plot z profiles
    %---------------
    axes( hand3 )
    cla
    hold on
    box on
    grid on
    
    plot( PixPos_Z, Target_Z, 'Linewidth', Target_Width, 'Linestyle', Target_Style, 'Color', Target_Color )
    plot( PixPos_Z, Mat1_Z, 'Linewidth', Mat1_Width, 'Linestyle', Mat1_Style, 'Color', Mat1_Color )
    if ~isempty( Mat2 )
        plot( PixPos_Z, Mat2_Z, 'Linewidth', Mat2_Width, 'Linestyle', Mat2_Style, 'Color', Mat2_Color )
    end
    
    xlim([min(PixPos_Z), max(PixPos_Z)+0.001])
    ylim([-0.1*max(options.targetflip_mat(:)), 1.1*max(options.targetflip_mat(:))])
    %xlabel('z [m]','Fontsize',8,'Fontweight','bold')
    ylabel('Flipangle','Fontsize',8,'Fontweight','bold')

end

function gui_PlotRF( RF, options )

    cla
    hold on
    box on

    %get positions of subpulses
    SubPos  = 1 : numel(RF)/options.nchannel : numel(RF);
    
    %plot ticks
    
    [AX,H1,H2] = plotyy( 1 : numel(RF), abs( RF ), 1 : numel(RF), angle( RF ), @stairs );
    set(get(AX(1),'Ylabel'),'String','RF Magnitude','Fontweight','bold') 
    set(get(AX(2),'Ylabel'),'String','RF Phase','Fontweight','bold')
    xlim([1, numel(RF)])
    
    %set x ticks
    set(AX(1),'XTick',[])
    set(AX(2),'XTick',SubPos)
    set(AX(2),'XTickLabel',arrayfun(@(x) sprintf('#%i',x) ,1 : 16,'UniformOutput',0))
    set(AX(2),'XColor','red')
	set(AX(2),'XGrid','on');
    set(AX(2),'GridLineStyle','-')
    xlabel({[],'Channel'},'Fontsize',9,'Color','red','Fontweight','bold' )
    
    set(AX(1), 'XLim', [1, numel(RF)])
    set(AX(2), 'XLim', [1, numel(RF)])
    linkaxes(AX, 'x');    
    
    set(H1,'Linewidth',1,'Linestyle','-')
    set(H2,'Linewidth',1,'Linestyle','-')

end

function h = gui_PlotTrajectoryVelocity3D( T )
       
    %error handling
    if  isempty( T.Sam.Gx ) || ...
        isempty( T.Sam.Gy ) || ...
        isempty( T.Sam.Gz ) || ...
        isempty( T.Sam.kx ) || ...
        isempty( T.Sam.ky ) || ...
        isempty( T.Sam.kz )
            warning('WarnTests:convertTest','Calculate sampled trajectory before calling the plot funtion')
            %compute sampled trajectory
            T = tra_CalculateSampledTrajectory3D( T );
    end

    %if vectors are very long, make them sparser
    AimLength   = 10000;
 	if numel(T.Sam.kx) > AimLength
        Step        = numel(T.Sam.kx) / AimLength;
	else
        Step        = 1;
	end
    tidx_plot   = 1 : ceil(Step) : numel( T.Sam.kx );
    kx_plot     = T.Sam.kx( tidx_plot );
    ky_plot     = T.Sam.ky( tidx_plot );
    kz_plot     = T.Sam.kz( tidx_plot );
    Gx_plot     = T.Sam.Gx( tidx_plot );
    Gy_plot     = T.Sam.Gy( tidx_plot );
    Gz_plot     = T.Sam.Gz( tidx_plot );
    if Step ~= round(Step)
        tidx_plot   = [ tidx_plot, numel( T.Sam.kx ) ];
        kx_plot     = [ kx_plot, T.Sam.kx(end) ];
        ky_plot     = [ ky_plot, T.Sam.ky(end) ];
        kz_plot     = [ kz_plot, T.Sam.kz(end) ];
        Gx_plot     = [ Gx_plot, T.Sam.Gx(end) ];
        Gy_plot     = [ Gy_plot, T.Sam.Gy(end) ];
        Gz_plot     = [ Gz_plot, T.Sam.Gz(end) ];
    end
    
    %create vector of velocitys
    V   = sqrt( (Gx_plot(:)).^2 + (Gy_plot(:)).^2 + (Gz_plot(:)).^2 );
    
    %prepare vector of RF-active entries
    tidx_playRF     = ones( size( tidx_plot ) );
    t_plot          = T.Sam.t( tidx_plot ) / 1e+3;
    if isfield( T.Pulse, 't_NoRF' )
        for r = 1 : size( T.Pulse.t_NoRF, 1 )
            tidx_playRF( t_plot >= T.Pulse.t_NoRF(r,1) & t_plot <= T.Pulse.t_NoRF(r,2) ) = 0;
        end
    end
    
    %Figure preparation
    
    kx_plot_noRF = kx_plot;
    ky_plot_noRF = ky_plot;
    kz_plot_noRF = kz_plot;

    kx_plot_noRF( tidx_playRF == 1 ) = NaN;
    ky_plot_noRF( tidx_playRF == 1 ) = NaN;
    kz_plot_noRF( tidx_playRF == 1 ) = NaN;
    
    kx_plot( tidx_playRF == 0 ) = NaN;
    ky_plot( tidx_playRF == 0 ) = NaN;
    kz_plot( tidx_playRF == 0 ) = NaN;
    V( tidx_playRF == 0 )       = NaN;
    
    %plot colorcoded trajectory
    h = mesh([kx_plot(:), kx_plot(:)], [ky_plot(:), ky_plot(:)], [kz_plot(:), kz_plot(:)], [V V], ...
        'EdgeColor', 'interp', 'FaceColor', 'none', 'Linewidth', 1.5);
    set(gca,'Clim',[0, max(V(:))])
    
    %plot dashed line where no RF is played
    plot3( kx_plot_noRF, ky_plot_noRF, kz_plot_noRF, 'Linestyle', '-.', 'Linewidth',1.5,'Color',[0.4,0.4,0.4] )
    
    %plot control points
    hold on
    %gui_PlotControlPoints( T );

    box on
    axis on
    grid on
    axis square
    
    if all( T.Sam.kz == 0 )
        set(gca,'View',[0 90])
	else
        set(gca,'View',[-46 22])
    end
    
    set(gca,'Color',[0.941, 0.941, 0.941])
    
    Cmap = ColormapHot( 340 ) ;
    colormap( Cmap( 1:256,: ) )
    
    return
    
    %plot axis
    arrow([-kmax, 0.0, 0.0],[+kmax, 0.0, 0.0],'FaceColor',[0.6, 0.6, 0.6],'EdgeColor',[0.4, 0.4, 0.4],'Width',1.0,'Length',10,'TipAngle',15)
    arrow([0.0, -kmax, 0.0],[0.0, +kmax, 0.0],'FaceColor',[0.6, 0.6, 0.6],'EdgeColor',[0.4, 0.4, 0.4],'Width',1.0,'Length',10,'TipAngle',15)
    arrow([0.0, 0.0, -kmax],[0.0, 0.0, +kmax],'FaceColor',[0.6, 0.6, 0.6],'EdgeColor',[0.4, 0.4, 0.4],'Width',1.0,'Length',10,'TipAngle',15)
    
    set(gca,'Visible','off','YTickLabel','',...
    'XTickLabel','',...
    'Position',[0.00229573615681785 0.011 0.8216 0.9779],...
    'PlotBoxAspectRatio',[1 1 1.15384615384615],...
    'DataAspectRatio',[1 1 1],...
    'CameraViewAngle',5.88355943384202);

    colorbar('peer',gca,...
    [0.839016056082764 0.0331491712707182 0.0733641127539791 0.94475138121547],'YTick',0 : 4 : 14);

end

function h = gui_PlotControlPoints( T )

    %error handling
    if numel(T.kx) ~= numel(T.ky) || numel(T.kx) ~= numel(T.kz)
       error('kx, ky, and kz vectors must have same length!') 
    end
    
    %define type of trajectory 2D or 3D
    switch size( T.R, 2 )
        case 4
            DimType = 3;
        case 3
            DimType = 2;
    end
    
    if (numel(T.kx) ~= size(T.R,1) || size(T.R,2) ~= 4) && DimType == 3
       error('3D Slope constraint matrix has wrong dimension!') 
    end
    if (numel(T.kx) ~= size(T.R,1) || size(T.R,2) ~= 3) && DimType == 2
       error('2D Slope constraint matrix has wrong dimension!') 
    end

    %include first and last points, add NaN constraints
    kx  = T.kx;
    ky  = T.ky;
    kz  = T.kz;
    R   = T.R;
    
    %plot the kspace nodes to the current figure
    kmax        = max(abs([kx(:); ky(:); kz(:)]));
    ArrowLength	= 0.2 * kmax;
    
    h           = nan( 1,3 );
    
    switch DimType
        
        case 3
            % ============================================================
            %   T H R E E   D I M E N S I O N A L   T R A J E C T O R Y
            % ============================================================
            AnnotShift   = [kmax/30,kmax/30,kmax/30];
            for k = 1 : numel(kx)
                switch R(k,4)
                    case 3
                        %no slope constraint
                        %-------------------
                        h(1) = plot3(kx(k),ky(k),kz(k),'*','Marker','x','Markersize',7,'Linewidth',2,'Color','blue');
                        %text(kx(k),ky(k),kz(k),{sprintf('  %i',k),''},'HorizontalAlignment','left','FontSize',FontSize,'FontWeight',FontWeight,'Color','blue')
                        AddAnnotation( kx(k), ky(k), kz(k), AnnotShift, k, 'blue' )
                    case 2
                        %layer constraint
                        %----------------
                        plot3(kx(k),ky(k),kz(k),'-*','Marker','x','Markersize',7,'Linewidth',2,'Color',[1.0, 0.5490, 0.0])
                        %text(kx(k),ky(k),kz(k),{sprintf('  %i',k),''},'HorizontalAlignment','left','FontSize',FontSize,'FontWeight',FontWeight,'Color',[1.0, 0.5490, 0.0])
                        AddAnnotation( kx(k), ky(k), kz(k), AnnotShift, k, [1.0, 0.5490, 0.0] )
                        warning('Layer constraints are currently not supported, as they result in difficult control regions!')
                    case 1
                        %vector constraint
                        %-----------------
                        VNorm   = R(k,1:3)./norm(R(k,1:3));
                        line( [kx(k) + VNorm(1) * ArrowLength/2, kx(k) - VNorm(1) * ArrowLength/2], ...
                              [ky(k) + VNorm(2) * ArrowLength/2, ky(k) - VNorm(2) * ArrowLength/2], ...
                              [kz(k) + VNorm(3) * ArrowLength/2, kz(k) - VNorm(3) * ArrowLength/2],'Linewidth',2,'Color',[0 0.6 0])
                        h(2) = plot3(kx(k),ky(k),kz(k),'Marker','x','Markersize',7,'Linewidth',2,'Color',[0 0.6 0]);
                        %text(kx(k),ky(k),kz(k),{sprintf('  %i',k),''}, 'FontSize',FontSize,'FontWeight',FontWeight,...
                        %                         'HorizontalAlignment','right', 'VerticalAlignment','bottom','Color',[0 0.6 0])
                        AddAnnotation( kx(k), ky(k), kz(k), AnnotShift, k, [0 0.6 0] )
                    case 0
                        %fixed gradients
                        %---------------
                        h(3) = plot3(kx(k),ky(k),kz(k),'*','Marker','x','Markersize',7,'Linewidth',2,'Color','red');
                        %text(kx(k),ky(k),kz(k),sprintf('  %i',k),'HorizontalAlignment','left','FontSize',FontSize,'FontWeight',FontWeight,'Color','red')
                        AddAnnotation( kx(k), ky(k), kz(k), AnnotShift, k, 'red' )
                    otherwise
                        error('Undefined 3D-constraint ID detected (%i)!',R(k,4))
                end
            end
            
        case 2
            % =======================================================
            %   T W O   D I M E N S I O N A L   T R A J E C T O R Y
            % =======================================================
            AnnotShift   = [kmax/30,kmax/30,1];
           for k = 1 : numel(kx)
                switch R(k,3)
                    case 2
                        %no slope constraint
                        %-------------------
                        h(1) = plot3(kx(k),ky(k),kz(k),'Marker','x','Markersize',7,'Linewidth',2,'Color','blue');
                        %text(kx(k),ky(k),kz(k),{sprintf('  %i',k),''},'HorizontalAlignment','left','FontSize',FontSize,'FontWeight',FontWeight,'Color','blue')
                        AddAnnotation( kx(k), ky(k), kz(k), AnnotShift, k, 'blue' )
                    case 1
                        %vector constraint
                        %-----------------
                        VNorm   = R(k,1:2)./norm(R(k,1:2));
                        line( [kx(k) + VNorm(1) * ArrowLength/2, kx(k) - VNorm(1) * ArrowLength/2], ...
                              [ky(k) + VNorm(2) * ArrowLength/2, ky(k) - VNorm(2) * ArrowLength/2],'Linewidth',2,'Color',[0 0.6 0])
                        h(2) = plot3(kx(k),ky(k),kz(k),'Marker','x','Markersize',7,'Linewidth',2,'Color',[0 0.6 0]);
                        %text(kx(k),ky(k),kz(k),{sprintf('  %i',k),''}, 'FontSize',FontSize,'FontWeight',FontWeight,...
                        %                         'HorizontalAlignment','right', 'VerticalAlignment','bottom','Color',[0 0.6 0])
                       	AddAnnotation( kx(k), ky(k), kz(k), AnnotShift, k, [0 0.6 0] )
                    case 0
                        %fixed gradients
                        %---------------
                        h(3) = plot3(kx(k),ky(k),kz(k),'Marker','x','Markersize',7,'Linewidth',2,'Color','red');
                        %text(kx(k),ky(k),kz(k),{sprintf('  %i',k),''},'HorizontalAlignment','left','FontSize',FontSize,'FontWeight',FontWeight,'Color','red')
                        AddAnnotation( kx(k), ky(k), kz(k), AnnotShift, k, 'red' )
                    otherwise
                        error('Undefined 2D-constraint ID detected (%i)!',R(k,3))
                end
            end 
            
    end

end
    
function AddAnnotation( x, y, z, Shift, idx, color )

    FontWeight  = 'bold';
    FontSize    = 12;

    text('Parent',gca,'VerticalAlignment','bottom','String',sprintf('%i',idx),...
        'Position',[x + Shift(1), y + Shift(2), z + Shift(3)],...
        'HorizontalAlignment','center',...
        'FontSize',12,...
        'EdgeColor',[0 0 0],...
        'Color',color,...
        'Fontweight',FontWeight,...
        'Fontsize',FontSize,...
        'BackgroundColor',[1 1 1]);

end
    