function [ kmat, zeroCols_final, T ] = PrepareSystemMatrixInput( T, options, TypeString, CentralIntegral )
     
    % Prepare some data, that is needed to set up the system matrix
    % correctly. E.g., this includes 

    if nargin < 3
       TypeString = '';
    end
    
    if nargin < 4
        CentralIntegral = 0;
    end

    kx  = T.Pulse.kx(:);
    ky  = T.Pulse.ky(:);
    kz  = T.Pulse.kz(:);
    t   = T.Pulse.t(:);
    
    
    [ ~, IdxPlayRF_perchannel ] = GetZeroRFIndices( T, options.nchannel );
    
    %add one entry, always set to one
    kmat_idx        = IdxPlayRF_perchannel; %[IdxPlayRF_perchannel(1); IdxPlayRF_perchannel];
    %located position, where vector changes from 0 to 1
    %these elements need to be set to one
    kmat_idx( diff(kmat_idx) == 1 ) = 1;

    if CentralIntegral

        %get vector of correct integration in each node, except first and last
        %node
        t_cen   = 0.5 * (t(2:end) + t(1:end-1));

        kx_cen  = interp1( t, kx, t_cen, 'linear' );
        ky_cen  = interp1( t, ky, t_cen, 'linear' );
        kz_cen  = interp1( t, kz, t_cen, 'linear' );

        %for each point in kx, get the projection on the intregral line
        pj_x    = nan(size(kx,1) - 2, 1);
        pj_y    = nan(size(ky,1) - 2, 1);
        pj_z    = nan(size(kz,1) - 2, 1);

        for p = 1 : numel( pj_x )
            x   = [kx(p+1), ky(p+1), kz(p+1)];
            %discontinuities
            P1  = [kx_cen(p),   ky_cen(p),   kz_cen(p)];
            P2  = [kx_cen(p+1), ky_cen(p+1), kz_cen(p+1)];

            %get orthogonal projection to estimate position of Gaussian
            r0      = P1;
            u       = P2 - P1;
            Po      = r0 + ((x - r0) * u')/(u * u')  * u;

            pj_x(p)     = kx(p+1) - Po(1);
            pj_y(p)     = ky(p+1) - Po(2);
            pj_z(p)     = kz(p+1) - Po(3);

        end

        %shift points
        kx_start    = 0.5 * ( kx(1) + kx(2) );
        ky_start    = 0.5 * ( ky(1) + ky(2) );
        kz_start    = 0.5 * ( kz(1) + kz(2) );
        kx_end      = 0.5 * ( kx(end-1) + kx(end) );
        ky_end      = 0.5 * ( ky(end-1) + ky(end) );
        kz_end      = 0.5 * ( kz(end-1) + kz(end) );

        kx_ccor     = kx_cen(2:end-1) + 0.5 * (pj_x(1:end-1) + pj_x(2:end));
        ky_ccor     = ky_cen(2:end-1) + 0.5 * (pj_y(1:end-1) + pj_y(2:end));
        kz_ccor     = kz_cen(2:end-1) + 0.5 * (pj_z(1:end-1) + pj_z(2:end));
    
        %add simplifiyed first samples
        kx_ccor     = [ kx(1) + (kx(1) - kx_start); kx_start; kx_ccor; kx_end; kx(end) + (kx(end) - kx_end) ];
        ky_ccor     = [ ky(1) + (ky(1) - ky_start); ky_start; ky_ccor; ky_end; ky(end) + (ky(end) - ky_end) ];
        kz_ccor     = [ kz(1) + (kz(1) - kz_start); kz_start; kz_ccor; kz_end; kz(end) + (kz(end) - kz_end) ];
    
    else
        kx_ccor  = kx; %[kx; kx(end)];
        ky_ccor  = ky; %[ky; ky(end)];
        kz_ccor  = kz; %[kz; kz(end)];
    end
    
    kmat_idx_cumsum     = cumsum( kmat_idx );
    
    %set a vector, that can be used to remove the invalid columns in the
    %system matrix
    zeroCols        = zeros( size( kmat_idx ) );
    %always a zero col at the end
    zeroCols(end)   = 1;
    zeroCols        = double( zeroCols | [diff( kmat_idx ) < 0; 0] );

    zeroCols_idx    = unique( kmat_idx_cumsum( zeroCols == 1 ) );
    
    RFfull_idx      = kmat_idx_cumsum; %cumsum( double( kmat_idx | zeroCols ) );
    
    %finally define the correct kmax
    if ~strcmp( TypeString, 'full' )
        kmat    = [kx_ccor( kmat_idx == 1 ), ky_ccor( kmat_idx == 1 ), kz_ccor( kmat_idx == 1 )];
    else
        kmat    = [kx_ccor(:), ky_ccor(:), kz_ccor(:)];
    end
    
    zeroCols_bin    = zeros( size( kmat, 1 ), 1 );
    zeroCols_bin( zeroCols_idx ) = 1;
    
    zeroCols_final  = repmat( zeroCols_idx, 1, options.nchannel ) + ( repmat( 0 : options.nchannel - 1, numel(zeroCols_idx), 1 ) ) * size( kmat, 1 );
    zeroCols_final  = zeroCols_final(:);
    
    RFfull_idx      = repmat( RFfull_idx, 1, options.nchannel ) + ( repmat( 0 : options.nchannel - 1, numel(RFfull_idx), 1 ) ) * size( kmat, 1 );
    RFfull_idx      = RFfull_idx(:);
    
    RFsparse_idx    = find( diff( RFfull_idx ) >= 1 );
   
    
 	% store results in T
    %--------------------
    T.Pulse.MatSize_perChannel   = size( kmat, 1 );
    T.Pulse.MatSize              = size( kmat, 1 ) * options.nchannel;
    
    T.Pulse.RFfullLength_perChannel   = numel( kx_ccor );
    T.Pulse.RFfullLength              = numel( kx_ccor ) * options.nchannel;
    
    T.Pulse.zeroCols_bin         = zeroCols_bin;
    T.Pulse.zeroCols             = zeroCols_final;
    T.Pulse.zeroCols_perChannel	= zeroCols_idx;
    T.Pulse.NumCols              = size( kmat, 1 ) * options.nchannel;
    
    T.Pulse.RFfull_NonZeros              = repmat( kmat_idx, options.nchannel, 1);
    T.Pulse.RFfull_NonZeros_perChannel   = kmat_idx;
    T.Pulse.RFsparse_NonZeros   	= [];
    T.Pulse.RFfull_idx           = RFfull_idx;
    T.Pulse.RFsparse_idx         = RFsparse_idx;

    T.Pulse.NumChannel           = options.nchannel;
    
end