function [ tlim, gradIn, gradOut, gradIn_In, gradIn_Out, gradOut_In, gradOut_Out] = FastTimeOptGrad_2D_deriv2( GxIn, GxOut, GyIn, GyOut, dkx, dky, Gmax, Smax, OptsVector )

    %#codegen
    %this method is used to solver a local trajectory segment in 2D
    %dimension, based on the gradient vectors in the control points. tlim
    %contains the smallest possible duration of the local segment
    %satisfying the gradient vectors as well as the Gradient constraints
    %Gmax and Smax. Additionall, this method returns the gradients of the
    %duration tlim with respect to the gradient vectors

    %check for correct input values
    if abs(GxIn) > Gmax
        GxIn = Gmax * sign(GxIn);
    end
    if abs(GyIn) > Gmax
        GyIn = Gmax * sign(GyIn);
    end
    if abs(GxOut) > Gmax
        GxOut = Gmax * sign(GxOut);
    end
    if abs(GyOut) > Gmax
        GyOut = Gmax * sign(GyOut);
    end
    
    %valid time ranges in X
    [ lowBoundsX, uppBoundsX ] = GetValidTimeRanges( GxIn, GxOut, dkx, Gmax, Smax );
    %valid time ranges in Y
    [ lowBoundsY, uppBoundsY ] = GetValidTimeRanges( GyIn, GyOut, dky, Gmax, Smax );
        
    if OptsVector( 1 )
        %remove parts of range making the full range incontinous
       [ lowBoundsX, uppBoundsX ] = MakeRangeContinous( lowBoundsX, uppBoundsX ); 
       [ lowBoundsY, uppBoundsY ] = MakeRangeContinous( lowBoundsY, uppBoundsY ); 
    end

    %get tlim
    [tlim, gradIn, gradOut, gradIn_In, gradIn_Out, gradOut_In, gradOut_Out] = GetMinimumTime2D( lowBoundsX, uppBoundsX, lowBoundsY, uppBoundsY, OptsVector );
   
end

function [ lowBoundsX, uppBoundsX ] = GetValidTimeRanges( GnIn, GnOut, dkn, Gmax, Smax )
    %Gamma       = 2.675222005e+2;
    Gamma   = 42.5775;
    %Gamma = 1;
    dknGamma    = dkn/Gamma;

    %final ranges
    %up to 5 overall ranges might be given for the solution, 2 ranges for the
    %fast convex or concave shape, 1 for the linear shape and 2 for the slow
    %convex or concave shape

    %R1_lowBound     = NaN;
    %R1_uppBound     = NaN;
    
    R2_lowBound     = NaN;
    R2_uppBound     = NaN;
    
    R3_lowBound     = NaN;
    R3_uppBound     = NaN;
    
    R4_lowBound     = NaN;
    R4_uppBound     = NaN;
    
    R5_lowBound     = NaN;
    R5_uppBound     = NaN;

    t_const     = abs(GnIn-GnOut)/Smax;
    I_const 	= (sign(GnIn-GnOut)*(GnIn^2-GnOut^2)*(1.0/2.0))/Smax;
    I_rest      = -I_const+dknGamma;
    
    %special case, GnIn =0, GnOut = 0, dkn = 0
    if GnIn == 0 && GnOut == 0 && dkn == 0
        lowBoundsX     = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0];
        uppBoundsX     = [Inf, NaN, NaN, NaN, NaN, NaN, NaN];
        return
    end
    
    %get adjusted sign parameter
  	AdjustedSign = sign(GnIn - GnOut);
    if AdjustedSign == 0
        %in case AdjustedSign == 0, i.e. GnIn == GnOut, we have to decide, 
        %as how to treat the problem 
        switch sign(I_rest)
            case +1
                AdjustedSign = -1; 
            case -1
                AdjustedSign = +1;
            otherwise
                AdjustedSign = +1; 
        end
        
    end
    
 	%get derivative of t_const_grad with respect to GnIn and GnOut
    if      AdjustedSign == +1
        %t_const  	= +(GnIn - GnOut)/Smax;
        t_const_gradIn      = 1.0/Smax;
        t_const_gradOut     = -1.0/Smax;
        
        %Set correct second derivatives, used for analytical Hessian
        t_const_gradIn_In        = 0.0;
        t_const_gradIn_Out       = 0.0;
        t_const_gradOut_In       = 0.0;
        t_const_gradOut_Out      = 0.0;
        
    else   %AdjustedSign == -1
        %t_const  	= -(GnIn - GnOut)/Smax;
        t_const_gradIn      = -1.0/Smax;
        t_const_gradOut     = 1.0/Smax;
        
        %Set correct second derivatives, used for analytical Hessian
        t_const_gradIn_In        = 0.0;
        t_const_gradIn_Out       = 0.0;
        t_const_gradOut_In       = 0.0;
        t_const_gradOut_Out      = 0.0;
    end

    %constant integral given be the connection of GnIn and GnOut
    switch AdjustedSign
        case -1
            %GnIn <= GnOut --> Increasing Shape

            %in the GnIn <= GnOut, the problem cannot be solved by the mono.
            %decreasing gradient type
            F2_lowBound         = NaN;
            F2_uppBound         = NaN;
            
            F2_lowBound_gradIn 	= NaN;
            F2_uppBound_gradIn 	= NaN;
            F2_lowBound_gradOut	= NaN;
            F2_uppBound_gradOut	= NaN;
            
            %Set correct second derivatives, used for analytical Hessian
            F2_lowBound_gradIn_In 	= NaN;
            F2_lowBound_gradIn_Out 	= NaN;
            F2_uppBound_gradIn_In 	= NaN;
            F2_uppBound_gradIn_Out 	= NaN;
            F2_lowBound_gradOut_In	= NaN;
            F2_lowBound_gradOut_Out	= NaN;
            F2_uppBound_gradOut_In	= NaN;
            F2_uppBound_gradOut_Out	= NaN;

            %depending on the signs of GnIn and GnOut, the range of the mono.
            %increasing gradient type is defined
            GnIn_Bound      = I_rest / GnIn;
            GnOut_Bound     = I_rest / GnOut;
            
            if isnan(GnIn_Bound)
                GnIn_Bound = 0.0;
            end
            if isnan(GnOut_Bound)
                GnOut_Bound = 0.0;
            end

            if      sign(GnIn_Bound) == +1 && sign(GnOut_Bound) == +1
                %F1_lowBound     = min([GnIn_Bound, GnOut_Bound]) + t_const;
                %F1_uppBound     = max([GnIn_Bound, GnOut_Bound]) + t_const;
                if      GnIn_Bound < GnOut_Bound
                    %set bounds
                    F1_lowBound 	= GnIn_Bound+t_const;
                    F1_uppBound     = GnOut_Bound+t_const;
                    
                    %Calculate gradients
                    if     AdjustedSign == +1
                        F1_lowBound_gradIn  = t_const_gradIn-1.0/Smax-1.0/GnIn^2*(dknGamma-(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        F1_uppBound_gradIn  = t_const_gradIn-GnIn/(GnOut*Smax);
                        F1_lowBound_gradOut = t_const_gradOut+GnOut/(GnIn*Smax);
                        F1_uppBound_gradOut = t_const_gradOut+1.0/Smax-1.0/GnOut^2*(dknGamma-(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        
                        %Set correct second derivatives, used for analytical Hessian
                        F1_lowBound_gradIn_In    = (1.0/GnIn^3*(Smax*dknGamma*2.0+GnOut^2+GnIn^3*Smax*t_const_gradIn_In))/Smax;
                        F1_lowBound_gradIn_Out   = t_const_gradIn_Out-(1.0/GnIn^2*GnOut)/Smax;
                        F1_uppBound_gradIn_In    = t_const_gradIn_In-1.0/(GnOut*Smax);
                        F1_uppBound_gradIn_Out   = t_const_gradIn_Out+(GnIn*1.0/GnOut^2)/Smax;
                        F1_lowBound_gradOut_In   = t_const_gradOut_In-(1.0/GnIn^2*GnOut)/Smax;
                        F1_lowBound_gradOut_Out  = t_const_gradOut_Out+1.0/(GnIn*Smax);
                        F1_uppBound_gradOut_In   = t_const_gradOut_In+(GnIn*1.0/GnOut^2)/Smax;
                        F1_uppBound_gradOut_Out  = (1.0/GnOut^3*(Smax*dknGamma*2.0-GnIn^2+GnOut^3*Smax*t_const_gradOut_Out))/Smax;
                        
                    else  %AdjustedSign == -1
                        
                        F1_lowBound_gradIn  = t_const_gradIn+1.0/Smax-1.0/GnIn^2*(dknGamma+(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        F1_uppBound_gradIn  = t_const_gradIn+GnIn/(GnOut*Smax);
                        F1_lowBound_gradOut = t_const_gradOut-GnOut/(GnIn*Smax);
                        F1_uppBound_gradOut = t_const_gradOut-1.0/Smax-1.0/GnOut^2*(dknGamma+(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        
                        %Set correct second derivatives, used for analytical Hessian
                        F1_lowBound_gradIn_In    = (1.0/GnIn^3*(Smax*dknGamma*2.0-GnOut^2+GnIn^3*Smax*t_const_gradIn_In))/Smax;
                        F1_lowBound_gradIn_Out   = t_const_gradIn_Out+(1.0/GnIn^2*GnOut)/Smax;
                        F1_uppBound_gradIn_In    = t_const_gradIn_In+1.0/(GnOut*Smax);
                        F1_uppBound_gradIn_Out   = t_const_gradIn_Out-(GnIn*1.0/GnOut^2)/Smax;
                        F1_lowBound_gradOut_In   = t_const_gradOut_In+(1.0/GnIn^2*GnOut)/Smax;
                        F1_lowBound_gradOut_Out  = t_const_gradOut_Out-1.0/(GnIn*Smax);
                        F1_uppBound_gradOut_In   = t_const_gradOut_In-(GnIn*1.0/GnOut^2)/Smax;
                        F1_uppBound_gradOut_Out  = (1.0/GnOut^3*(Smax*dknGamma*2.0+GnIn^2+GnOut^3*Smax*t_const_gradOut_Out))/Smax;
                        
                    end
                    
                else   %GnIn_Bound >= GnOut_Bound
                    %set bounds
                    F1_lowBound 	= GnOut_Bound+t_const;
                    F1_uppBound     = GnIn_Bound+t_const;
                    
                    %Calculate gradients
                    if     AdjustedSign == +1
                        
                        F1_lowBound_gradIn  = t_const_gradIn-GnIn/(GnOut*Smax);
                        F1_uppBound_gradIn  = t_const_gradIn-1.0/Smax-1.0/GnIn^2*(dknGamma-(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        F1_lowBound_gradOut = t_const_gradOut+1.0/Smax-1.0/GnOut^2*(dknGamma-(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        F1_uppBound_gradOut = t_const_gradOut+GnOut/(GnIn*Smax);
                        
                        %Set correct second derivatives, used for analytical Hessian
                        F1_lowBound_gradIn_In    = t_const_gradIn_In-1.0/(GnOut*Smax);
                        F1_lowBound_gradIn_Out   = t_const_gradIn_Out+(GnIn*1.0/GnOut^2)/Smax;
                        F1_uppBound_gradIn_In    = (1.0/GnIn^3*(Smax*dknGamma*2.0+GnOut^2+GnIn^3*Smax*t_const_gradIn_In))/Smax;
                        F1_uppBound_gradIn_Out   = t_const_gradIn_Out-(1.0/GnIn^2*GnOut)/Smax;
                        F1_lowBound_gradOut_In   = t_const_gradOut_In+(GnIn*1.0/GnOut^2)/Smax;
                        F1_lowBound_gradOut_Out  = (1.0/GnOut^3*(Smax*dknGamma*2.0-GnIn^2+GnOut^3*Smax*t_const_gradOut_Out))/Smax;
                        F1_uppBound_gradOut_In   = t_const_gradOut_In-(1.0/GnIn^2*GnOut)/Smax;
                        F1_uppBound_gradOut_Out  = t_const_gradOut_Out+1.0/(GnIn*Smax);
                        
                    else  %AdjustedSign == -1
                        F1_lowBound_gradIn  = t_const_gradIn+GnIn/(GnOut*Smax);
                        F1_uppBound_gradIn  = t_const_gradIn+1.0/Smax-1.0/GnIn^2*(dknGamma+(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        F1_lowBound_gradOut = t_const_gradOut-1.0/Smax-1.0/GnOut^2*(dknGamma+(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        F1_uppBound_gradOut = t_const_gradOut-GnOut/(GnIn*Smax);
                        
                        %Set correct second derivatives, used for analytical Hessian
                        F1_lowBound_gradIn_In    = t_const_gradIn_In+1.0/(GnOut*Smax);
                        F1_lowBound_gradIn_Out   = t_const_gradIn_Out-(GnIn*1.0/GnOut^2)/Smax;
                        F1_uppBound_gradIn_In    = (1.0/GnIn^3*(Smax*dknGamma*2.0-GnOut^2+GnIn^3*Smax*t_const_gradIn_In))/Smax;
                        F1_uppBound_gradIn_Out   = t_const_gradIn_Out+(1.0/GnIn^2*GnOut)/Smax;
                        F1_lowBound_gradOut_In   = t_const_gradOut_In-(GnIn*1.0/GnOut^2)/Smax;
                        F1_lowBound_gradOut_Out  = (1.0/GnOut^3*(Smax*dknGamma*2.0+GnIn^2+GnOut^3*Smax*t_const_gradOut_Out))/Smax;
                        F1_uppBound_gradOut_In   = t_const_gradOut_In+(1.0/GnIn^2*GnOut)/Smax;
                        F1_uppBound_gradOut_Out  = t_const_gradOut_Out-1.0/(GnIn*Smax);
                    end
                end
                
            elseif  sign(GnIn_Bound) == +1 && sign(GnOut_Bound) == -1
                F1_lowBound         = GnIn_Bound+t_const;
                if     AdjustedSign == +1
                    
                    F1_lowBound_gradIn  = t_const_gradIn-1.0/Smax-1.0/GnIn^2*(dknGamma-(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                    F1_lowBound_gradOut = t_const_gradOut+GnOut/(GnIn*Smax);
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F1_lowBound_gradIn_In    = (1.0/GnIn^3*(Smax*dknGamma*2.0+GnOut^2+GnIn^3*Smax*t_const_gradIn_In))/Smax;
                    F1_lowBound_gradIn_Out   = t_const_gradIn_Out-(1.0/GnIn^2*GnOut)/Smax;
                    F1_lowBound_gradOut_In   = t_const_gradOut_In-(1.0/GnIn^2*GnOut)/Smax;
                    F1_lowBound_gradOut_Out  = t_const_gradOut_Out+1.0/(GnIn*Smax);
                    
                else  %AdjustedSign == -1
                    
                    F1_lowBound_gradIn  = t_const_gradIn+1.0/Smax-1.0/GnIn^2*(dknGamma+(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                    F1_lowBound_gradOut = t_const_gradOut-GnOut/(GnIn*Smax);
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F1_lowBound_gradIn_In    = (1.0/GnIn^3*(Smax*dknGamma*2.0-GnOut^2+GnIn^3*Smax*t_const_gradIn_In))/Smax;
                    F1_lowBound_gradIn_Out   = t_const_gradIn_Out+(1.0/GnIn^2*GnOut)/Smax;
                    F1_lowBound_gradOut_In   = t_const_gradOut_In+(1.0/GnIn^2*GnOut)/Smax;
                    F1_lowBound_gradOut_Out  = t_const_gradOut_Out-1.0/(GnIn*Smax);
                    
                end
                
                F1_uppBound         = Inf;
                F1_uppBound_gradIn 	= NaN;
                F1_uppBound_gradOut	= NaN;
                
                %Set correct second derivatives, used for analytical Hessian
                F1_uppBound_gradIn_In    = NaN;
                F1_uppBound_gradIn_Out   = NaN;
                F1_uppBound_gradOut_In   = NaN;
                F1_uppBound_gradOut_Out  = NaN;

            elseif  sign(GnIn_Bound) == -1 && sign(GnOut_Bound) == +1
                F1_lowBound     = GnOut_Bound+t_const;
                %Calculate gradients
                if     AdjustedSign == +1
                    F1_lowBound_gradIn  = t_const_gradIn-GnIn/(GnOut*Smax);
                    F1_lowBound_gradOut = t_const_gradOut+1.0/Smax-1.0/GnOut^2*(dknGamma-(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F1_lowBound_gradIn_In    = t_const_gradIn_In-1.0/(GnOut*Smax);
                    F1_lowBound_gradIn_Out   = t_const_gradIn_Out+(GnIn*1.0/GnOut^2)/Smax;
                    F1_lowBound_gradOut_In   = t_const_gradOut_In+(GnIn*1.0/GnOut^2)/Smax;
                    F1_lowBound_gradOut_Out  = (1.0/GnOut^3*(Smax*dknGamma*2.0-GnIn^2+GnOut^3*Smax*t_const_gradOut_Out))/Smax;

                else  %AdjustedSign == -1
                    F1_lowBound_gradIn  = t_const_gradIn+GnIn/(GnOut*Smax);
                    F1_lowBound_gradOut = t_const_gradOut-1.0/Smax-1.0/GnOut^2*(dknGamma+(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F1_lowBound_gradIn_In    = t_const_gradIn_In+1.0/(GnOut*Smax);
                    F1_lowBound_gradIn_Out   = t_const_gradIn_Out-(GnIn*1.0/GnOut^2)/Smax;
                    F1_lowBound_gradOut_In   = t_const_gradOut_In-(GnIn*1.0/GnOut^2)/Smax;
                    F1_lowBound_gradOut_Out  = (1.0/GnOut^3*(Smax*dknGamma*2.0+GnIn^2+GnOut^3*Smax*t_const_gradOut_Out))/Smax;

                end
                
                F1_uppBound         = Inf;
                F1_uppBound_gradIn 	= NaN;
                F1_uppBound_gradOut	= NaN;
                
                %Set correct second derivatives, used for analytical Hessian
                F1_uppBound_gradIn_In    = NaN;
                F1_uppBound_gradIn_Out   = NaN;
                F1_uppBound_gradOut_In   = NaN;
                F1_uppBound_gradOut_Out  = NaN;

            elseif  sign(GnIn_Bound) == -1 && sign(GnOut_Bound) == -1
                
                F1_lowBound         = NaN;
                F1_uppBound         = NaN;
                F1_lowBound_gradIn  = NaN;
                F1_uppBound_gradIn 	= NaN;
                F1_lowBound_gradOut	= NaN;
                F1_uppBound_gradOut	= NaN;
                
                %Set correct second derivatives, used for analytical Hessian
                F1_lowBound_gradIn_In    = NaN;
                F1_lowBound_gradIn_Out   = NaN;
                F1_uppBound_gradIn_In    = NaN;
                F1_uppBound_gradIn_Out   = NaN;
                F1_lowBound_gradOut_In   = NaN;
                F1_lowBound_gradOut_Out  = NaN;
                F1_uppBound_gradOut_In   = NaN;
                F1_uppBound_gradOut_Out  = NaN;

            else
                if sign(GnIn) < sign(GnOut) || sign(GnIn) > sign(GnOut)
                    
                    F1_lowBound         = t_const;
                    F1_lowBound_gradIn  = t_const_gradIn;
                    F1_lowBound_gradOut = t_const_gradOut;
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F1_lowBound_gradIn_In    = t_const_gradIn_In;
                    F1_lowBound_gradIn_Out   = t_const_gradIn_Out;
                    F1_lowBound_gradOut_In   = t_const_gradOut_In;
                    F1_lowBound_gradOut_Out  = t_const_gradOut_Out;
                    
                    F1_uppBound         = Inf;
                    F1_uppBound_gradIn 	= NaN;
                    F1_uppBound_gradOut	= NaN;
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F1_uppBound_gradIn_In    = NaN;
                    F1_uppBound_gradIn_Out   = NaN;
                    F1_uppBound_gradOut_In   = NaN;
                    F1_uppBound_gradOut_Out  = NaN;
                    
                else
                    
                    F1_lowBound         = t_const;
                    F1_lowBound_gradIn  = t_const_gradIn;
                    F1_lowBound_gradOut = t_const_gradOut;
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F1_lowBound_gradIn_In    = t_const_gradIn_In;
                    F1_lowBound_gradIn_Out   = t_const_gradIn_Out;
                    F1_lowBound_gradOut_In   = t_const_gradOut_In;
                    F1_lowBound_gradOut_Out  = t_const_gradOut_Out;
                    
                    F1_uppBound         = t_const;
                    F1_uppBound_gradIn  = t_const_gradIn;
                    F1_uppBound_gradOut = t_const_gradOut;
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F1_uppBound_gradIn_In    = t_const_gradIn_In;
                    F1_uppBound_gradIn_Out   = t_const_gradIn_Out;
                    F1_uppBound_gradOut_In   = t_const_gradOut_In;
                    F1_uppBound_gradOut_Out  = t_const_gradOut_Out;
                    
                end
            %else
                %error('Undefined case: \n  GnIn = %.4f \n GnOut = %.4f \n   dkn = %.4f',GnIn,GnOut,dkn)
            end

        otherwise
            %GnIn >= GnOut --> Decreasing Shape
            I_rest   = -I_const+dknGamma;

            %in the GnIn <= GnOut, the problem cannot be solved by the mono.
            %decreasing gradient type
            F1_lowBound         = NaN;
            F1_uppBound         = NaN;
            F1_lowBound_gradIn	= NaN;
            F1_uppBound_gradIn	= NaN;
            F1_lowBound_gradOut	= NaN;
            F1_uppBound_gradOut	= NaN;
           
            %Set correct second derivatives, used for analytical Hessian
            F1_lowBound_gradIn_In    = NaN;
            F1_lowBound_gradIn_Out   = NaN;
            F1_uppBound_gradIn_In    = NaN;
            F1_uppBound_gradIn_Out   = NaN;
            F1_lowBound_gradOut_In   = NaN;
            F1_lowBound_gradOut_Out  = NaN;
            F1_uppBound_gradOut_In   = NaN;
            F1_uppBound_gradOut_Out  = NaN;

            %depending on the signs of GnIn and GnOut, the range of the mono.
            %increasing gradient type is defined
            GnIn_Bound      = I_rest / GnIn;
            GnOut_Bound     = I_rest / GnOut;

            if      sign(GnIn_Bound) == +1 && sign(GnOut_Bound) == +1
                %F2_lowBound     = min([GnIn_Bound, GnOut_Bound]) + t_const;
                %F2_uppBound     = max([GnIn_Bound, GnOut_Bound]) + t_const;
                if      GnIn_Bound < GnOut_Bound
                    
                    %set bounds
                    F2_lowBound 	= GnIn_Bound+t_const;
                    F2_uppBound     = GnOut_Bound+t_const;
                    
                    %Calculate gradients
                    if     AdjustedSign == +1
                        
                        F2_lowBound_gradIn  = t_const_gradIn-1.0/Smax-1.0/GnIn^2*(dknGamma-(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        F2_uppBound_gradIn  = t_const_gradIn-GnIn/(GnOut*Smax);
                        F2_lowBound_gradOut = t_const_gradOut+GnOut/(GnIn*Smax);
                        F2_uppBound_gradOut = t_const_gradOut+1.0/Smax-1.0/GnOut^2*(dknGamma-(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        
                        %Set correct second derivatives, used for analytical Hessian
                        F2_lowBound_gradIn_In    = (1.0/GnIn^3*(Smax*dknGamma*2.0+GnOut^2+GnIn^3*Smax*t_const_gradIn_In))/Smax;
                        F2_lowBound_gradIn_Out   = t_const_gradIn_Out-(1.0/GnIn^2*GnOut)/Smax;
                        F2_uppBound_gradIn_In    = t_const_gradIn_In-1.0/(GnOut*Smax);
                        F2_uppBound_gradIn_Out   = t_const_gradIn_Out+(GnIn*1.0/GnOut^2)/Smax;
                        F2_lowBound_gradOut_In   = t_const_gradOut_In-(1.0/GnIn^2*GnOut)/Smax;
                        F2_lowBound_gradOut_Out  = t_const_gradOut_Out+1.0/(GnIn*Smax);
                        F2_uppBound_gradOut_In   = t_const_gradOut_In+(GnIn*1.0/GnOut^2)/Smax;
                        F2_uppBound_gradOut_Out  = (1.0/GnOut^3*(Smax*dknGamma*2.0-GnIn^2+GnOut^3*Smax*t_const_gradOut_Out))/Smax;
                    
                    else  %AdjustedSign == -1
                        
                        F2_lowBound_gradIn  = t_const_gradIn+1.0/Smax-1.0/GnIn^2*(dknGamma+(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        F2_uppBound_gradIn  = t_const_gradIn+GnIn/(GnOut*Smax);
                        F2_lowBound_gradOut = t_const_gradOut-GnOut/(GnIn*Smax);
                        F2_uppBound_gradOut = t_const_gradOut-1.0/Smax-1.0/GnOut^2*(dknGamma+(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        
                        %Set correct second derivatives, used for analytical Hessian
                        F2_lowBound_gradIn_In    = (1.0/GnIn^3*(Smax*dknGamma*2.0-GnOut^2+GnIn^3*Smax*t_const_gradIn_In))/Smax;
                        F2_lowBound_gradIn_Out   = t_const_gradIn_Out+(1.0/GnIn^2*GnOut)/Smax;
                        F2_uppBound_gradIn_In    = t_const_gradIn_In+1.0/(GnOut*Smax);
                        F2_uppBound_gradIn_Out   = t_const_gradIn_Out-(GnIn*1.0/GnOut^2)/Smax;
                        F2_lowBound_gradOut_In   = t_const_gradOut_In+(1.0/GnIn^2*GnOut)/Smax;
                        F2_lowBound_gradOut_Out  = t_const_gradOut_Out-1.0/(GnIn*Smax);
                        F2_uppBound_gradOut_In   = t_const_gradOut_In-(GnIn*1.0/GnOut^2)/Smax;
                        F2_uppBound_gradOut_Out  = (1.0/GnOut^3*(Smax*dknGamma*2.0+GnIn^2+GnOut^3*Smax*t_const_gradOut_Out))/Smax;

                    end
            
                else   %GnIn_Bound >= GnOut_Bound
                    
                    %set bounds
                    F2_lowBound 	= GnOut_Bound+t_const;
                    F2_uppBound     = GnIn_Bound+t_const;
                    
                    %Calculate gradients
                    if     AdjustedSign == +1
                        
                        F2_lowBound_gradIn  = t_const_gradIn-GnIn/(GnOut*Smax);
                        F2_uppBound_gradIn  = t_const_gradIn-1.0/Smax-1.0/GnIn^2*(dknGamma-(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        F2_lowBound_gradOut = t_const_gradOut+1.0/Smax-1.0/GnOut^2*(dknGamma-(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        F2_uppBound_gradOut = t_const_gradOut+GnOut/(GnIn*Smax);
                        
                        %Set correct second derivatives, used for analytical Hessian
                        F2_lowBound_gradIn_In    = t_const_gradIn_In-1.0/(GnOut*Smax);
                        F2_lowBound_gradIn_Out   = t_const_gradIn_Out+(GnIn*1.0/GnOut^2)/Smax;
                        F2_uppBound_gradIn_In    = (1.0/GnIn^3*(Smax*dknGamma*2.0+GnOut^2+GnIn^3*Smax*t_const_gradIn_In))/Smax;
                        F2_uppBound_gradIn_Out   = t_const_gradIn_Out-(1.0/GnIn^2*GnOut)/Smax;
                        F2_lowBound_gradOut_In   = t_const_gradOut_In+(GnIn*1.0/GnOut^2)/Smax;
                        F2_lowBound_gradOut_Out  = (1.0/GnOut^3*(Smax*dknGamma*2.0-GnIn^2+GnOut^3*Smax*t_const_gradOut_Out))/Smax;
                        F2_uppBound_gradOut_In   = t_const_gradOut_In-(1.0/GnIn^2*GnOut)/Smax;
                        F2_uppBound_gradOut_Out  = t_const_gradOut_Out+1.0/(GnIn*Smax);

                    else  %AdjustedSign == -1
                        
                        F2_lowBound_gradIn  = t_const_gradIn+GnIn/(GnOut*Smax);
                        F2_uppBound_gradIn  = t_const_gradIn+1.0/Smax-1.0/GnIn^2*(dknGamma+(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        F2_lowBound_gradOut = t_const_gradOut-1.0/Smax-1.0/GnOut^2*(dknGamma+(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                        F2_uppBound_gradOut = t_const_gradOut-GnOut/(GnIn*Smax);
                        
                        %Set correct second derivatives, used for analytical Hessian
                        F2_lowBound_gradIn_In    = t_const_gradIn_In+1.0/(GnOut*Smax);
                        F2_lowBound_gradIn_Out   = t_const_gradIn_Out-(GnIn*1.0/GnOut^2)/Smax;
                        F2_uppBound_gradIn_In    = (1.0/GnIn^3*(Smax*dknGamma*2.0-GnOut^2+GnIn^3*Smax*t_const_gradIn_In))/Smax;
                        F2_uppBound_gradIn_Out   = t_const_gradIn_Out+(1.0/GnIn^2*GnOut)/Smax;
                        F2_lowBound_gradOut_In   = t_const_gradOut_In-(GnIn*1.0/GnOut^2)/Smax;
                        F2_lowBound_gradOut_Out  = (1.0/GnOut^3*(Smax*dknGamma*2.0+GnIn^2+GnOut^3*Smax*t_const_gradOut_Out))/Smax;
                        F2_uppBound_gradOut_In   = t_const_gradOut_In+(1.0/GnIn^2*GnOut)/Smax;
                        F2_uppBound_gradOut_Out  = t_const_gradOut_Out-1.0/(GnIn*Smax);
                        
                    end
                end

            elseif  sign(GnIn_Bound) == +1 && sign(GnOut_Bound) == -1
                F2_lowBound     = GnIn_Bound+t_const;
                %Calculate gradients
                if     AdjustedSign == +1
                    
                    F2_lowBound_gradIn  = t_const_gradIn-1.0/Smax-1.0/GnIn^2*(dknGamma-(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                    F2_lowBound_gradOut = t_const_gradOut+GnOut/(GnIn*Smax);
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F2_lowBound_gradIn_In    = (1.0/GnIn^3*(Smax*dknGamma*2.0+GnOut^2+GnIn^3*Smax*t_const_gradIn_In))/Smax;
                    F2_lowBound_gradIn_Out   = t_const_gradIn_Out-(1.0/GnIn^2*GnOut)/Smax;
                    F2_lowBound_gradOut_In   = t_const_gradOut_In-(1.0/GnIn^2*GnOut)/Smax;
                    F2_lowBound_gradOut_Out  = t_const_gradOut_Out+1.0/(GnIn*Smax);
                    
                else  %AdjustedSign == -1
                    
                    F2_lowBound_gradIn  = t_const_gradIn+1.0/Smax-1.0/GnIn^2*(dknGamma+(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                    F2_lowBound_gradOut = t_const_gradOut-GnOut/(GnIn*Smax);
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F2_lowBound_gradIn_In    = (1.0/GnIn^3*(Smax*dknGamma*2.0-GnOut^2+GnIn^3*Smax*t_const_gradIn_In))/Smax;
                    F2_lowBound_gradIn_Out   = t_const_gradIn_Out+(1.0/GnIn^2*GnOut)/Smax;
                    F2_lowBound_gradOut_In   = t_const_gradOut_In+(1.0/GnIn^2*GnOut)/Smax;
                    F2_lowBound_gradOut_Out  = t_const_gradOut_Out-1.0/(GnIn*Smax);
                    
                end
                
                F2_uppBound         = Inf;
                F2_uppBound_gradIn 	= NaN;
                F2_uppBound_gradOut	= NaN;
                
                %Set correct second derivatives, used for analytical Hessian
                F2_uppBound_gradIn_In    = NaN;
                F2_uppBound_gradIn_Out   = NaN;
                F2_uppBound_gradOut_In   = NaN;
                F2_uppBound_gradOut_Out  = NaN;

            elseif  sign(GnIn_Bound) == -1 && sign(GnOut_Bound) == +1
                F2_lowBound     = GnOut_Bound+t_const;
                %Calculate gradients
                if     AdjustedSign == +1
                    
                    F2_lowBound_gradIn  = t_const_gradIn-GnIn/(GnOut*Smax);
                    F2_lowBound_gradOut = t_const_gradOut+1.0/Smax-1.0/GnOut^2*(dknGamma-(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F2_lowBound_gradIn_In    = t_const_gradIn_In-1.0/(GnOut*Smax);
                    F2_lowBound_gradIn_Out   = t_const_gradIn_Out+(GnIn*1.0/GnOut^2)/Smax;
                    F2_lowBound_gradOut_In   = t_const_gradOut_In+(GnIn*1.0/GnOut^2)/Smax;
                    F2_lowBound_gradOut_Out  = (1.0/GnOut^3*(Smax*dknGamma*2.0-GnIn^2+GnOut^3*Smax*t_const_gradOut_Out))/Smax;
                    
                else  %AdjustedSign == -1
                    
                    F2_lowBound_gradIn  = t_const_gradIn+GnIn/(GnOut*Smax);
                    F2_lowBound_gradOut = t_const_gradOut-1.0/Smax-1.0/GnOut^2*(dknGamma+(GnIn^2*(1.0/2.0)-GnOut^2*(1.0/2.0))/Smax);
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F2_lowBound_gradIn_In    = t_const_gradIn_In+1.0/(GnOut*Smax);
                    F2_lowBound_gradIn_Out   = t_const_gradIn_Out-(GnIn*1.0/GnOut^2)/Smax;
                    F2_lowBound_gradOut_In   = t_const_gradOut_In-(GnIn*1.0/GnOut^2)/Smax;
                    F2_lowBound_gradOut_Out  = (1.0/GnOut^3*(Smax*dknGamma*2.0+GnIn^2+GnOut^3*Smax*t_const_gradOut_Out))/Smax;

                end
                
                F2_uppBound         = Inf;
                F2_uppBound_gradIn 	= NaN;
                F2_uppBound_gradOut	= NaN;
                
                %Set correct second derivatives, used for analytical Hessian
                F2_uppBound_gradIn_In    = NaN;
                F2_uppBound_gradIn_Out   = NaN;
                F2_uppBound_gradOut_In   = NaN;
                F2_uppBound_gradOut_Out  = NaN;

            elseif  sign(GnIn_Bound) == -1 && sign(GnOut_Bound) == -1
                
                F2_lowBound         = NaN;
                F2_uppBound         = NaN;
                F2_lowBound_gradIn 	= NaN;
                F2_uppBound_gradIn 	= NaN;
                F2_lowBound_gradOut	= NaN;
                F2_uppBound_gradOut	= NaN;
                
                %Set correct second derivatives, used for analytical Hessian
                F2_lowBound_gradIn_In    = NaN;
                F2_lowBound_gradIn_Out   = NaN;
                F2_uppBound_gradIn_In    = NaN;
                F2_uppBound_gradIn_Out   = NaN;
                F2_lowBound_gradOut_In   = NaN;
                F2_lowBound_gradOut_Out  = NaN;
                F2_uppBound_gradOut_In   = NaN;
                F2_uppBound_gradOut_Out  = NaN;

            else
                if sign(GnIn) < sign(GnOut) || sign(GnIn) > sign(GnOut)
                    
                    F2_lowBound         = t_const;
                    F2_lowBound_gradIn  = t_const_gradIn;
                    F2_lowBound_gradOut = t_const_gradOut;
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F2_lowBound_gradIn_In    = t_const_gradIn_In;
                    F2_lowBound_gradIn_Out   = t_const_gradIn_Out;
                    F2_lowBound_gradOut_In   = t_const_gradOut_In;
                    F2_lowBound_gradOut_Out  = t_const_gradOut_Out;
                    
                    F2_uppBound         = Inf;
                    F2_uppBound_gradIn 	= NaN;
                    F2_uppBound_gradOut	= NaN;
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F2_uppBound_gradIn_In    = NaN;
                    F2_uppBound_gradIn_Out   = NaN;
                    F2_uppBound_gradOut_In   = NaN;
                    F2_uppBound_gradOut_Out  = NaN;
                    
                else
                    
                    F2_lowBound         = t_const;
                    F2_lowBound_gradIn  = t_const_gradIn;
                    F2_lowBound_gradOut = t_const_gradOut;
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F2_lowBound_gradIn_In    = t_const_gradIn_In;
                    F2_lowBound_gradIn_Out   = t_const_gradIn_Out;
                    F2_lowBound_gradOut_In   = t_const_gradOut_In;
                    F2_lowBound_gradOut_Out  = t_const_gradOut_Out;
                    
                    F2_uppBound         = t_const;
                    F2_uppBound_gradIn  = t_const_gradIn;
                    F2_uppBound_gradOut = t_const_gradOut;
                    
                    %Set correct second derivatives, used for analytical Hessian
                    F2_uppBound_gradIn_In    = t_const_gradIn_In;
                    F2_uppBound_gradIn_Out   = t_const_gradIn_Out;
                    F2_uppBound_gradOut_In   = t_const_gradOut_In;
                    F2_uppBound_gradOut_Out  = t_const_gradOut_Out;
                    
                end
            %else
            	%error('Undefined case: \n  GnIn = %.4f \n GnOut = %.4f \n   dkn = %.4f',GnIn,GnOut,dkn)
            end

        %otherwise
            %error('Undefined case: \n  GnIn = %.4f \n GnOut = %.4f \n   dkn = %.4f',GnIn,GnOut,dkn)
    end

    %Get concave and convex parameters by subfunction
    switch AdjustedSign
        case -1
            %GnIn <= GnOut --> Increasing Shape
            switch sign(I_rest)
                case -1

                    % FAST:     F3 Convex shape at the beginning
                    %[ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB ] ...
                    %   = ConvexConcaveRange( GnIn, I_rest, t_const, Smax, Gmax );
                    [ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB, ...
                    F3_lowBoundA_gradIn,  F3_uppBoundA_gradIn,  F3_lowBoundB_gradIn,  F3_uppBoundB_gradIn, ...
                    F3_lowBoundA_gradOut, F3_uppBoundA_gradOut, F3_lowBoundB_gradOut, F3_uppBoundB_gradOut, ...
                    F3_lowBoundA_Hessian, F3_uppBoundA_Hessian, F3_lowBoundB_Hessian, F3_uppBoundB_Hessian ] = ConvexConcaveRange( 1, GnIn, GnOut, dknGamma, Smax, Gmax );

                    %back out second derivatives, we may change that later
                    %on to avoid the massive overhead, now we dont care
                    F3_lowBoundA_gradIn_In      = F3_lowBoundA_Hessian( 1 );
                    F3_lowBoundA_gradIn_Out     = F3_lowBoundA_Hessian( 2 );
                    F3_lowBoundA_gradOut_In     = F3_lowBoundA_Hessian( 3 );
                    F3_lowBoundA_gradOut_Out    = F3_lowBoundA_Hessian( 4 );
                    
                    F3_uppBoundA_gradIn_In      = F3_uppBoundA_Hessian( 1 );
                    F3_uppBoundA_gradIn_Out     = F3_uppBoundA_Hessian( 2 );
                    F3_uppBoundA_gradOut_In     = F3_uppBoundA_Hessian( 3 );
                    F3_uppBoundA_gradOut_Out    = F3_uppBoundA_Hessian( 4 );
                    
                    F3_lowBoundB_gradIn_In      = F3_lowBoundB_Hessian( 1 );
                    F3_lowBoundB_gradIn_Out     = F3_lowBoundB_Hessian( 2 );
                    F3_lowBoundB_gradOut_In     = F3_lowBoundB_Hessian( 3 );
                    F3_lowBoundB_gradOut_Out    = F3_lowBoundB_Hessian( 4 );
                    
                    F3_uppBoundB_gradIn_In      = F3_uppBoundB_Hessian( 1 );
                    F3_uppBoundB_gradIn_Out     = F3_uppBoundB_Hessian( 2 );
                    F3_uppBoundB_gradOut_In     = F3_uppBoundB_Hessian( 3 );
                    F3_uppBoundB_gradOut_Out    = F3_uppBoundB_Hessian( 4 );
                    
                    % SLOW:     F4 Concave shape at the end
                    %[ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB ] = ...
                    %    ConvexConcaveRange( GnOut, I_rest, t_const, Smax, Gmax );
                    [ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB, ...
                    F4_lowBoundA_gradIn,  F4_uppBoundA_gradIn,  F4_lowBoundB_gradIn,  F4_uppBoundB_gradIn, ...
                    F4_lowBoundA_gradOut, F4_uppBoundA_gradOut, F4_lowBoundB_gradOut, F4_uppBoundB_gradOut, ...
                    F4_lowBoundA_Hessian, F4_uppBoundA_Hessian, F4_lowBoundB_Hessian, F4_uppBoundB_Hessian ] = ConvexConcaveRange( 2, GnIn, GnOut, dknGamma, Smax, Gmax );

                    %back out second derivatives, we may change that later
                    %on to avoid the massive overhead, now we dont care
                    F4_lowBoundA_gradIn_In      = F4_lowBoundA_Hessian( 1 );
                    F4_lowBoundA_gradIn_Out     = F4_lowBoundA_Hessian( 2 );
                    F4_lowBoundA_gradOut_In     = F4_lowBoundA_Hessian( 3 );
                    F4_lowBoundA_gradOut_Out    = F4_lowBoundA_Hessian( 4 );
                    
                    F4_uppBoundA_gradIn_In      = F4_uppBoundA_Hessian( 1 );
                    F4_uppBoundA_gradIn_Out     = F4_uppBoundA_Hessian( 2 );
                    F4_uppBoundA_gradOut_In     = F4_uppBoundA_Hessian( 3 );
                    F4_uppBoundA_gradOut_Out    = F4_uppBoundA_Hessian( 4 );
                    
                    F4_lowBoundB_gradIn_In      = F4_lowBoundB_Hessian( 1 );
                    F4_lowBoundB_gradIn_Out     = F4_lowBoundB_Hessian( 2 );
                    F4_lowBoundB_gradOut_In     = F4_lowBoundB_Hessian( 3 );
                    F4_lowBoundB_gradOut_Out    = F4_lowBoundB_Hessian( 4 );
                    
                    F4_uppBoundB_gradIn_In      = F4_uppBoundB_Hessian( 1 );
                    F4_uppBoundB_gradIn_Out     = F4_uppBoundB_Hessian( 2 );
                    F4_uppBoundB_gradOut_In     = F4_uppBoundB_Hessian( 3 );
                    F4_uppBoundB_gradOut_Out    = F4_uppBoundB_Hessian( 4 );
                
                    %fast
                    R1_lowBound = [F3_lowBoundA, F3_lowBoundA_gradIn, F3_lowBoundA_gradOut, F3_lowBoundA_gradIn_In, F3_lowBoundA_gradIn_Out, F3_lowBoundA_gradOut_In, F3_lowBoundA_gradOut_Out ] ;
                    R1_uppBound = [F3_uppBoundA, F3_uppBoundA_gradIn, F3_uppBoundA_gradOut, F3_uppBoundA_gradIn_In, F3_uppBoundA_gradIn_Out, F3_uppBoundA_gradOut_In, F3_uppBoundA_gradOut_Out ] ;
                    
                    R2_lowBound = [F3_lowBoundB, F3_lowBoundB_gradIn, F3_lowBoundB_gradOut, F3_lowBoundB_gradIn_In, F3_lowBoundB_gradIn_Out, F3_lowBoundB_gradOut_In, F3_lowBoundB_gradOut_Out ] ;
                    R2_uppBound = [F3_uppBoundB, F3_uppBoundB_gradIn, F3_uppBoundB_gradOut, F3_uppBoundB_gradIn_In, F3_uppBoundB_gradIn_Out, F3_uppBoundB_gradOut_In, F3_uppBoundB_gradOut_Out ] ;

                    %linear
                    R3_lowBound = [F1_lowBound,  F1_lowBound_gradIn, F1_lowBound_gradOut, F1_lowBound_gradIn_In, F1_lowBound_gradIn_Out, F1_lowBound_gradOut_In, F1_lowBound_gradOut_Out ] ;
                    R3_uppBound = [F1_uppBound,  F1_uppBound_gradIn, F1_uppBound_gradOut, F1_uppBound_gradIn_In, F1_uppBound_gradIn_Out, F1_uppBound_gradOut_In, F1_uppBound_gradOut_Out ] ;

                    %slow
                    R4_lowBound = [F4_lowBoundA, F4_lowBoundA_gradIn, F4_lowBoundA_gradOut, F4_lowBoundA_gradIn_In, F4_lowBoundA_gradIn_Out, F4_lowBoundA_gradOut_In, F4_lowBoundA_gradOut_Out ] ;
                    R4_uppBound = [F4_uppBoundA, F4_uppBoundA_gradIn, F4_uppBoundA_gradOut, F4_uppBoundA_gradIn_In, F4_uppBoundA_gradIn_Out, F4_uppBoundA_gradOut_In, F4_uppBoundA_gradOut_Out ] ;
                    
                    R5_lowBound = [F4_lowBoundB, F4_lowBoundB_gradIn, F4_lowBoundB_gradOut, F4_lowBoundB_gradIn_In, F4_lowBoundB_gradIn_Out, F4_lowBoundB_gradOut_In, F4_lowBoundB_gradOut_Out ] ;
                    R5_uppBound = [F4_uppBoundB, F4_uppBoundB_gradIn, F4_uppBoundB_gradOut, F4_uppBoundB_gradIn_In, F4_uppBoundB_gradIn_Out, F4_uppBoundB_gradOut_In, F4_uppBoundB_gradOut_Out ] ;

                case +1

                    % FAST:     F4 Concave shape at the end
                    %[ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB ] = ...
                    %    ConvexConcaveRange( GnOut, I_rest, t_const, Smax, Gmax );
                    [ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB, ...
                    F4_lowBoundA_gradIn,  F4_uppBoundA_gradIn,  F4_lowBoundB_gradIn,  F4_uppBoundB_gradIn, ...
                    F4_lowBoundA_gradOut, F4_uppBoundA_gradOut, F4_lowBoundB_gradOut, F4_uppBoundB_gradOut, ...
                    F4_lowBoundA_Hessian, F4_uppBoundA_Hessian, F4_lowBoundB_Hessian, F4_uppBoundB_Hessian] = ConvexConcaveRange( 2, GnIn, GnOut, dknGamma, Smax, Gmax );

                    %back out second derivatives, we may change that later
                    %on to avoid the massive overhead, now we dont care
                    F4_lowBoundA_gradIn_In      = F4_lowBoundA_Hessian( 1 );
                    F4_lowBoundA_gradIn_Out     = F4_lowBoundA_Hessian( 2 );
                    F4_lowBoundA_gradOut_In     = F4_lowBoundA_Hessian( 3 );
                    F4_lowBoundA_gradOut_Out    = F4_lowBoundA_Hessian( 4 );
                    
                    F4_uppBoundA_gradIn_In      = F4_uppBoundA_Hessian( 1 );
                    F4_uppBoundA_gradIn_Out     = F4_uppBoundA_Hessian( 2 );
                    F4_uppBoundA_gradOut_In     = F4_uppBoundA_Hessian( 3 );
                    F4_uppBoundA_gradOut_Out    = F4_uppBoundA_Hessian( 4 );
                    
                    F4_lowBoundB_gradIn_In      = F4_lowBoundB_Hessian( 1 );
                    F4_lowBoundB_gradIn_Out     = F4_lowBoundB_Hessian( 2 );
                    F4_lowBoundB_gradOut_In     = F4_lowBoundB_Hessian( 3 );
                    F4_lowBoundB_gradOut_Out    = F4_lowBoundB_Hessian( 4 );
                    
                    F4_uppBoundB_gradIn_In      = F4_uppBoundB_Hessian( 1 );
                    F4_uppBoundB_gradIn_Out     = F4_uppBoundB_Hessian( 2 );
                    F4_uppBoundB_gradOut_In     = F4_uppBoundB_Hessian( 3 );
                    F4_uppBoundB_gradOut_Out    = F4_uppBoundB_Hessian( 4 );
                
                    % SLOW:     F3 Convex shape at the beginning
                    %[ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB ] ...
                    %    = ConvexConcaveRange( GnIn, I_rest, t_const, Smax, Gmax );
                    [ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB, ...
                    F3_lowBoundA_gradIn,  F3_uppBoundA_gradIn,  F3_lowBoundB_gradIn,  F3_uppBoundB_gradIn, ...
                    F3_lowBoundA_gradOut, F3_uppBoundA_gradOut, F3_lowBoundB_gradOut, F3_uppBoundB_gradOut, ...
                    F3_lowBoundA_Hessian, F3_uppBoundA_Hessian, F3_lowBoundB_Hessian, F3_uppBoundB_Hessian] = ConvexConcaveRange( 1, GnIn, GnOut, dknGamma, Smax, Gmax );

                    %back out second derivatives, we may change that later
                    %on to avoid the massive overhead, now we dont care
                    F3_lowBoundA_gradIn_In      = F3_lowBoundA_Hessian( 1 );
                    F3_lowBoundA_gradIn_Out     = F3_lowBoundA_Hessian( 2 );
                    F3_lowBoundA_gradOut_In     = F3_lowBoundA_Hessian( 3 );
                    F3_lowBoundA_gradOut_Out    = F3_lowBoundA_Hessian( 4 );
                    
                    F3_uppBoundA_gradIn_In      = F3_uppBoundA_Hessian( 1 );
                    F3_uppBoundA_gradIn_Out     = F3_uppBoundA_Hessian( 2 );
                    F3_uppBoundA_gradOut_In     = F3_uppBoundA_Hessian( 3 );
                    F3_uppBoundA_gradOut_Out    = F3_uppBoundA_Hessian( 4 );
                    
                    F3_lowBoundB_gradIn_In      = F3_lowBoundB_Hessian( 1 );
                    F3_lowBoundB_gradIn_Out     = F3_lowBoundB_Hessian( 2 );
                    F3_lowBoundB_gradOut_In     = F3_lowBoundB_Hessian( 3 );
                    F3_lowBoundB_gradOut_Out    = F3_lowBoundB_Hessian( 4 );
                    
                    F3_uppBoundB_gradIn_In      = F3_uppBoundB_Hessian( 1 );
                    F3_uppBoundB_gradIn_Out     = F3_uppBoundB_Hessian( 2 );
                    F3_uppBoundB_gradOut_In     = F3_uppBoundB_Hessian( 3 );
                    F3_uppBoundB_gradOut_Out    = F3_uppBoundB_Hessian( 4 );
                
                    %fast
                    R1_lowBound     = [F4_lowBoundA, F4_lowBoundA_gradIn, F4_lowBoundA_gradOut, F4_lowBoundA_gradIn_In, F4_lowBoundA_gradIn_Out, F4_lowBoundA_gradOut_In, F4_lowBoundA_gradOut_Out ] ;
                    R1_uppBound     = [F4_uppBoundA, F4_uppBoundA_gradIn, F4_uppBoundA_gradOut, F4_uppBoundA_gradIn_In, F4_uppBoundA_gradIn_Out, F4_uppBoundA_gradOut_In, F4_uppBoundA_gradOut_Out ] ;
                    
                    R2_lowBound     = [F4_lowBoundB, F4_lowBoundB_gradIn, F4_lowBoundB_gradOut, F4_lowBoundB_gradIn_In, F4_lowBoundB_gradIn_Out, F4_lowBoundB_gradOut_In, F4_lowBoundB_gradOut_Out ] ;
                    R2_uppBound     = [F4_uppBoundB, F4_uppBoundB_gradIn, F4_uppBoundB_gradOut, F4_uppBoundB_gradIn_In, F4_uppBoundB_gradIn_Out, F4_uppBoundB_gradOut_In, F4_uppBoundB_gradOut_Out ] ;

                    %linear
                    R3_lowBound     = [F1_lowBound, F1_lowBound_gradIn, F1_lowBound_gradOut, F1_lowBound_gradIn_In, F1_lowBound_gradIn_Out, F1_lowBound_gradOut_In, F1_lowBound_gradOut_Out ] ;
                    R3_uppBound     = [F1_uppBound, F1_uppBound_gradIn, F1_uppBound_gradOut, F1_uppBound_gradIn_In, F1_uppBound_gradIn_Out, F1_uppBound_gradOut_In, F1_uppBound_gradOut_Out ] ;

                    %slow
                    R4_lowBound     = [F3_lowBoundA, F3_lowBoundA_gradIn, F3_lowBoundA_gradOut, F3_lowBoundA_gradIn_In, F3_lowBoundA_gradIn_Out, F3_lowBoundA_gradOut_In, F3_lowBoundA_gradOut_Out ] ;
                    R4_uppBound     = [F3_uppBoundA, F3_uppBoundA_gradIn, F3_uppBoundA_gradOut, F3_uppBoundA_gradIn_In, F3_uppBoundA_gradIn_Out, F3_uppBoundA_gradOut_In, F3_uppBoundA_gradOut_Out ] ;
                    
                    R5_lowBound     = [F3_lowBoundB, F3_lowBoundB_gradIn, F3_lowBoundB_gradOut, F3_lowBoundB_gradIn_In, F3_lowBoundB_gradIn_Out, F3_lowBoundB_gradOut_In, F3_lowBoundB_gradOut_Out ] ;
                    R5_uppBound     = [F3_uppBoundB, F3_uppBoundB_gradIn, F3_uppBoundB_gradOut, F3_uppBoundB_gradIn_In, F3_uppBoundB_gradIn_Out, F3_uppBoundB_gradOut_In, F3_uppBoundB_gradOut_Out ] ;
                    
                otherwise
                %no residual integration; I_rest = 0
                    if (sign(GnIn) == -1 && sign(GnOut) == +1) || ...
                       (sign(GnIn) == +1 && sign(GnOut) == -1)  
                            %set full increasing
                            R1_lowBound     = [F1_lowBound, F1_lowBound_gradIn, F1_lowBound_gradOut, F1_lowBound_gradIn_In, F1_lowBound_gradIn_Out, F1_lowBound_gradOut_In, F1_lowBound_gradOut_Out ] ;
                            R1_uppBound     = [F1_uppBound, F1_uppBound_gradIn, F1_uppBound_gradOut, F1_uppBound_gradIn_In, F1_uppBound_gradIn_Out, F1_uppBound_gradOut_In, F1_uppBound_gradOut_Out ] ;
                    else
                    
                        if abs(GnIn) <= abs(GnOut)
                            %GnOffset    = GnIn;
                            ID          = 1;
                        else
                            %GnOffset    = GnOut;
                            ID          = 2;
                        end

                        %Depending on the sign of GnOffset, a convex or concave
                        %shape will be used
                        %[ F34_lowBoundA, F34_uppBoundA, ~, ~ ] ...
                        %    = ConvexConcaveRange( GnOffset, I_rest, t_const, Smax, Gmax );
                        [ F34_lowBoundA, F34_uppBoundA, ~, ~, ...
                        F34_lowBoundA_gradIn,  F34_uppBoundA_gradIn,  ~, ~, ...
                        F34_lowBoundA_gradOut, F34_uppBoundA_gradOut, ~, ~, ...
                        F34_lowBoundA_Hessian, F34_uppBoundA_Hessian, ~, ~] = ConvexConcaveRange( ID, GnIn, GnOut, dknGamma, Smax, Gmax );
                        
                        %back out second derivatives, we may change that later
                        %on to avoid the massive overhead, now we dont care
                        F34_lowBoundA_gradIn_In      = F34_lowBoundA_Hessian( 1 );
                        F34_lowBoundA_gradIn_Out     = F34_lowBoundA_Hessian( 2 );
                        F34_lowBoundA_gradOut_In     = F34_lowBoundA_Hessian( 3 );
                        F34_lowBoundA_gradOut_Out    = F34_lowBoundA_Hessian( 4 );

                        F34_uppBoundA_gradIn_In      = F34_uppBoundA_Hessian( 1 );
                        F34_uppBoundA_gradIn_Out     = F34_uppBoundA_Hessian( 2 );
                        F34_uppBoundA_gradOut_In     = F34_uppBoundA_Hessian( 3 );
                        F34_uppBoundA_gradOut_Out    = F34_uppBoundA_Hessian( 4 );

                        %F34_lowBoundB_gradIn_In      = F34_lowBoundB_Hessian( 1 );
                        %F34_lowBoundB_gradIn_Out     = F34_lowBoundB_Hessian( 2 );
                        %F34_lowBoundB_gradOut_In     = F34_lowBoundB_Hessian( 3 );
                        %F34_lowBoundB_gradOut_Out    = F34_lowBoundB_Hessian( 4 );

                        %F34_uppBoundB_gradIn_In      = F34_uppBoundB_Hessian( 1 );
                        %F34_uppBoundB_gradIn_Out     = F34_uppBoundB_Hessian( 2 );
                        %F34_uppBoundB_gradOut_In     = F34_uppBoundB_Hessian( 3 );
                        %F34_uppBoundB_gradOut_Out    = F34_uppBoundB_Hessian( 4 );
                    
                        %no faster solution than increasing possible
                        R1_lowBound     = [NaN, NaN, NaN, NaN, NaN, NaN, NaN];
                        R1_uppBound     = [NaN, NaN, NaN, NaN, NaN, NaN, NaN];
                        
                        R2_lowBound     = [NaN, NaN, NaN, NaN, NaN, NaN, NaN];
                        R2_uppBound     = [NaN, NaN, NaN, NaN, NaN, NaN, NaN];

                        R3_lowBound     = [F1_lowBound, F1_lowBound_gradIn, F1_lowBound_gradOut, F1_lowBound_gradIn_In, F1_lowBound_gradIn_Out, F1_lowBound_gradOut_In, F1_lowBound_gradOut_Out ] ;
                        R3_uppBound     = [F1_uppBound, F1_uppBound_gradIn, F1_uppBound_gradOut, F1_uppBound_gradIn_In, F1_uppBound_gradIn_Out, F1_uppBound_gradOut_In, F1_uppBound_gradOut_Out ] ;

                        R4_lowBound     = [F34_lowBoundA, F34_lowBoundA_gradIn, F34_lowBoundA_gradOut, F34_lowBoundA_gradIn_In, F34_lowBoundA_gradIn_Out, F34_lowBoundA_gradOut_In, F34_lowBoundA_gradOut_Out ] ;
                        R4_uppBound     = [F34_uppBoundA, F34_uppBoundA_gradIn, F34_uppBoundA_gradOut, F34_uppBoundA_gradIn_In, F34_uppBoundA_gradIn_Out, F34_uppBoundA_gradOut_In, F34_uppBoundA_gradOut_Out ] ;

                        R5_lowBound     = [NaN, NaN, NaN, NaN, NaN, NaN, NaN];
                        R5_uppBound     = [NaN, NaN, NaN, NaN, NaN, NaN, NaN];
                    
                    end
            end

        otherwise
            %GnIn >= GnOut --> Decreasing Shape
            switch sign(I_rest)
                case -1
                    % FAST:     F4 Convex shape at the end
                    %[ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB ] = ...
                    %    ConvexConcaveRange( GnOut, I_rest, t_const, Smax, Gmax );
                    [ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB, ...
                    F4_lowBoundA_gradIn,  F4_uppBoundA_gradIn,  F4_lowBoundB_gradIn,  F4_uppBoundB_gradIn, ...
                    F4_lowBoundA_gradOut, F4_uppBoundA_gradOut, F4_lowBoundB_gradOut, F4_uppBoundB_gradOut, ...
                    F4_lowBoundA_Hessian, F4_uppBoundA_Hessian, F4_lowBoundB_Hessian, F4_uppBoundB_Hessian] = ConvexConcaveRange( 2, GnIn, GnOut, dknGamma, Smax, Gmax );

                    %back out second derivatives, we may change that later
                    %on to avoid the massive overhead, now we dont care
                    F4_lowBoundA_gradIn_In      = F4_lowBoundA_Hessian( 1 );
                    F4_lowBoundA_gradIn_Out     = F4_lowBoundA_Hessian( 2 );
                    F4_lowBoundA_gradOut_In     = F4_lowBoundA_Hessian( 3 );
                    F4_lowBoundA_gradOut_Out    = F4_lowBoundA_Hessian( 4 );
                    
                    F4_uppBoundA_gradIn_In      = F4_uppBoundA_Hessian( 1 );
                    F4_uppBoundA_gradIn_Out     = F4_uppBoundA_Hessian( 2 );
                    F4_uppBoundA_gradOut_In     = F4_uppBoundA_Hessian( 3 );
                    F4_uppBoundA_gradOut_Out    = F4_uppBoundA_Hessian( 4 );
                    
                    F4_lowBoundB_gradIn_In      = F4_lowBoundB_Hessian( 1 );
                    F4_lowBoundB_gradIn_Out     = F4_lowBoundB_Hessian( 2 );
                    F4_lowBoundB_gradOut_In     = F4_lowBoundB_Hessian( 3 );
                    F4_lowBoundB_gradOut_Out    = F4_lowBoundB_Hessian( 4 );
                    
                    F4_uppBoundB_gradIn_In      = F4_uppBoundB_Hessian( 1 );
                    F4_uppBoundB_gradIn_Out     = F4_uppBoundB_Hessian( 2 );
                    F4_uppBoundB_gradOut_In     = F4_uppBoundB_Hessian( 3 );
                    F4_uppBoundB_gradOut_Out    = F4_uppBoundB_Hessian( 4 );
                
                    % SLOW:     F3 Concave shape at the beginning
                    %[ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB ] ...
                    %    = ConvexConcaveRange( GnIn, I_rest, t_const, Smax, Gmax );
                    [ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB, ...
                    F3_lowBoundA_gradIn,  F3_uppBoundA_gradIn,  F3_lowBoundB_gradIn,  F3_uppBoundB_gradIn, ...
                    F3_lowBoundA_gradOut, F3_uppBoundA_gradOut, F3_lowBoundB_gradOut, F3_uppBoundB_gradOut, ...
                    F3_lowBoundA_Hessian, F3_uppBoundA_Hessian, F3_lowBoundB_Hessian, F3_uppBoundB_Hessian] = ConvexConcaveRange( 1, GnIn, GnOut, dknGamma, Smax, Gmax );

                    %back out second derivatives, we may change that later
                    %on to avoid the massive overhead, now we dont care
                    F3_lowBoundA_gradIn_In      = F3_lowBoundA_Hessian( 1 );
                    F3_lowBoundA_gradIn_Out     = F3_lowBoundA_Hessian( 2 );
                    F3_lowBoundA_gradOut_In     = F3_lowBoundA_Hessian( 3 );
                    F3_lowBoundA_gradOut_Out    = F3_lowBoundA_Hessian( 4 );
                    
                    F3_uppBoundA_gradIn_In      = F3_uppBoundA_Hessian( 1 );
                    F3_uppBoundA_gradIn_Out     = F3_uppBoundA_Hessian( 2 );
                    F3_uppBoundA_gradOut_In     = F3_uppBoundA_Hessian( 3 );
                    F3_uppBoundA_gradOut_Out    = F3_uppBoundA_Hessian( 4 );
                    
                    F3_lowBoundB_gradIn_In      = F3_lowBoundB_Hessian( 1 );
                    F3_lowBoundB_gradIn_Out     = F3_lowBoundB_Hessian( 2 );
                    F3_lowBoundB_gradOut_In     = F3_lowBoundB_Hessian( 3 );
                    F3_lowBoundB_gradOut_Out    = F3_lowBoundB_Hessian( 4 );
                    
                    F3_uppBoundB_gradIn_In      = F3_uppBoundB_Hessian( 1 );
                    F3_uppBoundB_gradIn_Out     = F3_uppBoundB_Hessian( 2 );
                    F3_uppBoundB_gradOut_In     = F3_uppBoundB_Hessian( 3 );
                    F3_uppBoundB_gradOut_Out    = F3_uppBoundB_Hessian( 4 );
                
                    %fast
                    R1_lowBound     = [F4_lowBoundA, F4_lowBoundA_gradIn, F4_lowBoundA_gradOut, F4_lowBoundA_gradIn_In, F4_lowBoundA_gradIn_Out, F4_lowBoundA_gradOut_In, F4_lowBoundA_gradOut_Out ] ;
                    R1_uppBound     = [F4_uppBoundA, F4_uppBoundA_gradIn, F4_uppBoundA_gradOut, F4_uppBoundA_gradIn_In, F4_uppBoundA_gradIn_Out, F4_uppBoundA_gradOut_In, F4_uppBoundA_gradOut_Out ] ;
                    
                    R2_lowBound     = [F4_lowBoundB, F4_lowBoundB_gradIn, F4_lowBoundB_gradOut, F4_lowBoundB_gradIn_In, F4_lowBoundB_gradIn_Out, F4_lowBoundB_gradOut_In, F4_lowBoundB_gradOut_Out ] ;
                    R2_uppBound     = [F4_uppBoundB, F4_uppBoundB_gradIn, F4_uppBoundB_gradOut, F4_uppBoundB_gradIn_In, F4_uppBoundB_gradIn_Out, F4_uppBoundB_gradOut_In, F4_uppBoundB_gradOut_Out ] ;

                    %linear
                    R3_lowBound     = [F2_lowBound, F2_lowBound_gradIn, F2_lowBound_gradOut, F2_lowBound_gradIn_In, F2_lowBound_gradIn_Out, F2_lowBound_gradOut_In, F2_lowBound_gradOut_Out ] ;
                    R3_uppBound     = [F2_uppBound, F2_uppBound_gradIn, F2_uppBound_gradOut, F2_uppBound_gradIn_In, F2_uppBound_gradIn_Out, F2_uppBound_gradOut_In, F2_uppBound_gradOut_Out ] ;

                    %slow
                    R4_lowBound     = [F3_lowBoundA, F3_lowBoundA_gradIn, F3_lowBoundA_gradOut, F3_lowBoundA_gradIn_In, F3_lowBoundA_gradIn_Out, F3_lowBoundA_gradOut_In, F3_lowBoundA_gradOut_Out ] ;
                    R4_uppBound     = [F3_uppBoundA, F3_uppBoundA_gradIn, F3_uppBoundA_gradOut, F3_uppBoundA_gradIn_In, F3_uppBoundA_gradIn_Out, F3_uppBoundA_gradOut_In, F3_uppBoundA_gradOut_Out ] ;
                    
                    R5_lowBound     = [F3_lowBoundB, F3_lowBoundB_gradIn, F3_lowBoundB_gradOut, F3_lowBoundB_gradIn_In, F3_lowBoundB_gradIn_Out, F3_lowBoundB_gradOut_In, F3_lowBoundB_gradOut_Out ] ;
                    R5_uppBound     = [F3_uppBoundB, F3_uppBoundB_gradIn, F3_uppBoundB_gradOut, F3_uppBoundB_gradIn_In, F3_uppBoundB_gradIn_Out, F3_uppBoundB_gradOut_In, F3_uppBoundB_gradOut_Out ] ;

                case +1

                    % FAST:     F3 Convex shape at the beginning
                    %[ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB ] ...
                    %    = ConvexConcaveRange( GnIn, I_rest, t_const, Smax, Gmax );
                    [ F3_lowBoundA, F3_uppBoundA, F3_lowBoundB, F3_uppBoundB, ...
                    F3_lowBoundA_gradIn,  F3_uppBoundA_gradIn,  F3_lowBoundB_gradIn,  F3_uppBoundB_gradIn, ...
                    F3_lowBoundA_gradOut, F3_uppBoundA_gradOut, F3_lowBoundB_gradOut, F3_uppBoundB_gradOut, ...
                    F3_lowBoundA_Hessian, F3_uppBoundA_Hessian, F3_lowBoundB_Hessian, F3_uppBoundB_Hessian] = ConvexConcaveRange( 1, GnIn, GnOut, dknGamma, Smax, Gmax );

                    %back out second derivatives, we may change that later
                    %on to avoid the massive overhead, now we dont care
                    F3_lowBoundA_gradIn_In      = F3_lowBoundA_Hessian( 1 );
                    F3_lowBoundA_gradIn_Out     = F3_lowBoundA_Hessian( 2 );
                    F3_lowBoundA_gradOut_In     = F3_lowBoundA_Hessian( 3 );
                    F3_lowBoundA_gradOut_Out    = F3_lowBoundA_Hessian( 4 );
                    
                    F3_uppBoundA_gradIn_In      = F3_uppBoundA_Hessian( 1 );
                    F3_uppBoundA_gradIn_Out     = F3_uppBoundA_Hessian( 2 );
                    F3_uppBoundA_gradOut_In     = F3_uppBoundA_Hessian( 3 );
                    F3_uppBoundA_gradOut_Out    = F3_uppBoundA_Hessian( 4 );
                    
                    F3_lowBoundB_gradIn_In      = F3_lowBoundB_Hessian( 1 );
                    F3_lowBoundB_gradIn_Out     = F3_lowBoundB_Hessian( 2 );
                    F3_lowBoundB_gradOut_In     = F3_lowBoundB_Hessian( 3 );
                    F3_lowBoundB_gradOut_Out    = F3_lowBoundB_Hessian( 4 );
                    
                    F3_uppBoundB_gradIn_In      = F3_uppBoundB_Hessian( 1 );
                    F3_uppBoundB_gradIn_Out     = F3_uppBoundB_Hessian( 2 );
                    F3_uppBoundB_gradOut_In     = F3_uppBoundB_Hessian( 3 );
                    F3_uppBoundB_gradOut_Out    = F3_uppBoundB_Hessian( 4 );
                
                    % SLOW:     F4 Concave shape at the end
                    %[ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB ] = ...
                    %    ConvexConcaveRange( GnOut, I_rest, t_const, Smax, Gmax );
                    [ F4_lowBoundA, F4_uppBoundA, F4_lowBoundB, F4_uppBoundB, ...
                    F4_lowBoundA_gradIn,  F4_uppBoundA_gradIn,  F4_lowBoundB_gradIn,  F4_uppBoundB_gradIn, ...
                    F4_lowBoundA_gradOut, F4_uppBoundA_gradOut, F4_lowBoundB_gradOut, F4_uppBoundB_gradOut, ...
                    F4_lowBoundA_Hessian, F4_uppBoundA_Hessian, F4_lowBoundB_Hessian, F4_uppBoundB_Hessian] = ConvexConcaveRange( 2, GnIn, GnOut, dknGamma, Smax, Gmax );

                    %back out second derivatives, we may change that later
                    %on to avoid the massive overhead, now we dont care
                    F4_lowBoundA_gradIn_In      = F4_lowBoundA_Hessian( 1 );
                    F4_lowBoundA_gradIn_Out     = F4_lowBoundA_Hessian( 2 );
                    F4_lowBoundA_gradOut_In     = F4_lowBoundA_Hessian( 3 );
                    F4_lowBoundA_gradOut_Out    = F4_lowBoundA_Hessian( 4 );
                    
                    F4_uppBoundA_gradIn_In      = F4_uppBoundA_Hessian( 1 );
                    F4_uppBoundA_gradIn_Out     = F4_uppBoundA_Hessian( 2 );
                    F4_uppBoundA_gradOut_In     = F4_uppBoundA_Hessian( 3 );
                    F4_uppBoundA_gradOut_Out    = F4_uppBoundA_Hessian( 4 );
                    
                    F4_lowBoundB_gradIn_In      = F4_lowBoundB_Hessian( 1 );
                    F4_lowBoundB_gradIn_Out     = F4_lowBoundB_Hessian( 2 );
                    F4_lowBoundB_gradOut_In     = F4_lowBoundB_Hessian( 3 );
                    F4_lowBoundB_gradOut_Out    = F4_lowBoundB_Hessian( 4 );
                    
                    F4_uppBoundB_gradIn_In      = F4_uppBoundB_Hessian( 1 );
                    F4_uppBoundB_gradIn_Out     = F4_uppBoundB_Hessian( 2 );
                    F4_uppBoundB_gradOut_In     = F4_uppBoundB_Hessian( 3 );
                    F4_uppBoundB_gradOut_Out    = F4_uppBoundB_Hessian( 4 );
                
                    %fast
                    R1_lowBound     = [F3_lowBoundA, F3_lowBoundA_gradIn, F3_lowBoundA_gradOut, F3_lowBoundA_gradIn_In, F3_lowBoundA_gradIn_Out, F3_lowBoundA_gradOut_In, F3_lowBoundA_gradOut_Out ] ;
                    R1_uppBound     = [F3_uppBoundA, F3_uppBoundA_gradIn, F3_uppBoundA_gradOut, F3_uppBoundA_gradIn_In, F3_uppBoundA_gradIn_Out, F3_uppBoundA_gradOut_In, F3_uppBoundA_gradOut_Out ] ;
                    
                    R2_lowBound     = [F3_lowBoundB, F3_lowBoundB_gradIn, F3_lowBoundB_gradOut, F3_lowBoundB_gradIn_In, F3_lowBoundB_gradIn_Out, F3_lowBoundB_gradOut_In, F3_lowBoundB_gradOut_Out ] ;
                    R2_uppBound     = [F3_uppBoundB, F3_uppBoundB_gradIn, F3_uppBoundB_gradOut, F3_uppBoundB_gradIn_In, F3_uppBoundB_gradIn_Out, F3_uppBoundB_gradOut_In, F3_uppBoundB_gradOut_Out ] ;

                    %linear
                    R3_lowBound     = [F2_lowBound, F2_lowBound_gradIn, F2_lowBound_gradOut, F2_lowBound_gradIn_In, F2_lowBound_gradIn_Out, F2_lowBound_gradOut_In, F2_lowBound_gradOut_Out ] ;
                    R3_uppBound     = [F2_uppBound, F2_uppBound_gradIn, F2_uppBound_gradOut, F2_uppBound_gradIn_In, F2_uppBound_gradIn_Out, F2_uppBound_gradOut_In, F2_uppBound_gradOut_Out ] ;

                    %slow
                    R4_lowBound     = [F4_lowBoundA, F4_lowBoundA_gradIn, F4_lowBoundA_gradOut, F4_lowBoundA_gradIn_In, F4_lowBoundA_gradIn_Out, F4_lowBoundA_gradOut_In, F4_lowBoundA_gradOut_Out ] ;
                    R4_uppBound     = [F4_uppBoundA, F4_uppBoundA_gradIn, F4_uppBoundA_gradOut, F4_uppBoundA_gradIn_In, F4_uppBoundA_gradIn_Out, F4_uppBoundA_gradOut_In, F4_uppBoundA_gradOut_Out ] ;
                    
                    R5_lowBound     = [F4_lowBoundB, F4_lowBoundB_gradIn, F4_lowBoundB_gradOut, F4_lowBoundB_gradIn_In, F4_lowBoundB_gradIn_Out, F4_lowBoundB_gradOut_In, F4_lowBoundB_gradOut_Out ] ;
                    R5_uppBound     = [F4_uppBoundB, F4_uppBoundB_gradIn, F4_uppBoundB_gradOut, F4_uppBoundB_gradIn_In, F4_uppBoundB_gradIn_Out, F4_uppBoundB_gradOut_In, F4_uppBoundB_gradOut_Out ] ;
                    
                otherwise
                    %no residual integration; I_rest = 0
                    if (sign(GnIn) == -1 && sign(GnOut) == +1) || ...
                       (sign(GnIn) == +1 && sign(GnOut) == -1)  
                            %set full decreasing
                            R1_lowBound     = [F2_lowBound, F2_lowBound_gradIn, F2_lowBound_gradOut, F2_lowBound_gradIn_In, F2_lowBound_gradIn_Out, F2_lowBound_gradOut_In, F2_lowBound_gradOut_Out ] ;
                            R1_uppBound     = [F2_uppBound, F2_uppBound_gradIn, F2_uppBound_gradOut, F2_uppBound_gradIn_In, F2_uppBound_gradIn_Out, F2_uppBound_gradOut_In, F2_uppBound_gradOut_Out ] ;
                    else
                    
                        if abs(GnIn) <= abs(GnOut)
                            %GnOffset    = GnIn;
                            ID          = 1;
                        else
                            %GnOffset    = GnOut;
                            ID          = 2;
                        end

                        %Depending on the sign of GnOffset, a convex or concave
                        %shape will be used
                        %[ F34_lowBoundA, F34_uppBoundA, ~, ~ ] ...
                        %    = ConvexConcaveRange( GnOffset, I_rest, t_const, Smax, Gmax );
                        [ F34_lowBoundA, F34_uppBoundA, ~, ~, ...
                        F34_lowBoundA_gradIn,  F34_uppBoundA_gradIn,  ~,  ~, ...
                        F34_lowBoundA_gradOut, F34_uppBoundA_gradOut, ~,  ~, ...
                        F34_lowBoundA_Hessian, F34_uppBoundA_Hessian, ~, ~ ] = ConvexConcaveRange( ID, GnIn, GnOut, dknGamma, Smax, Gmax );

                        
                        %back out second derivatives, we may change that later
                        %on to avoid the massive overhead, now we dont care
                        F34_lowBoundA_gradIn_In      = F34_lowBoundA_Hessian( 1 );
                        F34_lowBoundA_gradIn_Out     = F34_lowBoundA_Hessian( 2 );
                        F34_lowBoundA_gradOut_In     = F34_lowBoundA_Hessian( 3 );
                        F34_lowBoundA_gradOut_Out    = F34_lowBoundA_Hessian( 4 );

                        F34_uppBoundA_gradIn_In      = F34_uppBoundA_Hessian( 1 );
                        F34_uppBoundA_gradIn_Out     = F34_uppBoundA_Hessian( 2 );
                        F34_uppBoundA_gradOut_In     = F34_uppBoundA_Hessian( 3 );
                        F34_uppBoundA_gradOut_Out    = F34_uppBoundA_Hessian( 4 );

                        %F34_lowBoundB_gradIn_In      = F34_lowBoundB_Hessian( 1 );
                        %F34_lowBoundB_gradIn_Out     = F34_lowBoundB_Hessian( 2 );
                        %F34_lowBoundB_gradOut_In     = F34_lowBoundB_Hessian( 3 );
                        %F34_lowBoundB_gradOut_Out    = F34_lowBoundB_Hessian( 4 );

                        %F34_uppBoundB_gradIn_In      = F34_uppBoundB_Hessian( 1 );
                        %F34_uppBoundB_gradIn_Out     = F34_uppBoundB_Hessian( 2 );
                        %F34_uppBoundB_gradOut_In     = F34_uppBoundB_Hessian( 3 );
                        %F34_uppBoundB_gradOut_Out    = F34_uppBoundB_Hessian( 4 );
                    
                        %no faster solution than increasing possible
                        R1_lowBound     = [NaN, NaN, NaN, NaN, NaN, NaN, NaN];
                        R1_uppBound     = [NaN, NaN, NaN, NaN, NaN, NaN, NaN];
                        
                        R2_lowBound     = [NaN, NaN, NaN, NaN, NaN, NaN, NaN];
                        R2_uppBound     = [NaN, NaN, NaN, NaN, NaN, NaN, NaN];

                        R3_lowBound     = [F2_lowBound, F2_lowBound_gradIn, F2_lowBound_gradOut, F2_lowBound_gradIn_In, F2_lowBound_gradIn_Out, F2_lowBound_gradOut_In, F2_lowBound_gradOut_Out ] ;
                        R3_uppBound     = [F2_uppBound, F2_uppBound_gradIn, F2_uppBound_gradOut, F2_uppBound_gradIn_In, F2_uppBound_gradIn_Out, F2_uppBound_gradOut_In, F2_uppBound_gradOut_Out ] ;

                        R4_lowBound     = [F34_lowBoundA, F34_lowBoundA_gradIn, F34_lowBoundA_gradOut, F34_lowBoundA_gradIn_In, F34_lowBoundA_gradIn_Out, F34_lowBoundA_gradOut_In, F34_lowBoundA_gradOut_Out ] ;
                        R4_uppBound     = [F34_uppBoundA, F34_uppBoundA_gradIn, F34_uppBoundA_gradOut, F34_uppBoundA_gradIn_In, F34_uppBoundA_gradIn_Out, F34_uppBoundA_gradOut_In, F34_uppBoundA_gradOut_Out ] ;

                        R5_lowBound     = [NaN, NaN, NaN, NaN, NaN, NaN, NaN];
                        R5_uppBound     = [NaN, NaN, NaN, NaN, NaN, NaN, NaN];
                    
                    end     
            end
    end

    %Get concatenated ranges
    lowBoundsX     = nan(5,7);
    uppBoundsX     = nan(5,7);

    while(1)
        %Set low limits of part 1
        lowBoundsX(1,:) = R1_lowBound;
        uppBoundsX(1,:) = R1_uppBound;

        %return, if upper bound is inf
        if isinf(R1_uppBound(1)) && isnan(R3_lowBound(1)) || ...
           isinf(R1_uppBound(1)) && isinf(R3_lowBound(1))     
            break;
        end

        if      R1_uppBound(1) >= R3_lowBound(1)
            uppBoundsX(1,:) = R3_lowBound;
        elseif  R2_lowBound(1) >= R3_lowBound(1)
            lowBoundsX(2,:) = R2_lowBound;
            uppBoundsX(2,:) = R2_uppBound;
            %again, adjust upper
            if R2_uppBound(1) >= R3_lowBound(1)
                uppBoundsX(2,:) = R3_lowBound;
            end
        end
        
        if isinf(R2_uppBound(1)) && isnan(R3_lowBound(1)) && isnan(R3_uppBound(1))
            lowBoundsX(2,:) = R2_lowBound;
            uppBoundsX(2,:) = R2_uppBound;
            break;
        end

        if isnan(R3_lowBound(1)) && isnan(R3_uppBound(1))
           break; 
        end
        
        %set linear phase bounds
        lowBoundsX(3,:) = R3_lowBound;
        uppBoundsX(3,:) = R3_uppBound;

        %return, if upper bound is inf
        if isinf(R3_uppBound(1))
            break;
        end 

        %Set low limits of part 3
        lowBoundsX(4,:) = R4_lowBound;
        uppBoundsX(4,:) = R4_uppBound;

        %adjust to linear bounds
        if R4_lowBound(1) < R3_uppBound(1)
            lowBoundsX(4,:) = R3_uppBound;
        end

        %return, if upper bound is inf
        if isinf(R4_uppBound(1))
            break;
        end 

        %Set low limits of part 3
        lowBoundsX(5,:) = R5_lowBound;
        uppBoundsX(5,:) = R5_uppBound;

        %return, if upper bound is inf
        if isinf(R5_uppBound(1))
            break;
        end 

    end

    %unequal nans in time ranges
%     if min(isnan(lowBoundsX) == isnan(uppBoundsX) == 0)
%         error('NaN occurred in time ranges!')
%     end

    C_low_isnan 	=  ~isnan(lowBoundsX(:,1)) ;
    lowBoundsX     = lowBoundsX( C_low_isnan, : );
        
    C_upp_isnan 	=  ~isnan(uppBoundsX(:,1)) ;
    uppBoundsX     = uppBoundsX( C_upp_isnan, : );
    
    %round to 8 digits
    %lowBoundsX = round( lowBoundsX * 1e+8 ) / 1e+8;
    %uppBoundsX = round( uppBoundsX * 1e+8 ) / 1e+8;

end
    
    
%Subfunction to deal with the convex and concave parts
function [ lowBoundA, uppBoundA, lowBoundB, uppBoundB, ...
           lowBoundA_gradIn,  uppBoundA_gradIn,  lowBoundB_gradIn,  uppBoundB_gradIn, ...
           lowBoundA_gradOut, uppBoundA_gradOut, lowBoundB_gradOut, uppBoundB_gradOut, ...
           lowBoundA_Hessian, uppBoundA_Hessian, lowBoundB_Hessian, uppBoundB_Hessian ] = ConvexConcaveRange( ID, GnIn, GnOut, dknGamma, Smax, Gmax )

    %the output is given in vector form each
    % lowBoundA_Hessian = [lowBoundA_gradIn_In, lowBoundA_gradIn_Out, lowBoundA_gradOut_In, lowBoundA_gradOut_Out];
 	% uppBoundA_Hessian = [uppBoundA_gradIn_In, uppBoundA_gradIn_Out, uppBoundA_gradOut_In, uppBoundA_gradOut_Out];
  	% lowBoundB_Hessian = [lowBoundB_gradIn_In, lowBoundB_gradIn_Out, lowBoundB_gradOut_In, lowBoundB_gradOut_Out];
    % uppBoundB_Hessian = [uppBoundB_gradIn_In, uppBoundB_gradIn_Out, uppBoundB_gradOut_In, uppBoundB_gradOut_Out];
  
    %set GnOffset according to ID
    if ID == 1
        GnOffset = GnIn;
    else
        GnOffset = GnOut;
    end
       
       
    %recalculate the following values in this method rather than handing
    %them to the function, since GnIn, GnOut and dknGamma are needed here
    %in order to calculate the gradients
    
    t_const  	= abs(GnIn-GnOut)/Smax;
    I_const 	= (sign(GnIn-GnOut)*(GnIn^2-GnOut^2)*(1.0/2.0))/Smax;
    I_rest      = -I_const+dknGamma;
    
    %get adjusted sign parameter
    AdjustedSign = sign(GnIn - GnOut);
    if AdjustedSign == 0
        %in case AdjustedSign == 0, i.e. GnIn == GnOut, we have to decide,
        %as how to treat the problem
        switch sign(I_rest)
            case +1
                AdjustedSign = -1;
            case -1
                AdjustedSign = +1;
            otherwise
                AdjustedSign = +1;
        end
        
    end
    
    %get derivative of t_const_grad with respect to GnIn and GnOut
    if      AdjustedSign == +1
        
        %t_const  	= +(GnIn - GnOut)/Smax;
        t_const_gradIn      = 1.0/Smax;
        t_const_gradOut     = -1.0/Smax;
        
        %Set correct second derivatives, used for analytical Hessian
        t_const_gradIn_In        = 0.0;
        t_const_gradIn_Out       = 0.0;
        t_const_gradOut_In       = 0.0;
        t_const_gradOut_Out      = 0.0;
        
    else   %AdjustedSign == -1
        
        %t_const  	= -(GnIn - GnOut)/Smax;
        t_const_gradIn      = -1.0/Smax;
        t_const_gradOut     = 1.0/Smax;
        
        %Set correct second derivatives, used for analytical Hessian
        t_const_gradIn_In        = 0.0;
        t_const_gradIn_Out       = 0.0;
        t_const_gradOut_In       = 0.0;
        t_const_gradOut_Out      = 0.0;
        
    end
       
       
    %check special case 
    if GnOffset == 0 && I_rest == 0
        %set bounds
        lowBoundA   = t_const;
        uppBoundA   = Inf;
        lowBoundB   = NaN;
        uppBoundB   = NaN;
        
        %set In gradients
        lowBoundA_gradIn   = t_const_gradIn;
        uppBoundA_gradIn   = NaN;
        lowBoundB_gradIn   = NaN;
        uppBoundB_gradIn   = NaN;
        
        %Set correct second derivatives, used for analytical Hessian
        lowBoundA_gradIn_In      = t_const_gradIn_In;
        lowBoundA_gradIn_Out     = t_const_gradIn_Out;
        uppBoundA_gradIn_In      = NaN;
        uppBoundA_gradIn_Out     = NaN;
        lowBoundB_gradIn_In      = NaN;
        lowBoundB_gradIn_Out     = NaN;
        uppBoundB_gradIn_In      = NaN;
        uppBoundB_gradIn_Out     = NaN;
        
        %set Out gradients
        lowBoundA_gradOut  = t_const_gradOut;
        uppBoundA_gradOut  = NaN;
        lowBoundB_gradOut  = NaN;
        uppBoundB_gradOut  = NaN;
        
        %Set correct second derivatives, used for analytical Hessian
        lowBoundA_gradOut_In     = t_const_gradOut_In;
        lowBoundA_gradOut_Out    = t_const_gradOut_Out;
        uppBoundA_gradOut_In     = NaN;
        uppBoundA_gradOut_Out    = NaN;
        lowBoundB_gradOut_In     = NaN;
        lowBoundB_gradOut_Out    = NaN;
        uppBoundB_gradOut_In     = NaN;
        uppBoundB_gradOut_Out    = NaN;
        
        %prepare hessian gradient outputs
        lowBoundA_Hessian = [lowBoundA_gradIn_In, lowBoundA_gradIn_Out, lowBoundA_gradOut_In, lowBoundA_gradOut_Out];
        uppBoundA_Hessian = [uppBoundA_gradIn_In, uppBoundA_gradIn_Out, uppBoundA_gradOut_In, uppBoundA_gradOut_Out];
        lowBoundB_Hessian = [lowBoundB_gradIn_In, lowBoundB_gradIn_Out, lowBoundB_gradOut_In, lowBoundB_gradOut_Out];
        uppBoundB_Hessian = [uppBoundB_gradIn_In, uppBoundB_gradIn_Out, uppBoundB_gradOut_In, uppBoundB_gradOut_Out];
        
        return
    end
    
    %this method calculates the ranges of convex and concave shapes
    switch sign(GnOffset)
        case +1
            %================================================================
            % POSITIVE GnOffset
            %================================================================
            switch sign(I_rest)
                case +1
                    %------------------------------
                    %-------   I_rest > 0   -------
                    %------------------------------
                    %I_rest is positive: The fastest accumulation is given by
                    %adding a concave shape to the linear phase. The slowest
                    %accumulation depends on whether the integral of the linear
                    %phase minus a triangular shape reaching the zero gradient
                    %line is larger than I_rest. If it is so, the time range is
                    %divided into two sub-ranges, including G_end being in the
                    %negative range. otherwise the range is continous.
                    
                    %get fastest accumulation; check integral at +Gmax constraint
                    t_raise         = (Gmax-GnOffset)/Smax;
                    I_switch        = t_raise*(Gmax+GnOffset);
                    
                    %get lower bound
                    %---------------
                    if      I_rest <= I_switch
                        %   use triangular shape positive
                        %   -(2*(GnOffset + (GnOffset^2 + I_rest*Smax)^(1/2)))/Smax
                        %   -(2*(GnOffset - (GnOffset^2 + I_rest*Smax)^(1/2)))/Smax
                        
                        t_rest_a   	= -(GnOffset*2.0+sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        t_rest_b   	= -(GnOffset*2.0-sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        
                        %Calculate gradients
                        if     AdjustedSign == +1 && ID == 1
                            
                            t_rest_a_gradIn  = -2.0/Smax-(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradIn  = -2.0/Smax+(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_a_gradOut = -(GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradOut = (GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        elseif AdjustedSign == +1 && ID == 2
                            
                            t_rest_a_gradIn  = (GnIn*1.0/sqrt(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0)))/Smax;
                            t_rest_b_gradIn  = -(GnIn*1.0/sqrt(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0)))/Smax;
                            t_rest_a_gradOut = -2.0/Smax-(GnOut*1.0/sqrt(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))*3.0)/Smax;
                            t_rest_b_gradOut = -2.0/Smax+(GnOut*1.0/sqrt(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))*3.0)/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2*3.0)*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2*3.0)*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*6.0-GnIn^2*3.0)*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*6.0-GnIn^2*3.0)*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
   
                        elseif AdjustedSign == -1 && ID == 1
                            
                            t_rest_a_gradIn  = -2.0/Smax-(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))*3.0)/Smax;
                            t_rest_b_gradIn  = -2.0/Smax+(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))*3.0)/Smax;
                            t_rest_a_gradOut = (GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradOut = -(GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*6.0-GnOut^2*3.0)*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*6.0-GnOut^2*3.0)*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2*3.0)*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2*3.0)*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            
                        else  %AdjustedSign == -1 && ID == 2
                            
                            t_rest_a_gradIn  = -(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradIn  = (GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_a_gradOut = -2.0/Smax-(GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradOut = -2.0/Smax+(GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            
                        end
                        
                        %Set correct gradients and bounds
                        if t_rest_a > t_rest_b
                            lowBoundA          = t_const+t_rest_a;
                            lowBoundA_gradIn   = t_const_gradIn+t_rest_a_gradIn;
                            lowBoundA_gradOut  = t_const_gradOut+t_rest_a_gradOut;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            lowBoundA_gradIn_In      = t_const_gradIn_In+t_rest_a_gradIn_In;
                            lowBoundA_gradIn_Out     = t_const_gradIn_Out+t_rest_a_gradIn_Out;
                            lowBoundA_gradOut_In     = t_const_gradOut_In+t_rest_a_gradOut_In;
                            lowBoundA_gradOut_Out    = t_const_gradOut_Out+t_rest_a_gradOut_Out;
                            
                        else
                            
                            lowBoundA          = t_const+t_rest_b;
                            lowBoundA_gradIn   = t_const_gradIn+t_rest_b_gradIn;
                            lowBoundA_gradOut  = t_const_gradOut+t_rest_b_gradOut;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            lowBoundA_gradIn_In      = t_const_gradIn_In+t_rest_b_gradIn_In;
                            lowBoundA_gradIn_Out     = t_const_gradIn_Out+t_rest_b_gradIn_Out;
                            lowBoundA_gradOut_In     = t_const_gradOut_In+t_rest_b_gradOut_In;
                            lowBoundA_gradOut_Out    = t_const_gradOut_Out+t_rest_b_gradOut_Out;
                            
                        end
                        
                        
                    else
                        %use trapezoidal shape positive
                        %   (I_rest + t_raise*(Gmax - GnOffset))/Gmax
                        
                        %TODO
                        t_rest     	= (I_rest + t_raise*(Gmax - GnOffset))/Gmax;
                        
                        %Calculate gradients
                        if     AdjustedSign == +1 && ID == 1
                            
                            t_rest_gradIn  = -(GnIn/Smax + (2*Gmax - 2*GnIn)/Smax)/Gmax;
                            t_rest_gradOut = GnOut/(Gmax*Smax);
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = 1/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = 1/(Gmax*Smax);

                            
                        elseif AdjustedSign == +1 && ID == 2
                            
                            t_rest_gradIn  = -GnIn/(Gmax*Smax);
                            t_rest_gradOut = (GnOut/Smax - (2*Gmax - 2*GnOut)/Smax)/Gmax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = -1/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = 3/(Gmax*Smax);
                            
                        elseif AdjustedSign == -1 && ID == 1
                            
                            t_rest_gradIn  = (GnIn/Smax - (2*Gmax - 2*GnIn)/Smax)/Gmax;
                            t_rest_gradOut = -GnOut/(Gmax*Smax);
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = 3/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = -1/(Gmax*Smax);

                        else  %AdjustedSign == -1 && ID == 2
                            
                            t_rest_gradIn  = GnIn/(Gmax*Smax);
                            t_rest_gradOut = -(GnOut/Smax + (2*Gmax - 2*GnOut)/Smax)/Gmax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = 1/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = 1/(Gmax*Smax);

                        end
                        
                        %Set correct gradients and bounds
                        lowBoundA          = t_const+t_rest;
                        lowBoundA_gradIn   = t_const_gradIn+t_rest_gradIn;
                        lowBoundA_gradOut  = t_const_gradOut+t_rest_gradOut;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        lowBoundA_gradIn_In      = t_const_gradIn_In+t_rest_gradIn_In;
                        lowBoundA_gradIn_Out     = t_const_gradIn_Out+t_rest_gradIn_Out;
                        lowBoundA_gradOut_In     = t_const_gradOut_In+t_rest_gradOut_In;
                        lowBoundA_gradOut_Out    = t_const_gradOut_Out+t_rest_gradOut_Out;
                        
                    end
                    
                    %get upper bound
                    %---------------
                    
                    %decide how to reach slow accumulation.
                    t_zero  = GnOffset/Smax;
                    I_zero  = GnOffset*t_zero;
                    
                    %if the integral accumulated within reaching G = 0 is
                    %smaller I_rest, the upper t bound is Inf. Otherwise we
                    %need to find the last time point in the positive part and
                    %the first time point in the negative allowing the correct
                    %accumulation
                    if I_rest >= I_zero
                        %only one infitine bound
                        %lowBoundA    = F1_uppBound;      %Unsave
                        uppBoundA 	= Inf;
                        %no subdivided range
                        lowBoundB   = NaN;
                        uppBoundB   = NaN;
                        
                        %set IN gradients
                        uppBoundA_gradIn   = NaN;
                        lowBoundB_gradIn   = NaN;
                        uppBoundB_gradIn   = NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradIn_In      = NaN;
                        uppBoundA_gradIn_Out     = NaN;
                        lowBoundB_gradIn_In      = NaN;
                        lowBoundB_gradIn_Out     = NaN;
                        uppBoundB_gradIn_In      = NaN;
                        uppBoundB_gradIn_Out     = NaN;
                        
                        %set OUT gradients
                        uppBoundA_gradOut  = NaN;
                        lowBoundB_gradOut  = NaN;
                        uppBoundB_gradOut  = NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradOut_In     = NaN;
                        uppBoundA_gradOut_Out    = NaN;
                        lowBoundB_gradOut_In     = NaN;
                        lowBoundB_gradOut_Out    = NaN;
                        uppBoundB_gradOut_In     = NaN;
                        uppBoundB_gradOut_Out    = NaN;
                        
                    else
                        %create subdivided bounds
                        t_zero_a = (GnOffset*2.0-sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        t_zero_b = (GnOffset*2.0+sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        
                        %Calculate gradients
                        if     AdjustedSign == +1 && ID == 1
                            
                            t_zero_a_gradIn  = 2.0/Smax-(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))*3.0)/Smax;
                            t_zero_b_gradIn  = 2.0/Smax+(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))*3.0)/Smax;
                            t_zero_a_gradOut = (GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_b_gradOut = -(GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_zero_a_gradIn_In       = ((Smax*dknGamma*6.0+GnOut^2*3.0)*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_a_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_zero_b_gradIn_In       = ((Smax*dknGamma*6.0+GnOut^2*3.0)*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_b_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_zero_a_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_zero_a_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2*3.0)*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_b_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_zero_b_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2*3.0)*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        elseif AdjustedSign == +1 && ID == 2
                            
                            t_zero_a_gradIn  = -(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_b_gradIn  = (GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_a_gradOut = 2.0/Smax-(GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_b_gradOut = 2.0/Smax+(GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_zero_a_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_a_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_b_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_b_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_a_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_a_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_b_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_b_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;

                        elseif AdjustedSign == -1 && ID == 1
                            
                            t_zero_a_gradIn  = 2.0/Smax-(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_b_gradIn  = 2.0/Smax+(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_a_gradOut = -(GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_b_gradOut = (GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_zero_a_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_a_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_b_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_b_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_a_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_a_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_b_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_b_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;

                        else  %AdjustedSign == -1 && ID == 2
                            
                            t_zero_a_gradIn  = (GnIn*1.0/sqrt(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0)))/Smax;
                            t_zero_b_gradIn  = -(GnIn*1.0/sqrt(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0)))/Smax;
                            t_zero_a_gradOut = 2.0/Smax-(GnOut*1.0/sqrt(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))*3.0)/Smax;
                            t_zero_b_gradOut = 2.0/Smax+(GnOut*1.0/sqrt(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))*3.0)/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_zero_a_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2*3.0)*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_a_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_zero_b_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2*3.0)*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_b_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_zero_a_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_zero_a_gradOut_Out     = ((Smax*dknGamma*6.0+GnIn^2*3.0)*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_b_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_zero_b_gradOut_Out     = ((Smax*dknGamma*6.0+GnIn^2*3.0)*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;

                        end
                        
                        %Set correct gradients and bounds
                        uppBoundA 	= t_const+t_zero_a;
                        lowBoundB 	= t_const+t_zero_b;
                        uppBoundB  	= Inf;
                        
                        %set IN gradients
                        uppBoundA_gradIn 	= t_const_gradIn+t_zero_a_gradIn;
                        lowBoundB_gradIn 	= t_const_gradIn+t_zero_b_gradIn;
                        uppBoundB_gradIn 	= NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradIn_In      = t_const_gradIn_In+t_zero_a_gradIn_In;
                        uppBoundA_gradIn_Out     = t_const_gradIn_Out+t_zero_a_gradIn_Out;
                        lowBoundB_gradIn_In      = t_const_gradIn_In+t_zero_b_gradIn_In;
                        lowBoundB_gradIn_Out     = t_const_gradIn_Out+t_zero_b_gradIn_Out;
                        uppBoundB_gradIn_In      = NaN;
                        uppBoundB_gradIn_Out     = NaN;
                        
                        %set OUT gradients
                        uppBoundA_gradOut 	= t_const_gradOut+t_zero_a_gradOut;
                        lowBoundB_gradOut 	= t_const_gradOut+t_zero_b_gradOut;
                        uppBoundB_gradOut 	= NaN;
                       
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradOut_In     = t_const_gradOut_In+t_zero_a_gradOut_In;
                        uppBoundA_gradOut_Out    = t_const_gradOut_Out+t_zero_a_gradOut_Out;
                        lowBoundB_gradOut_In     = t_const_gradOut_In+t_zero_b_gradOut_In;
                        lowBoundB_gradOut_Out    = t_const_gradOut_Out+t_zero_b_gradOut_Out;
                        uppBoundB_gradOut_In     = NaN;
                        uppBoundB_gradOut_Out    = NaN;
                        
                    end
                    
                otherwise
                    %------------------------------
                    %-------   I_rest < 0   -------
                    %------------------------------

                    %get time point and integral, at which -Gmax constraint is
                    %reached
                    t_fall  	= (Gmax+GnOffset)/Smax;
                    I_switch 	= -t_fall*(Gmax-GnOffset);
                    
                    %get lower bounds
                    %------------------
                    if      I_rest >= I_switch
                        %use triangular shape negative
                        %   (2*(GnOffset + (GnOffset^2 - I_rest*Smax)^(1/2)))/Smax
                        %   (2*(GnOffset - (GnOffset^2 - I_rest*Smax)^(1/2)))/Smax
                        
                        t_rest_a   	= (GnOffset*2.0+sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        t_rest_b   	= (GnOffset*2.0-sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        
                        %Calculate gradients
                        if     AdjustedSign == +1 && ID == 1
                            
                            t_rest_a_gradIn  = 2.0/Smax+(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))*3.0)/Smax;
                            t_rest_b_gradIn  = 2.0/Smax-(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))*3.0)/Smax;
                            t_rest_a_gradOut = -(GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradOut = (GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*6.0+GnOut^2*3.0)*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*6.0+GnOut^2*3.0)*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2*3.0)*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2*3.0)*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;

                        elseif AdjustedSign == +1 && ID == 2
                            
                            t_rest_a_gradIn  = (GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradIn  = -(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_a_gradOut = 2.0/Smax+(GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradOut = 2.0/Smax-(GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        elseif AdjustedSign == -1 && ID == 1
                            
                            t_rest_a_gradIn  = 2.0/Smax+(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradIn  = 2.0/Smax-(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_a_gradOut = (GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradOut = -(GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        else  %AdjustedSign == -1 && ID == 2
                            
                            t_rest_a_gradIn  = -(GnIn*1.0/sqrt(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0)))/Smax;
                            t_rest_b_gradIn  = (GnIn*1.0/sqrt(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0)))/Smax;
                            t_rest_a_gradOut = 2.0/Smax+(GnOut*1.0/sqrt(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))*3.0)/Smax;
                            t_rest_b_gradOut = 2.0/Smax-(GnOut*1.0/sqrt(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))*3.0)/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2*3.0)*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2*3.0)*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*6.0+GnIn^2*3.0)*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*6.0+GnIn^2*3.0)*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        end
                        
                        %lowBoundA  	= max([t_rest_a, t_rest_b]) + t_const;

                        %Set correct gradients and bounds
                        if t_rest_a > t_rest_b
                            
                            lowBoundA          = t_const+t_rest_a;
                            lowBoundA_gradIn   = t_const_gradIn+t_rest_a_gradIn;
                            lowBoundA_gradOut  = t_const_gradOut+t_rest_a_gradOut;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            lowBoundA_gradIn_In      = t_const_gradIn_In+t_rest_a_gradIn_In;
                            lowBoundA_gradIn_Out     = t_const_gradIn_Out+t_rest_a_gradIn_Out;
                            lowBoundA_gradOut_In     = t_const_gradOut_In+t_rest_a_gradOut_In;
                            lowBoundA_gradOut_Out    = t_const_gradOut_Out+t_rest_a_gradOut_Out;

                        else
                            
                            lowBoundA          = t_const+t_rest_b;
                            lowBoundA_gradIn   = t_const_gradIn+t_rest_b_gradIn;
                            lowBoundA_gradOut  = t_const_gradOut+t_rest_b_gradOut;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            lowBoundA_gradIn_In      = t_const_gradIn_In+t_rest_b_gradIn_In;
                            lowBoundA_gradIn_Out     = t_const_gradIn_Out+t_rest_b_gradIn_Out;
                            lowBoundA_gradOut_In     = t_const_gradOut_In+t_rest_b_gradOut_In;
                            lowBoundA_gradOut_Out    = t_const_gradOut_Out+t_rest_b_gradOut_Out;
                            
                        end
                        
                        uppBoundA  	= Inf;
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                        %set IN gradients
                        uppBoundA_gradIn 	= NaN;
                        lowBoundB_gradIn 	= NaN;
                        uppBoundB_gradIn 	= NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradIn_In      = NaN;
                        uppBoundA_gradIn_Out     = NaN;
                        lowBoundB_gradIn_In      = NaN;
                        lowBoundB_gradIn_Out     = NaN;
                        uppBoundB_gradIn_In      = NaN;
                        uppBoundB_gradIn_Out     = NaN;
                        
                        %set IN gradients
                        uppBoundA_gradOut 	= NaN;
                        lowBoundB_gradOut 	= NaN;
                        uppBoundB_gradOut 	= NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradOut_In     = NaN;
                        uppBoundA_gradOut_Out    = NaN;
                        lowBoundB_gradOut_In     = NaN;
                        lowBoundB_gradOut_Out    = NaN;
                        uppBoundB_gradOut_In     = NaN;
                        uppBoundB_gradOut_Out    = NaN;
                        
                    else

                        %use trapezoidal shape positive
                        %   -(I_rest - t_fall*(Gmax + GnOffset))/Gmax
                        
                        t_rest      = -(I_rest - t_fall*(Gmax + GnOffset))/Gmax;
                        
                        %Calculate gradients
                        if     AdjustedSign == +1 && ID == 1
                            
                            t_rest_gradIn  = (GnIn/Smax + (2*Gmax + 2*GnIn)/Smax)/Gmax;
                            t_rest_gradOut = -GnOut/(Gmax*Smax);
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = 3/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = -1/(Gmax*Smax);

                        elseif AdjustedSign == +1 && ID == 2
                            
                            t_rest_gradIn  = GnIn/(Gmax*Smax);
                            t_rest_gradOut = -(GnOut/Smax - (2*Gmax + 2*GnOut)/Smax)/Gmax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = 1/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = 1/(Gmax*Smax);

                        elseif AdjustedSign == -1 && ID == 1
                            
                            t_rest_gradIn  = -(GnIn/Smax - (2*Gmax + 2*GnIn)/Smax)/Gmax;
                            t_rest_gradOut = GnOut/(Gmax*Smax);
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = 1/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = 1/(Gmax*Smax);

                        else  %AdjustedSign == -1 && ID == 2
                            
                            t_rest_gradIn  = -GnIn/(Gmax*Smax);
                            t_rest_gradOut = (GnOut/Smax + (2*Gmax + 2*GnOut)/Smax)/Gmax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = -1/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = 3/(Gmax*Smax);

                        end
                        
                        lowBoundA	= t_rest + t_const;
                        uppBoundA 	= Inf;
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                        %Set correct gradients
                        lowBoundA_gradIn   = t_const_gradIn+t_rest_gradIn;
                        lowBoundA_gradOut  = t_const_gradOut+t_rest_gradOut;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        lowBoundA_gradIn_In      = t_const_gradIn_In+t_rest_gradIn_In;
                        lowBoundA_gradIn_Out     = t_const_gradIn_Out+t_rest_gradIn_Out;
                        lowBoundA_gradOut_In     = t_const_gradOut_In+t_rest_gradOut_In;
                        lowBoundA_gradOut_Out    = t_const_gradOut_Out+t_rest_gradOut_Out;
                        
                        uppBoundA_gradIn   = NaN;
                        uppBoundA_gradOut  = NaN;
                        lowBoundB_gradIn   = NaN;
                        lowBoundB_gradOut  = NaN;
                        uppBoundB_gradIn   = NaN;
                        uppBoundB_gradOut  = NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradIn_In      = NaN;
                        uppBoundA_gradIn_Out     = NaN;
                        uppBoundA_gradOut_In     = NaN;
                        uppBoundA_gradOut_Out    = NaN;
                        lowBoundB_gradIn_In      = NaN;
                        lowBoundB_gradIn_Out     = NaN;
                        lowBoundB_gradOut_In     = NaN;
                        lowBoundB_gradOut_Out    = NaN;
                        uppBoundB_gradIn_In      = NaN;
                        uppBoundB_gradIn_Out     = NaN;
                        uppBoundB_gradOut_In     = NaN;
                        uppBoundB_gradOut_Out    = NaN;

                    end
                    
            end
            
        otherwise
            %================================================================
            % NEGATIVE GnOffset
            %================================================================
            switch sign(I_rest)
                case {+1,0}
                    %------------------------------
                    %-------   I_rest > 0   -------
                    %------------------------------

                    %get time point and integral, at which +Gmax constraint is
                    %reached
                    t_raise   	= (Gmax-GnOffset)/Smax;
                    I_switch 	= t_raise*(Gmax+GnOffset);
                    
                    %check minus bounds
                    %------------------
                    if      I_rest <= I_switch
                        %use triangular shape negative
                        %   -(2*(GnOffset + (GnOffset^2 + I_rest*Smax)^(1/2)))/Smax
                        %   -(2*(GnOffset - (GnOffset^2 + I_rest*Smax)^(1/2)))/Smax
                        
                        t_rest_a   	= -(GnOffset*2.0+sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        t_rest_b   	= -(GnOffset*2.0-sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        
                        %Calculate gradients
                        if     AdjustedSign == +1 && ID == 1
                            
                            t_rest_a_gradIn  = -2.0/Smax-(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradIn  = -2.0/Smax+(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_a_gradOut = -(GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradOut = (GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        elseif AdjustedSign == +1 && ID == 2
                            
                            t_rest_a_gradIn  = (GnIn*1.0/sqrt(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0)))/Smax;
                            t_rest_b_gradIn  = -(GnIn*1.0/sqrt(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0)))/Smax;
                            t_rest_a_gradOut = -2.0/Smax-(GnOut*1.0/sqrt(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))*3.0)/Smax;
                            t_rest_b_gradOut = -2.0/Smax+(GnOut*1.0/sqrt(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))*3.0)/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2*3.0)*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2*3.0)*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*6.0-GnIn^2*3.0)*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*6.0-GnIn^2*3.0)*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        elseif AdjustedSign == -1 && ID == 1
                            
                            t_rest_a_gradIn  = -2.0/Smax-(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))*3.0)/Smax;
                            t_rest_b_gradIn  = -2.0/Smax+(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))*3.0)/Smax;
                            t_rest_a_gradOut = (GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradOut = -(GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*6.0-GnOut^2*3.0)*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*6.0-GnOut^2*3.0)*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2*3.0)*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2*3.0)*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;

                        else  %AdjustedSign == -1 && ID == 2
                            
                            t_rest_a_gradIn  = -(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradIn  = (GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_a_gradOut = -2.0/Smax-(GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradOut = -2.0/Smax+(GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        end
                        
                        %Set correct gradients and bounds
                        if t_rest_a > t_rest_b
                            
                            lowBoundA          = t_const+t_rest_a;
                            lowBoundA_gradIn   = t_const_gradIn+t_rest_a_gradIn;
                            lowBoundA_gradOut  = t_const_gradOut+t_rest_a_gradOut;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            lowBoundA_gradIn_In      = t_const_gradIn_In+t_rest_a_gradIn_In;
                            lowBoundA_gradIn_Out     = t_const_gradIn_Out+t_rest_a_gradIn_Out;
                            lowBoundA_gradOut_In     = t_const_gradOut_In+t_rest_a_gradOut_In;
                            lowBoundA_gradOut_Out    = t_const_gradOut_Out+t_rest_a_gradOut_Out;
                            
                        else
                            
                            lowBoundA          = t_const+t_rest_b;
                            lowBoundA_gradIn   = t_const_gradIn+t_rest_b_gradIn;
                            lowBoundA_gradOut  = t_const_gradOut+t_rest_b_gradOut;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            lowBoundA_gradIn_In      = t_const_gradIn_In+t_rest_b_gradIn_In;
                            lowBoundA_gradIn_Out     = t_const_gradIn_Out+t_rest_b_gradIn_Out;
                            lowBoundA_gradOut_In     = t_const_gradOut_In+t_rest_b_gradOut_In;
                            lowBoundA_gradOut_Out    = t_const_gradOut_Out+t_rest_b_gradOut_Out;
                            
                        end
                        
                        uppBoundA  	= Inf;
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                        %set IN gradients
                        uppBoundA_gradIn 	= NaN;
                        lowBoundB_gradIn	= NaN;
                        uppBoundB_gradIn	= NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradIn_In       = NaN;
                        uppBoundA_gradIn_Out      = NaN;
                        lowBoundB_gradIn_In      = NaN;
                        lowBoundB_gradIn_Out     = NaN;
                        uppBoundB_gradIn_In      = NaN;
                        uppBoundB_gradIn_Out     = NaN;

                        %set OUT gradients
                        uppBoundA_gradOut 	= NaN;
                        lowBoundB_gradOut	= NaN;
                        uppBoundB_gradOut	= NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradOut_In     = NaN;
                        uppBoundA_gradOut_Out    = NaN;
                        lowBoundB_gradOut_In     = NaN;
                        lowBoundB_gradOut_Out    = NaN;
                        uppBoundB_gradOut_In     = NaN;
                        uppBoundB_gradOut_Out    = NaN;
                        
                    else
                        %use trapezoidal shape positive
                        %   (I_rest + t_raise*(Gmax - GnOffset))/Gmax
                        
                        t_rest    	= (I_rest + t_raise*(Gmax - GnOffset))/Gmax;
                        
                        %Calculate gradients
                        if     AdjustedSign == +1 && ID == 1
                            
                            t_rest_gradIn  = -(GnIn/Smax + (2*Gmax - 2*GnIn)/Smax)/Gmax;
                            t_rest_gradOut = GnOut/(Gmax*Smax);
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = 1/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = 1/(Gmax*Smax);

                        elseif AdjustedSign == +1 && ID == 2
                            
                            t_rest_gradIn  = -GnIn/(Gmax*Smax);
                            t_rest_gradOut = (GnOut/Smax - (2*Gmax - 2*GnOut)/Smax)/Gmax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = -1/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = 3/(Gmax*Smax);

                        elseif AdjustedSign == -1 && ID == 1
                            
                            t_rest_gradIn  = (GnIn/Smax - (2*Gmax - 2*GnIn)/Smax)/Gmax;
                            t_rest_gradOut = -GnOut/(Gmax*Smax);
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = 3/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = -1/(Gmax*Smax);

                        else  %AdjustedSign == -1 && ID == 2
                            
                            t_rest_gradIn  = GnIn/(Gmax*Smax);
                            t_rest_gradOut = -(GnOut/Smax + (2*Gmax - 2*GnOut)/Smax)/Gmax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = 1/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = 1/(Gmax*Smax);

                        end
                        
                        lowBoundA	= t_rest + t_const;
                        uppBoundA  	= Inf;
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                        %Set correct gradients
                        lowBoundA_gradIn   = t_const_gradIn+t_rest_gradIn;
                        lowBoundA_gradOut  = t_const_gradOut+t_rest_gradOut;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        lowBoundA_gradIn_In      = t_const_gradIn_In+t_rest_gradIn_In;
                        lowBoundA_gradIn_Out     = t_const_gradIn_Out+t_rest_gradIn_Out;
                        lowBoundA_gradOut_In     = t_const_gradOut_In+t_rest_gradOut_In;
                        lowBoundA_gradOut_Out    = t_const_gradOut_Out+t_rest_gradOut_Out;
                        
                        uppBoundA_gradIn   = NaN;
                        uppBoundA_gradOut  = NaN;
                        lowBoundB_gradIn   = NaN;
                        lowBoundB_gradOut  = NaN;
                        uppBoundB_gradIn   = NaN;
                        uppBoundB_gradOut  = NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradIn_In      = NaN;
                        uppBoundA_gradIn_Out     = NaN;
                        uppBoundA_gradOut_In     = NaN;
                        uppBoundA_gradOut_Out    = NaN;
                        lowBoundB_gradIn_In      = NaN;
                        lowBoundB_gradIn_Out     = NaN;
                        lowBoundB_gradOut_In     = NaN;
                        lowBoundB_gradOut_Out    = NaN;
                        uppBoundB_gradIn_In      = NaN;
                        uppBoundB_gradIn_Out     = NaN;
                        uppBoundB_gradOut_In     = NaN;
                        uppBoundB_gradOut_Out    = NaN;
                        
                    end
                    
                otherwise
                    %------------------------------
                    %-------   I_rest < 0   -------
                    %------------------------------
                    %I_rest is negative: The fastest accumulation is given by
                    %subtracting a convex shape to the linear phase. The slowest
                    %accumulation depends on whether the integral of the linear
                    %phase plus a triangular shape reaching the zero gradient
                    %line is smaller than I_rest. If it is so, the time range is
                    %divided into two sub-ranges, including G_end being in the
                    %positive range. otherwise the range is continous.
                    
                    %get fastest accumulation; check integral at -Gmax constraint
                    t_fall   	= (Gmax+GnOffset)/Smax;
                    I_switch 	= -t_fall*(Gmax-GnOffset);
                    
                    %get lower bounds
                    %-----------------
                    if      I_rest >= I_switch
                        %use triangular shape positive
                        %   (2*(GnOffset + (GnOffset^2 - I_rest*Smax)^(1/2)))/Smax
                        %   (2*(GnOffset - (GnOffset^2 - I_rest*Smax)^(1/2)))/Smax
                        
                        t_rest_a   	= (GnOffset*2.0+sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        t_rest_b   	= (GnOffset*2.0-sqrt(GnOffset^2-I_rest*Smax)*2.0)/Smax;
                        
                        %Calculate gradients
                        if     AdjustedSign == +1 && ID == 1
                            
                            t_rest_a_gradIn  = 2.0/Smax+(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))*3.0)/Smax;
                            t_rest_b_gradIn  = 2.0/Smax-(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))*3.0)/Smax;
                            t_rest_a_gradOut = -(GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradOut = (GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*6.0+GnOut^2*3.0)*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*6.0+GnOut^2*3.0)*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2*3.0)*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2*3.0)*1.0/(-Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;

                        elseif AdjustedSign == +1 && ID == 2
                            
                            t_rest_a_gradIn  = (GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradIn  = -(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_a_gradOut = 2.0/Smax+(GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradOut = 2.0/Smax-(GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        elseif AdjustedSign == -1 && ID == 1
                            
                            t_rest_a_gradIn  = 2.0/Smax+(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradIn  = 2.0/Smax-(GnIn*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_a_gradOut = (GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_rest_b_gradOut = -(GnOut*1.0/sqrt(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*2.0-GnIn^2)*1.0/(-Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        else  %AdjustedSign == -1 && ID == 2
                            
                            t_rest_a_gradIn  = -(GnIn*1.0/sqrt(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0)))/Smax;
                            t_rest_b_gradIn  = (GnIn*1.0/sqrt(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0)))/Smax;
                            t_rest_a_gradOut = 2.0/Smax+(GnOut*1.0/sqrt(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))*3.0)/Smax;
                            t_rest_b_gradOut = 2.0/Smax-(GnOut*1.0/sqrt(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))*3.0)/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_a_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2*3.0)*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_rest_a_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_b_gradIn_In       = ((Smax*dknGamma*2.0-GnOut^2*3.0)*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradIn_Out      = (GnIn*GnOut*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_a_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_rest_a_gradOut_Out     = ((Smax*dknGamma*6.0+GnIn^2*3.0)*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_rest_b_gradOut_In      = (GnIn*GnOut*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_rest_b_gradOut_Out     = ((Smax*dknGamma*6.0+GnIn^2*3.0)*1.0/(-Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        end
                        
                        %Set correct gradients and bounds
                        if t_rest_a > t_rest_b
                            
                            lowBoundA          = t_const+t_rest_a;
                            lowBoundA_gradIn   = t_const_gradIn+t_rest_a_gradIn;
                            lowBoundA_gradOut  = t_const_gradOut+t_rest_a_gradOut;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            lowBoundA_gradIn_In      = t_const_gradIn_In+t_rest_a_gradIn_In;
                            lowBoundA_gradIn_Out     = t_const_gradIn_Out+t_rest_a_gradIn_Out;
                            lowBoundA_gradOut_In     = t_const_gradOut_In+t_rest_a_gradOut_In;
                            lowBoundA_gradOut_Out    = t_const_gradOut_Out+t_rest_a_gradOut_Out;

                        else
                            
                            lowBoundA          = t_const+t_rest_b;
                            lowBoundA_gradIn   = t_const_gradIn+t_rest_b_gradIn;
                            lowBoundA_gradOut  = t_const_gradOut+t_rest_b_gradOut;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            lowBoundA_gradIn_In      = t_const_gradIn_In+t_rest_b_gradIn_In;
                            lowBoundA_gradIn_Out     = t_const_gradIn_Out+t_rest_b_gradIn_Out;
                            lowBoundA_gradOut_In     = t_const_gradOut_In+t_rest_b_gradOut_In;
                            lowBoundA_gradOut_Out    = t_const_gradOut_Out+t_rest_b_gradOut_Out;
                            
                        end
                        
                    else
                        %use trapezoidal shape negative
                        %   (I_rest + t_fall*(Gmax - GnOffset))/Gmax
                        
                        t_rest    	= -(I_rest - t_fall*(Gmax + GnOffset))/Gmax;
                        
                        %Calculate gradients
                        if     AdjustedSign == +1 && ID == 1
                            
                            t_rest_gradIn  = (GnIn/Smax + (2*Gmax + 2*GnIn)/Smax)/Gmax;
                            t_rest_gradOut = -GnOut/(Gmax*Smax);
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = 3/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = -1/(Gmax*Smax);

                        elseif AdjustedSign == +1 && ID == 2
                            
                            t_rest_gradIn  = GnIn/(Gmax*Smax);
                            t_rest_gradOut = -(GnOut/Smax - (2*Gmax + 2*GnOut)/Smax)/Gmax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = 1/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = 1/(Gmax*Smax);

                        elseif AdjustedSign == -1 && ID == 1
                            
                            t_rest_gradIn  = -(GnIn/Smax - (2*Gmax + 2*GnIn)/Smax)/Gmax;
                            t_rest_gradOut = GnOut/(Gmax*Smax);
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = 1/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = 1/(Gmax*Smax);

                        else  %AdjustedSign == -1 && ID == 2
                            
                            t_rest_gradIn  = -GnIn/(Gmax*Smax);
                            t_rest_gradOut = (GnOut/Smax + (2*Gmax + 2*GnOut)/Smax)/Gmax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_rest_gradIn_In         = -1/(Gmax*Smax);
                            t_rest_gradIn_Out        = 0;
                            t_rest_gradOut_In        = 0;
                            t_rest_gradOut_Out       = 3/(Gmax*Smax);

                        end

                        lowBoundA 	= t_const+t_rest;
                        
                        %Set correct gradients
                        lowBoundA_gradIn   = t_const_gradIn+t_rest_gradIn;
                        lowBoundA_gradOut  = t_const_gradOut+t_rest_gradOut;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        lowBoundA_gradIn_In      = t_const_gradIn_In+t_rest_gradIn_In;
                        lowBoundA_gradIn_Out     = t_const_gradIn_Out+t_rest_gradIn_Out;
                        lowBoundA_gradOut_In     = t_const_gradOut_In+t_rest_gradOut_In;
                        lowBoundA_gradOut_Out    = t_const_gradOut_Out+t_rest_gradOut_Out;
                        
                    end
                    
                    
                    %decide how to reach slow accumulation.
                    t_zero  = -GnOffset/Smax;
                    I_zero  = GnOffset*t_zero;
                    
                    %if the integral accumulated within reaching G = 0 is
                    %smaller I_rest, the upper t bound is Inf. Otherwise we
                    %need to find the last time point in the positive part and
                    %the first time point in the negative allowing the correct
                    %accumulation
                    if I_rest <= I_zero
                        %only one infitine bound
                        uppBoundA 	= Inf;
                        %no subdivided ranges
                        lowBoundB  	= NaN;
                        uppBoundB  	= NaN;
                        
                        %set IN gradients
                        uppBoundA_gradIn 	= NaN;
                        lowBoundB_gradIn 	= NaN;
                        uppBoundB_gradIn 	= NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradIn_In      = NaN;
                        uppBoundA_gradIn_Out     = NaN;
                        lowBoundB_gradIn_In      = NaN;
                        lowBoundB_gradIn_Out     = NaN;
                        uppBoundB_gradIn_In      = NaN;
                        uppBoundB_gradIn_Out     = NaN;
                        
                        %set OUT gradients
                        uppBoundA_gradOut 	= NaN;
                        lowBoundB_gradOut 	= NaN;
                        uppBoundB_gradOut 	= NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradOut_In     = NaN;
                        uppBoundA_gradOut_Out    = NaN;
                        lowBoundB_gradOut_In     = NaN;
                        lowBoundB_gradOut_Out    = NaN;
                        uppBoundB_gradOut_In     = NaN;
                        uppBoundB_gradOut_Out    = NaN;
 
                    else
                        %create subdivided bounds
                        t_zero_a = -(GnOffset*2.0+sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        t_zero_b = -(GnOffset*2.0-sqrt(GnOffset^2+I_rest*Smax)*2.0)/Smax;
                        
                        %Calculate gradients
                        if     AdjustedSign == +1 && ID == 1
                            
                            t_zero_a_gradIn  = -2.0/Smax-(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_b_gradIn  = -2.0/Smax+(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_a_gradOut = -(GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_b_gradOut = (GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_zero_a_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_a_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_b_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_b_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_a_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_a_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_b_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_b_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        elseif AdjustedSign == +1 && ID == 2
                            
                            t_zero_a_gradIn  = (GnIn*1.0/sqrt(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0)))/Smax;
                            t_zero_b_gradIn  = -(GnIn*1.0/sqrt(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0)))/Smax;
                            t_zero_a_gradOut = -2.0/Smax-(GnOut*1.0/sqrt(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))*3.0)/Smax;
                            t_zero_b_gradOut = -2.0/Smax+(GnOut*1.0/sqrt(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))*3.0)/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_zero_a_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2*3.0)*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_a_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_zero_b_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2*3.0)*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_b_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_zero_a_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_zero_a_gradOut_Out     = ((Smax*dknGamma*6.0-GnIn^2*3.0)*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_b_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_zero_b_gradOut_Out     = ((Smax*dknGamma*6.0-GnIn^2*3.0)*1.0/(Smax*dknGamma-GnIn^2*(1.0/2.0)+GnOut^2*(3.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        elseif AdjustedSign == -1 && ID == 1
                            
                            t_zero_a_gradIn  = -2.0/Smax-(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))*3.0)/Smax;
                            t_zero_b_gradIn  = -2.0/Smax+(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))*3.0)/Smax;
                            t_zero_a_gradOut = (GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_b_gradOut = -(GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_zero_a_gradIn_In       = ((Smax*dknGamma*6.0-GnOut^2*3.0)*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_a_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_zero_b_gradIn_In       = ((Smax*dknGamma*6.0-GnOut^2*3.0)*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_b_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_zero_a_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-3.0/2.0))/Smax;
                            t_zero_a_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2*3.0)*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_b_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(3.0/2.0))/Smax;
                            t_zero_b_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2*3.0)*1.0/(Smax*dknGamma+GnIn^2*(3.0/2.0)-GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;

                        else  %AdjustedSign == -1 && ID == 2
                            
                            t_zero_a_gradIn  = -(GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_b_gradIn  = (GnIn*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_a_gradOut = -2.0/Smax-(GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            t_zero_b_gradOut = -2.0/Smax+(GnOut*1.0/sqrt(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0)))/Smax;
                            
                            %Set correct second derivatives, used for analytical Hessian
                            t_zero_a_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_a_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_b_gradIn_In       = ((Smax*dknGamma*2.0+GnOut^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_b_gradIn_Out      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_a_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;
                            t_zero_a_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_b_gradOut_In      = (GnIn*GnOut*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(-1.0/2.0))/Smax;
                            t_zero_b_gradOut_Out     = ((Smax*dknGamma*2.0+GnIn^2)*1.0/(Smax*dknGamma+GnIn^2*(1.0/2.0)+GnOut^2*(1.0/2.0))^(3.0/2.0)*(1.0/2.0))/Smax;

                        end

                        %lowBoundA    = F1_uppBound;      %Unsave
                        uppBoundA	= t_zero_a + t_const;
                        lowBoundB 	= t_const+t_zero_b;
                        uppBoundB 	= Inf;
                        
                        %set IN gradients
                        uppBoundA_gradIn 	= t_const_gradIn+t_zero_a_gradIn;
                        lowBoundB_gradIn 	= t_const_gradIn+t_zero_b_gradIn;
                        uppBoundB_gradIn 	= NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradIn_In      = t_const_gradIn_In+t_zero_a_gradIn_In;
                        uppBoundA_gradIn_Out     = t_const_gradIn_Out+t_zero_a_gradIn_Out;
                        lowBoundB_gradIn_In      = t_const_gradIn_In+t_zero_b_gradIn_In;
                        lowBoundB_gradIn_Out     = t_const_gradIn_Out+t_zero_b_gradIn_Out;
                        uppBoundB_gradIn_In      = NaN;
                        uppBoundB_gradIn_Out     = NaN;
                        
                        %set OUT gradients
                        uppBoundA_gradOut 	= t_const_gradOut+t_zero_a_gradOut;
                        lowBoundB_gradOut 	= t_const_gradOut+t_zero_b_gradOut;
                        uppBoundB_gradOut 	= NaN;
                        
                        %Set correct second derivatives, used for analytical Hessian
                        uppBoundA_gradOut_In     = t_const_gradOut_In+t_zero_a_gradOut_In;
                        uppBoundA_gradOut_Out    = t_const_gradOut_Out+t_zero_a_gradOut_Out;
                        lowBoundB_gradOut_In     = t_const_gradOut_In+t_zero_b_gradOut_In;
                        lowBoundB_gradOut_Out    = t_const_gradOut_Out+t_zero_b_gradOut_Out;
                        uppBoundB_gradOut_In     = NaN;
                        uppBoundB_gradOut_Out    = NaN;
                        
                    end
                    
            end
            
    end
    
    %prepare hessian gradient outputs
    lowBoundA_Hessian = [lowBoundA_gradIn_In, lowBoundA_gradIn_Out, lowBoundA_gradOut_In, lowBoundA_gradOut_Out];
 	uppBoundA_Hessian = [uppBoundA_gradIn_In, uppBoundA_gradIn_Out, uppBoundA_gradOut_In, uppBoundA_gradOut_Out];
  	lowBoundB_Hessian = [lowBoundB_gradIn_In, lowBoundB_gradIn_Out, lowBoundB_gradOut_In, lowBoundB_gradOut_Out];
    uppBoundB_Hessian = [uppBoundB_gradIn_In, uppBoundB_gradIn_Out, uppBoundB_gradOut_In, uppBoundB_gradOut_Out];
 
end

function [ lowBounds, uppBounds ] = MakeRangeContinous( lowBounds, uppBounds )

    %this method converts the given range in lowBounds and uppBounds to a
    %continous range by dropping the ranges in the beginning, that are not
    %continous. WARNING: this might lead to solutions, which are
    %suboptimal, where on the other hand, the convergence of the convex
    %optimzation might be influenced positively
    
    if size(lowBounds,1) == 1
        return;
    end

    %compare lower Bounds to upper Bounds
    ConInd = double( find( abs(lowBounds(2:end,1) - uppBounds(1:end-1,1)) > 0.001 , 1, 'last') );
    if ~isempty(ConInd)
        %delete ranges that make the full range incontinous
        lowBounds(1 : ConInd(1),:) = [];
        uppBounds(1 : ConInd(1),:) = [];
    end

end
